Package: ibr
Version: 2.4-2
Date: 2026-06-01
Title: Iterative Bias Reduction
Authors@R: c(person("Pierre-Andre", "Cornillon", role = c("aut", "cre"),
                     email = "pierre-andre.cornillon@univ-rennes2.fr"),
              person("Nicolas", "Hengartner", role = "aut"),
              person("Eric", "Matzner-Lober", role = "aut"))
Depends: R (>= 2.11.1), mgcv
Imports: stats, graphics
Description: Multivariate smoothing using iterative bias reduction with kernel, thin plate splines, Duchon splines or low rank splines. 
License: GPL (>= 2)
NeedsCompilation: yes
Packaged: 2026-06-02 11:50:19 UTC; pac
Author: Pierre-Andre Cornillon [aut, cre],
  Nicolas Hengartner [aut],
  Eric Matzner-Lober [aut]
Maintainer: Pierre-Andre Cornillon <pierre-andre.cornillon@univ-rennes2.fr>
Repository: CRAN
Date/Publication: 2026-06-02 12:00:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-06-02 16:13:51 UTC; unix
Archs: ibr.so.dSYM

Package: ibr
Version: 2.4-2
Date: 2026-06-01
Title: Iterative Bias Reduction
Authors@R: c(person("Pierre-Andre", "Cornillon", role = c("aut", "cre"),
                     email = "pierre-andre.cornillon@univ-rennes2.fr"),
              person("Nicolas", "Hengartner", role = "aut"),
              person("Eric", "Matzner-Lober", role = "aut"))
Depends: R (>= 2.11.1), mgcv
Imports: stats, graphics
Description: Multivariate smoothing using iterative bias reduction with kernel, thin plate splines, Duchon splines or low rank splines. 
License: GPL (>= 2)
NeedsCompilation: yes
Packaged: 2026-06-02 11:50:19 UTC; pac
Author: Pierre-Andre Cornillon [aut, cre],
  Nicolas Hengartner [aut],
  Eric Matzner-Lober [aut]
Maintainer: Pierre-Andre Cornillon <pierre-andre.cornillon@univ-rennes2.fr>
Repository: CRAN
Date/Publication: 2026-06-02 12:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-06-02 13:46:52 UTC; unix
Archs: ibr.so.dSYM

export(AIC.ibr,AICc,AICc.ibr,BIC.ibr,bwchoice,cvobs,departnoyau,dssmoother,dsSx,DuchonQ,DuchonS,
       forward,GCV,GCV.ibr,ibr,ibr.fit,kernelKx,kernelSx,lambdachoice,lambdachoicelr,lrsmoother,
       npregress,plot.forwardibr,plot.ibr,predict.ibr,predict.npregress,print.ibr,print.npregress,print.summary.ibr,print.summary.npregress,
       residuals.ibr,summary.ibr,summary.npregress)

importFrom(stats, BIC,delete.response, is.empty.model, model.frame,model.matrix, model.offset, model.response, na.fail,na.omit, optimize, predict, quantile, residuals, sd, terms, uniroot)
importFrom(graphics, plot,abline,image,text)
importFrom(mgcv,PredictMat,smoothCon)
S3method(plot,ibr)
S3method(predict,ibr)
S3method(print,ibr)
S3method(residuals,ibr)
S3method(summary,ibr)
S3method(print,npregress)
S3method(predict,npregress)
S3method(summary,npregress)
S3method(plot,forwardibr)
S3method(print,summary.ibr)
S3method(print,summary.npregress)
S3method(AIC,ibr)
S3method(BIC,ibr)
S3method(GCV,ibr)
S3method(AICc,ibr)
useDynLib(ibr, .registration = TRUE, .fixes = "ibr_")

print.npregress <- function(x, digits = max(3, getOption("digits") - 3), ...) {
  cat("\nDf:",format(round(x$df,digits-1)),"; bandwidth:", format(round(x$bandwidth,digits)), "\n")
  cat("Chosen by:", x$call$criterion, "\n")
}

predict.npregress <- function(object, newdata, interval= c("none", "confidence", "prediction"), deriv=FALSE, ...) {
  interval <- match.arg(interval)
  if ((interval == "prediction")|(interval == "confidence")) {
    warning("Interval for prediction/confidence is not implemented yet\n")
  }
    x <- object$call$x
    y <- object$call$y
  if (missing(newdata) || is.null(newdata)) {
    if (!deriv) return(object$fitted) else newdata <- x
  } else {
    if (any(is.na(newdata))) stop("NA's in newdata\n")
    if (!is.numeric(newdata)&(is.data.frame(newdata))) {
      newdata <- newdata[,1]
      if (!is.numeric(newdata)) stop("first column of data-frame is not numeric\n")
    }
    if (is.matrix(newdata)) {
      newdata <- as.vector(newdata)
    }
    if (!is.numeric(newdata)) stop("newdata must be a numeric vector (or a data-frame with first column of numeric type)\n")
  }
  kern <- c("g", "e", "q", "u")
  kernelint <- which(object$call$kernel==kern)
  ## autre methode
  if (object$call$degree==0) {
    methode <- "reg"
    prov <- .Call(ibr_npreg,as.double(x),as.double(y),as.double(newdata),as.double(object$bandwidth),as.integer(kernelint))
    deriv <- FALSE
  }
  if (object$call$degree==1) {
    prov <- .Call(ibr_npregpol,as.double(x),as.double(y),as.double(newdata),as.double(object$bandwidth),as.integer(kernelint))
  }
  if (!deriv) {
    Yres <- prov[[1]]
  } else {
    Yres <- list(yhat=prov[[1]],deriv=prov[[2]])  
  }
  if (object$call$degree>1) stop("Not implemented. Please consider using KernSmooth or another library for degree greater or equal to 2\n")
  return(Yres)
}

calcA <- function(X,bx,kernelx="g") {
  K <- kernelKx(X, NULL, bx, kernelx)
  listeS <- .Call(ibr_Amatrix, K)
  listeS[[3]] <- sum(diag(K)/listeS[[2]])
  listeS[[2]] <- 1/sqrt(listeS[[2]])
  names(listeS) <- c("S", "Ddemi", "df")
  return(listeS)
}

iterchoiceS1lrcv <- function(X,y,lambda,rank,bs,listvarx,df,ddlmini,ntest,ntrain,Kfold,type,npermut,seed,Kmin,Kmax,criterion,m,s,fraction) {
  choixssecv2 <- function(k,sel,y,lambdalist,rank,S1,valpr,SSx,tUy,ddlmin,index0) {
    sse <- 0
    for (j in 1:length(sel)) {
      if (attr(sel,"type")=="timeseries") {
        nj <- length(y[-(sel[[j]][1]:n)])
      } else {
        nj <- length(y[-sel[[j]]])
    }
      sommerk <- rev(sumvalpr(k,nj,rev(valpr[[j]]),nj-index0[j]+1,nj-ddlmin[j]+1))
      beta <- S1[[j]]$vectors%*%(sommerk[1:rank]*tUy[[j]][1:rank])
      beta <- t((1-valpr[[j]][1:rank])*t(S1[[j]]$Rm1U)) %*%t(S1[[j]]$vectors) %*% beta
      Yrescv <- as.vector(SSx[[j]]%*%beta)
      sse <- sse+sum((y[sel[[j]]]-Yrescv)^2)
    }
    return(sse)
  }
  choixsapcv2 <- function(k,sel,y,lambdalist,rank,S1,valpr,SSx,tUy,ddlmin,index0) {
    sap <- 0
    for (j in 1:length(sel)) {
      if (attr(sel,"type")=="timeseries") {
        nj <- length(y[-(sel[[j]][1]:n)])
      } else {
        nj <- length(y[-sel[[j]]])
      }
      sommerk <- rev(sumvalpr(k,nj,rev(valpr[[j]]),nj-index0[j]+1,nj-ddlmin[j]+1))
      beta <- S1[[j]]$vectors%*%(sommerk[1:rank]*tUy[[j]][1:rank])
      beta <- t((1-valpr[[j]][1:rank])*t(S1[[j]]$Rm1U)) %*%t(S1[[j]]$vectors) %*% beta
      Yrescv <- as.vector(SSx[[j]]%*%beta)
      sap <- sap+sum(abs((y[sel[[j]]]-Yrescv)/y[sel[[j]]]))
    }
    return(sap)
  }
  
  n <- nrow(X)
  sel <- cvobs(n,ntest,ntrain,Kfold,type,npermut,seed)
  res <- list(minimum=0,objective=.Machine$double.xmax)
  objectif <- .Machine$double.xmax
  fraction <- sort(unique(c(fraction[(fraction<Kmax)&(fraction>Kmin)],Kmin,Kmax)))
  dep <- length(fraction)
  SSx <- as.list(rep(0,length(sel)))
  S1 <- valpr <- tUy <-lambdalist <- SSx
  ddlmin <- index0 <-  rep(0,length(sel))
  for (j in 1:length(sel)) {
    if (attr(sel,"type")=="timeseries") {
      XA <- X[-(sel[[j]][1]:n),,drop=FALSE]
      YA <- y[-(sel[[j]][1]:n)]
    } else  {
      XA <- X[-sel[[j]],,drop=FALSE]
      YA <- y[-sel[[j]]]
    }  
    if (is.null(lambda)&(!is.null(df))) {
      lambdalist[[j]] <- lambdachoicelr(XA,ddlmini*df,m,s,rank,itermax=100,bs,listvarx)
    } else lambdalist[[j]] <- lambda
    S1[[j]] <- lrsmoother(XA, bs,listvarx,lambdalist[[j]],m,s,rank) 
    tUy[[j]] <- as.vector(crossprod(S1[[j]]$vectors, YA))
    if (any(zapsmall(S1[[j]]$values-1,digits=9)==0)) {
      ddlmin[j] <-  sum(zapsmall(S1[[j]]$values-1,digits=9)==0)
    } else ddlmin[j] <- NA
    if (any(zapsmall(S1[[j]]$values,digits=9)==0)) {
      index0[j] <-  which(zapsmall(S1[[j]]$values,digits=9)==0)[1]
    } else index0[j] <- rank+1
    valpr[[j]] <- 1-c(S1[[j]]$values,rep(0,nrow(XA)-rank))
    valpr[[j]][valpr[[j]]<0] <- 0
    SSx[[j]] <- PredictMat(S1[[j]]$smoothobject,data.frame(X[sel[[j]],,drop=FALSE]))
    S1[[j]]$smoothobject <- NULL
  }
  if (criterion=="rmse") {
    repeat {
      mini <- fraction[dep-1]
      objectif <- choixssecv2(mini,sel=sel,y=y,lambdalist=lambdalist,rank=rank,S1=S1,valpr=valpr,SSx=SSx,tUy=tUy,ddlmin=ddlmin,index0=index0)
      if (objectif<.Machine$double.xmax) {
        bb <- fraction[dep]
        break 
      }
      dep <- dep-1
      if (dep==1) stop(paste("decrease Kmax below",fraction[2]))
    }
    phi <- (sqrt(5) - 1)/2
    repeat {
      x1 <- mini + (1-phi)*(bb-mini)
      objectif <- choixssecv2(x1,sel=sel,y=y,lambdalist=lambdalist,rank=rank,S1=S1,valpr=valpr,SSx=SSx,tUy=tUy,ddlmin=ddlmin,index0=index0)
      if (objectif<.Machine$double.xmax) break else {
        bb <- x1
      }
    }
    fraction <- c(fraction[1:(dep-1)],bb)
    for (i in 1:(length(fraction)-1)) {
      res1 <- stats::optimize(choixssecv2,lower=fraction[i],upper=fraction[i+1],tol=0.5,sel=sel,y=y,lambdalist=lambdalist,rank=rank,S1=S1,valpr=valpr,SSx=SSx,tUy=tUy,ddlmin=ddlmin,index0=index0)
      if (res1$objective<res$objective) res <- res1
    }
  } else {
    repeat {
      mini <- fraction[dep-1]
      objectif <- choixsapcv2(mini,sel=sel,y=y,lambdalist=lambdalist,rank=rank,S1=S1,valpr=valpr,SSx=SSx,tUy=tUy,ddlmin=ddlmin,index0=index0)
      if (objectif<.Machine$double.xmax) {
        bb <- fraction[dep]
        break 
      }
      dep <- dep-1
      if (dep==1) stop(paste("decrease Kmax below",fraction[2]))
    }
    phi <- (sqrt(5) - 1)/2
    repeat {
      x1 <- mini + (1-phi)*(bb-mini)
      objectif <- choixsapcv2(x1,sel=sel,y=y,lambdalist=lambdalist,rank=rank,S1=S1,valpr=valpr,SSx=SSx,tUy=tUy,ddlmin=ddlmin,index0=index0)
      if (objectif<.Machine$double.xmax) break else {
        bb <- x1
      }
    }
    fraction <- c(fraction[1:(dep-1)],bb)
    for (i in 1:(length(fraction)-1)) {
      res1 <- stats::optimize(choixsapcv2,lower=fraction[i],upper=fraction[i+1],tol=0.5,sel=sel,y=y,lambdalist=lambdalist,rank=rank,S1=S1,valpr=valpr,SSx=SSx,tUy=tUy,ddlmin=ddlmin,index0=index0)
      if (res1$objective<res$objective) res <- res1
    }
  }
  return(list(iter=round(res$minimum),objective=res$objective/sum(unlist(lapply(sel,length)))))
}

DuchonQ <- function(x,xk,m=2,s=0,symmetric=TRUE) {
  p <- ncol(x)
  nx <- nrow(x)
  if (!symmetric) {
    if (ncol(xk)!=p) stop("number of variables of x and xk must be the same\n")
    nxk <- nrow(xk)
  } else {
    nxk <- nx
    xk <- 0
  }
  k <- 2*m + 2*s - p
  negatif <- if ((1-2*((floor(k/2)+1)%%2))==-1) 1 else 0
  if (k%%2==0) {
    res <- .Call(ibr_semikerlog,as.double(x),as.double(xk),as.double(k/2),
                 as.integer(c(nx,p,nxk,negatif,symmetric)))
   } else {
    res <- .Call(ibr_semikerpow,as.double(x),as.double(xk),as.double(k/2),
                 as.integer(c(nx,p,nxk,negatif,symmetric)))
  }
 return(matrix(res,nrow=nx,ncol=nxk))
}

# fields, Tools for spatial data
# Copyright 2004-2011, Institute for Mathematics Applied Geosciences
# University Corporation for Atmospheric Research
# Licensed under the GPL -- www.gpl.org/licenses/gpl.html
DuchonS <- function(x, m = 2) {
    if (m < 1) 
        stop("'m' has to be larger than zero.")
    if (!is.matrix(x)) 
        x <- as.matrix(x)
    d <- ncol(x)
    n <- nrow(x)
    nterms<- choose((m + d -1),d)
    temp <- .Call(ibr_polynom, as.integer(c(m, n, d, n, nterms, n)),
        des = as.double(x))
    return(matrix(temp, nrow = n))
}

dsSx <- function(X,Xetoile,m=2,s=0) {
  n <- nrow(X)
  d <- ncol(X)
  n <- nrow(X)
  netoile <- nrow(Xetoile)
  d <- ncol(X)
  if (ncol(Xetoile)!=d) stop("the number of variables do not match")
  Sgu <-  DuchonS(Xetoile,m)
  Kgu <- DuchonQ(Xetoile,X,m,s,symmetric=FALSE)
  return(list(Sgu=Sgu,Qgu=Kgu))
}

predict.ibr <- function(object, newdata, interval= c("none", "confidence", "prediction"), ...) {
    if (!inherits(object, "ibr")) stop("must be an ibr object\n")
    interval <- match.arg(interval)
    if ((interval == "prediction")|(interval == "confidence")) {
      warning("Interval for prediction/confidence is not implemented yet\n")
    }
  if (missing(newdata) || is.null(newdata)) {
    Yres <- object$fitted
  } else {
      if (any(is.na(newdata))) stop("NA's in newdata\n")
      tt <- terms(object)
      Terms <- delete.response(tt)
      m <- model.frame(Terms, newdata, na.action = na.fail)
      newdata <- model.matrix(Terms, m)
      attributes(newdata) <- attributes(newdata)[c("dim","dimnames")]
      t <- terms(object)
      mf <- object$call
      m <- match(c("formula", "data", "subset"), names(mf), 0L)
      mf <- mf[c(1L, m)]
      mf$drop.unused.levels <- TRUE
      mf[[1L]] <- quote(stats::model.frame)
      mf <- eval(mf, environment(t))
      x <- model.matrix(t, mf)
    if (object$parcall$scaled) {
        newdata  <- scale(newdata,center=object$parcall$mean,scale=object$parcall$sd)
        x <- scale(x,center=object$parcall$mean,scale=object$parcall$sd)
    }
    if (object$parcall$smoother=="k") {
      SSx <- kernelSx(X=x,newdata,bx=object$bandwidth,kernelx=object$parcall$kernel)
      Yres <- as.vector(SSx%*%object$beta)
    }
    if ((object$parcall$smoother=="ds")|(object$parcall$smoother=="tps")) {
      SSx <- dsSx(x,newdata,m=object$parcall$m,s=object$parcall$s)
      Yres <- as.vector(SSx$Sgu%*%object$beta$d+SSx$Qgu%*%object$beta$c)
    }    
    if ((object$parcall$smoother=="lrds")|(object$parcall$smoother=="lrtps")) {
        SSx <- PredictMat(object$parcall$smoothobject,data.frame(newdata))
        Yres <- as.vector(SSx%*%object$beta)
    }    
  }
  return(Yres)
}

summary.npregress<- function(object, criteria="call", ...) {
  y <- object$call$y
  df <- object$df
  r <- object$residuals
  n <- length(r)
  stderr <- sqrt(sum(r^2)/(n-df))
  sigma2 <- stderr^2
  if (any(criteria=="call")) {
    criteria <- object$call$criterion
    anscrit <- NULL
  } else {
    crit <-c("aic","aicc","gcv","bic","gmdl")
    if (all(!(criteria%in%crit))) stop(paste("criteria are:",crit,"\n"))
    criteria <- criteria[criteria%in%crit]
    anscrit <- NULL
  }
  if (any(criteria=="gcv"))  anscrit <- c(anscrit,log(sigma2)-2*log(1-df/n))
  if (any(criteria=="aic"))  anscrit <- c(anscrit,log(sigma2)+2*df/n)
  if (any(criteria=="aicc"))  anscrit <- c(anscrit,log(sigma2)+1+(2*(df+1))/(n-df-2))
  if (any(criteria=="bic"))  anscrit <- c(anscrit,log(sigma2) + log(n)*(df)/n)
  if (any(criteria=="gmdl")) {
    Sbul <-   n*sigma2/(n-df)
  anscrit <- c(anscrit,log(Sbul)+df/n*log((sum(y^2)-n*sigma2)/(df*Sbul)))
  }
  if ((criteria!="user")&(!is.null(anscrit))) {
  names(anscrit) <- criteria
} else {
  anscrit <- "No Informative Criterion"
  names(anscrit) <- criteria
}
  ans <- list(residuals=r,Std.Error=stderr,Df=df,Resid.Df=n-df,criteria=anscrit,kernel=object$call$kernel,crit4bw=object$call$criterion,bandwidth=object$bandwidth)
  class(ans) <- "summary.npregress"
  ans
}

iterchoiceS1lrcve <- function(X,y,lambda,rank,bs,listvarx,df,ddlmini,ntest,ntrain,Kfold,type,npermut,seed,Kmin,Kmax,m,s) {
  iter <- 1
  MSE <- MAP <- rep(.Machine$double.xmax,length(Kmax-Kmin+1))
  n <- nrow(X)
 sel <- cvobs(n,ntest,ntrain,Kfold,type,npermut,seed)
  SSx <- as.list(rep(0,length(sel)))
  sumvalprk <- lambdalist <- S1 <- valpr <- tUy <- SSx
  nj <- rep(0,length(sel))
  for (j in 1:length(sel)) {
    if (attr(sel,"type")=="timeseries") {
      XA <- X[-(sel[[j]][1]:n),,drop=FALSE]
      YA <- y[-(sel[[j]][1]:n)]
    } else  {
      XA <- X[-sel[[j]],,drop=FALSE]
      YA <- y[-sel[[j]]]
    }  
    if (is.null(lambda)&(!is.null(df))) {
      lambdalist[[j]] <- lambdachoicelr(XA,ddlmini*df,m,s,rank,100,bs,listvarx)
    } else lambdalist[[j]] <- lambda
    nj[j] <- length(YA)
    S1[[j]] <- lrsmoother(XA, bs,listvarx,lambdalist[[j]],m,s,rank)
    tUy[[j]] <- as.vector(crossprod(S1[[j]]$vectors, YA))
    valpr0 <- S1[[j]]$values[-(1:ddlmini)]
    valpr[[j]] <- rep(0,rank)
    valpr[[j]][-(1:ddlmini)] <- (1-valpr0)
    sumvalprk[[j]] <- rep(1,rank)
    SSx[[j]] <- PredictMat(S1[[j]]$smoothobject,data.frame(X[sel[[j]],,drop=FALSE]))
  }
  if (Kmin>1) {
    for (k in 1:(Kmin-1)){
      for (j in 1:length(sel)) {
        sumvalprk[[j]] <- sumvalprk[[j]]+valpr[[j]]^k
      }
    }
  }
  for (k in Kmin:Kmax) {
    sse <- 0
    sap <- 0
    for (j in 1:length(sel)) {
  beta <- S1[[j]]$vectors%*%(sumvalprk[[j]]*tUy[[j]][1:rank])
  beta <- t((1-valpr[[j]])*t(S1[[j]]$Rm1U)) %*%t(S1[[j]]$vectors) %*% beta
       Yrescv <- as.vector(SSx[[j]]%*%beta)
      sse <- sse+sum((y[sel[[j]]]-Yrescv)^2)
      sap <- sap+sum(abs((y[sel[[j]]]-Yrescv)/y[sel[[j]]]))
      sumvalprk[[j]] <- sumvalprk[[j]]+valpr[[j]]^k
    }
    MSE[iter] <- sse/sum(n-nj)
    MAP[iter] <- sap/sum(n-nj)
    iter <- iter+1
  }
  return(list(rmse=sqrt(MSE),map=MAP))
}     

fittedS1 <- function(n,U,tUy,eigenvaluesS1,ddlmini,k) { 
  valpr0 <- eigenvaluesS1[-(1:ddlmini)]
  valpr <- rep(1,n)
  valpr[-(1:ddlmini)] <- 1-(1-valpr0)^k
  fk <- U%*%(valpr*tUy)
  return(list(fit=as.vector(fk),trace=sum(valpr)))
}

critAgcv <- function(k,valpr,tPADmdemiY,DdemiPA,n,ddlmini,ddlmaxi){
  ImLambdak <- rep(0,n)
  if (ddlmini>=1) {
    valpr0 <- valpr[-(1:ddlmini)]
    ImLambdak[-(1:ddlmini)] <- (1-valpr0)^k
  } else {
    ImLambdak <- (1-valpr)^k
  }   
  sigma2 <- sum((DdemiPA%*%(ImLambdak*tPADmdemiY))^2)/n
  logsigma2 <- log(sigma2)
  traceSk <- sum(1-ImLambdak)
  if (traceSk>ddlmaxi)  return(.Machine$double.xmax)
  if ((sigma2<1e-10)|(traceSk>((1-1e-10)*n))) return(.Machine$double.xmax)
  return(logsigma2 - 2*log(1-traceSk/n))
}

critAaicc <- function(k,valpr,tPADmdemiY,DdemiPA,n,ddlmini,ddlmaxi){
  ImLambdak <- rep(0,n)
  if (ddlmini>=1) {
    valpr0 <- valpr[-(1:ddlmini)]
    ImLambdak[-(1:ddlmini)] <- (1-valpr0)^k
  } else {
    ImLambdak <- (1-valpr)^k
  }   
  sigma2 <- sum((DdemiPA%*%(ImLambdak*tPADmdemiY))^2)/n
  logsigma2 <- log(sigma2)
  traceSk <- sum(1-ImLambdak)
  if (traceSk>ddlmaxi)  return(.Machine$double.xmax)
  if ((sigma2<1e-10)|(traceSk>((1-1e-10)*n))) return(.Machine$double.xmax)
  return(logsigma2 + 1+ 2*(traceSk+1)/(n-traceSk-2))
}

critAaic <- function(k,valpr,tPADmdemiY,DdemiPA,n,ddlmini,ddlmaxi){
  ImLambdak <- rep(0,n)
  if (ddlmini>=1) {
    valpr0 <- valpr[-(1:ddlmini)]
    ImLambdak[-(1:ddlmini)] <- (1-valpr0)^k
  } else {
    ImLambdak <- (1-valpr)^k
  }   
  sigma2 <- sum((DdemiPA%*%(ImLambdak*tPADmdemiY))^2)/n
  logsigma2 <- log(sigma2)
  traceSk <- sum(1-ImLambdak)
  if (traceSk>ddlmaxi)  return(.Machine$double.xmax)
  if ((sigma2<1e-10)|(traceSk>((1-1e-10)*n))) return(.Machine$double.xmax)
  return(logsigma2 + 2*(traceSk)/(n))
}
critAbic <- function(k,valpr,tPADmdemiY,DdemiPA,n,ddlmini,ddlmaxi){
  ImLambdak <- rep(0,n)
  if (ddlmini>=1) {
    valpr0 <- valpr[-(1:ddlmini)]
    ImLambdak[-(1:ddlmini)] <- (1-valpr0)^k
  } else {
    ImLambdak <- (1-valpr)^k
  }   
  sigma2 <- sum((DdemiPA%*%(ImLambdak*tPADmdemiY))^2)/n
  logsigma2 <- log(sigma2)
  traceSk <- sum(1-ImLambdak)
  if (traceSk>ddlmaxi)  return(.Machine$double.xmax)
  if ((sigma2<1e-10)|(traceSk>((1-1e-10)*n))) return(.Machine$double.xmax)
  return(logsigma2 + log(n)*(traceSk)/(n))
}
critAgmdl <- function(k,valpr,tPADmdemiY,DdemiPA,n,ddlmini,ddlmaxi,Y){
  ImLambdak <- rep(0,n)
  if (ddlmini>=1) {
    valpr0 <- valpr[-(1:ddlmini)]
    ImLambdak[-(1:ddlmini)] <- (1-valpr0)^k
  } else {
    ImLambdak <- (1-valpr)^k
  }   
  sigma2 <- sum((DdemiPA%*%(ImLambdak*tPADmdemiY))^2)/n
  logsigma2 <- log(sigma2)
  traceSk <- sum(1-ImLambdak)
  Sbul <- n*sigma2/(n-traceSk)
  if (traceSk>ddlmaxi)  return(.Machine$double.xmax)
  if ((sigma2<1e-10)|(traceSk>((1-1e-10)*n))) return(.Machine$double.xmax)
  return(log(Sbul)+traceSk/n*log((sum(Y^2)-n*sigma2)/(traceSk*Sbul)))
}

critS1gcv <- function(k,valpr,tUy2,n,ddlmini,ddlmaxi){
  valpr0 <- valpr[-(1:ddlmini)]
  ImLambdak <- rep(0,n)
  ImLambdak[-(1:ddlmini)] <- (1-valpr0)^k
  logsigma2 <- log(sum((ImLambdak^2)*(tUy2)))-log(n)  
  traceSk <- sum(1-ImLambdak)
  if (traceSk>ddlmaxi)  return(.Machine$double.xmax)
  if ((logsigma2<log(1e-10))|(traceSk>((1-1e-10)*n))) return(.Machine$double.xmax)
  return(logsigma2 - 2*log(1-traceSk/n))
}

critS1aicc <- function(k,valpr,tUy2,n,ddlmini,ddlmaxi){
  valpr0 <- valpr[-(1:ddlmini)]
  ImLambdak <- rep(0,n)
  ImLambdak[-(1:ddlmini)] <- (1-valpr0)^k
  logsigma2 <- log(sum((ImLambdak^2)*(tUy2)))-log(n)  
  traceSk <- sum(1-ImLambdak)
  if (traceSk>ddlmaxi)  return(.Machine$double.xmax)
  if ((logsigma2<log(1e-10))|(traceSk>((1-1e-10)*n))) return(.Machine$double.xmax)
  return(logsigma2 + 1+ 2*(traceSk+1)/(n-traceSk-2))
}

critS1aic <- function(k,valpr,tUy2,n,ddlmini,ddlmaxi){
  valpr0 <- valpr[-(1:ddlmini)]
  ImLambdak <- rep(0,n)
  ImLambdak[-(1:ddlmini)] <- (1-valpr0)^k
  logsigma2 <- log(sum((ImLambdak^2)*(tUy2)))-log(n)  
  traceSk <- sum(1-ImLambdak)
  if (traceSk>ddlmaxi)  return(.Machine$double.xmax)
  if ((logsigma2<log(1e-10))|(traceSk>((1-1e-10)*n))) return(.Machine$double.xmax)
  return(logsigma2 + 2*(traceSk)/(n))
}
critS1bic <- function(k,valpr,tUy2,n,ddlmini,ddlmaxi){
  valpr0 <- valpr[-(1:ddlmini)]
  ImLambdak <- rep(0,n)
  ImLambdak[-(1:ddlmini)] <- (1-valpr0)^k
  logsigma2 <- log(sum((ImLambdak^2)*(tUy2)))-log(n)  
  traceSk <- sum(1-ImLambdak)
  if (traceSk>ddlmaxi)  return(.Machine$double.xmax)
  if ((logsigma2<log(1e-10))|(traceSk>((1-1e-10)*n))) return(.Machine$double.xmax)
  return(logsigma2 + log(n)*(traceSk)/(n))
}
critS1gmdl <- function(k,valpr,tUy2,n,ddlmini,ddlmaxi,Y){
  valpr0 <- valpr[-(1:ddlmini)]
  ImLambdak <- rep(0,n)
  ImLambdak[-(1:ddlmini)] <- (1-valpr0)^k
  sigma2 <- sum((ImLambdak^2)*(tUy2))/n
  logsigma2 <- log(sigma2)
  traceSk <- sum(1-ImLambdak)
    Sbul <- n*sigma2/(n-traceSk)
  if (traceSk>ddlmaxi)  return(.Machine$double.xmax)
  if ((logsigma2<log(1e-10))|(traceSk>((1-1e-10)*n))) return(.Machine$double.xmax)
    return(log(Sbul)+traceSk/n*log((sum(Y^2)-n*sigma2)/(traceSk*Sbul)))
}

 
 

iterchoiceS1cve <- function(X,y,lambda,df,ddlmini,ntest,ntrain,Kfold,type,npermut,seed,Kmin,Kmax,m,s) {
  iter <- 1
  MSE <- MAP <- rep(.Machine$double.xmax,length(Kmax-Kmin+1))
  n <- nrow(X)
  sel <- cvobs(n,ntest,ntrain,Kfold,type,npermut,seed)
  U <- as.list(rep(0,length(sel)))
  sumvalprk <- lambdalist <- S1 <- valpr <- tUy <- Sp <- preprod <- SSx <- U
  nj <- rep(0,length(sel))
  for (j in 1:length(sel)) {
    if (attr(sel,"type")=="timeseries") {
      XA <- X[-(sel[[j]][1]:n),,drop=FALSE]
      YA <- y[-(sel[[j]][1]:n)]
    } else  {
      XA <- X[-sel[[j]],,drop=FALSE]
      YA <- y[-sel[[j]]]
    }  
    if (is.null(lambda)&(!is.null(df))) {
      lambdalist[[j]] <- lambdachoice(XA,ddlmini*df,m=m,s=s,itermax=100)
    } else lambdalist[[j]] <- lambda
    nj[j] <- length(YA)
    S1[[j]] <- dssmoother(XA, YA,lambda=lambdalist[[j]],m=m,s=s) 
    vp1.S1 <- eigen(S1[[j]]$H,symmetric=TRUE)
    U[[j]] <- vp1.S1$vect
    tUy[[j]] <- as.vector(crossprod(U[[j]], YA))
    valpr0 <- vp1.S1$values[-(1:ddlmini)]
    valpr[[j]] <- rep(0,nj[j])
    valpr[[j]][-(1:ddlmini)] <- (1-valpr0)
    sumvalprk[[j]] <- rep(1,nj[j])
    qrSgu <- qr(S1[[j]]$Sgu)
    F2 <- qr.Q(qrSgu,complete=TRUE)[,-(1:ncol(S1[[j]]$Sgu))]
    ainv <- t(F2)%*%S1[[j]]$Qgu%*%F2
    diag(ainv) <- diag(ainv)+lambdalist[[j]]
    Sp[[j]] <- -lambda*F2%*%(solve(ainv))%*%t(F2)
    SSx[[j]] <- dsSx(X=XA,X[sel[[j]],,drop=FALSE],m,s)
    preprod[[j]] <- solve(qr.R(qrSgu))%*%(t(qr.Q(qrSgu)))
  }
  if (Kmin>1) {
    for (k in 1:(Kmin-1)){
      for (j in 1:length(sel)) {
        sumvalprk[[j]] <- sumvalprk[[j]]+valpr[[j]]^k
      }
    }
  }
  for (k in Kmin:Kmax) {
    sse <- 0
    sap <- 0
    for (j in 1:length(sel)) {
      beta <- U[[j]]%*%(sumvalprk[[j]]*tUy[[j]])
      cgubeta <- as.vector(Sp[[j]]%*%beta)/(-lambdalist[[j]])
      dgubeta <- preprod[[j]]%*%(as.matrix(beta)-(S1[[j]])$Qgu%*%cgubeta)
      Yrescv <- as.vector((SSx[[j]])$Sgu%*%dgubeta+(SSx[[j]])$Qgu%*%cgubeta)
      sse <- sse+sum((y[sel[[j]]]-Yrescv)^2)
      sap <- sap+sum(abs((y[sel[[j]]]-Yrescv)/y[sel[[j]]]))
      sumvalprk[[j]] <- sumvalprk[[j]]+valpr[[j]]^k
    }
    MSE[iter] <- sse/sum(n-nj)
    MAP[iter] <- sap/sum(n-nj)
    iter <- iter+1
  }
  return(list(rmse=sqrt(MSE),map=MAP))
}     

betaS1lr <- function(n,U,tUy,eigenvaluesS1,ddlmini,k,lambda,rank,Rm1U,index0) {
  prov <- rev(sumvalpr(k,n,rev(1-eigenvaluesS1),n-index0+1,n-ddlmini+1))
  beta <- U%*%(prov[1:rank]*tUy[1:rank])
  beta <- t(eigenvaluesS1[1:rank]*t(Rm1U)) %*%t(U) %*% beta
  return(beta)
}

print.summary.npregress <- function(x, digits = max(3, getOption("digits") - 3), ...) {
  resid <- x$residuals
  df <- x$df
  nam <- c("Min", "1Q", "Median", "3Q", "Max")
  rq <-structure(quantile(resid), names = nam)
  cat("Residuals:\n")
  print(rq, digits = digits, ...)
  cat("Residual standard error:", format(signif(x$Std.Error, 
        digits)), "on", format(round(x$Resid.Df,1)), "degrees of freedom\n")
  if (x$criteria!="user") {
    crite <- structure(x$criteria, names = names(x$criteria))
    print(crite, digits = digits, ...)
  }
  kernelsmooth <- c("gaussian","Epanechnikov","uniform","quartic")
  resu <- match(x$kernel,c("g","e","u","q"))
  cat("Kernel:",kernelsmooth[resu],"(with",format(signif(x$Df, 
        digits)),"df)\n")
  cat("\nBandwidth:", format(signif(x$bandwidth,digits)), "chosen by", x$crit4bw, "\n")
  invisible(x)
}

lambdachoicelr <- function(x,ddlobjectif,m=2,s=0,rank,itermax,bs,listvarx) {
  n <- nrow(x)
  d <- ncol(x)
  ddlmin <- choose(m+d-1,m-1)
  if (ddlobjectif<=ddlmin) stop(paste("the objective df is too small, choose it greater than",ddlmin))
    if (bs=="tp") { 
        objet <- list(term=listvarx,bs.dim=rank,fixed=FALSE,dim=ncol(x),p.order=m,by="NA",label=paste("s(",paste(listvarx,collapse=","),")",sep=""),xt=NULL,id=NULL,sp=NULL)
        attr(objet,"class") <- "tp.smooth.spec"
    } else {
        objet <- list(term=colnames(x),bs.dim=rank,fixed=FALSE,dim=ncol(x),p.order=c(m,s),by="NA",label=paste("s(",paste(listvarx,collapse=","),")",sep=""),xt=NULL,id=NULL,sp=NULL)
        attr(objet,"class") <- "ds.smooth.spec"       
    }
  sm <- mgcv::smoothCon(objet,data=data.frame(x),knots=NULL)[[1]]
    Xlr <- sm$X
    Slr <- sm$S[[1]]
    qrx <- qr(Xlr)
    Rlr <- qr.R(qrx)
    vp <- eigen(forwardsolve(t(Rlr),t(forwardsolve(t(Rlr),Slr))),symmetric=TRUE,only.values=TRUE)$values
    ## trace : sum( 1/(1+lambda*vp) )
  trace <- function(loglambda,vp1) {
    sum(1/(1+exp(loglambda)*vp1)) - ddlobjectif
  }
  l1 <- 1
 for (k in 1:25) {
    tr <- sum( 1/(1+l1*vp) )
       if (tr <= ddlobjectif)  break
        l1 <- l1 * 4
    }
     l2 <- 1
    for (k in 1:25) {
        tr <- sum( 1/(1+l2*vp) )
         if (tr > ddlobjectif) break
        l2 <- l2/4
    }
    resultat <- stats::uniroot(trace,c(log(l2),log(l1)),vp1=vp,maxiter =itermax)
    return(exp(resultat$root))
}

print.summary.ibr <- function(x, displaybw=FALSE, digits = max(3, getOption("digits") - 3), ...) {
  resid <- x$residuals
  df <- x$df
  nam <- c("Min", "1Q", "Median", "3Q", "Max")
  rq <-structure(quantile(resid), names = nam)
  cat("Residuals:\n")
  print(rq, digits = digits, ...)
  cat("Residual standard error:", format(signif(x$Std.Error, 
        digits)), "on", format(round(x$Resid.Df,1)), "degrees of freedom\n")
  cat("\nInitial df:", format(round(x$Initial.Df,2)), "; Final df:", format(round(x$Final.Df,2)), "\n")
  if (!is.null(x$criteria)) {
      crite <-structure(x$criteria, names = names(x$criteria))
      if (length(crite)>1) {
          cat("\nCriteria:\n")
      }
      print(crite, digits = digits, ...)
  if ((length(crite)>1)&&(!all(is.na(x$iterations)))) {
    cat("Iterations:\n")
  crite2 <- structure(x$iterations,names = names(x$iterations))
    print(crite2, digits = digits, ...)
}
  }
  cat("\nNumber of iterations:", x$iter, "chosen by", x$crit4iter, "\n")
  kernelsmooth <- c("gaussian kernel","Epanechnikov kernel","uniform kernel","quartic kernel")
  resu <- match(x$kernel,c("g","e","u","q"))
  if (x$smoother=="k") {
    cat("Base smoother:",kernelsmooth[resu],"(with",format(round(x$Initial.Df,2)),"df)\n")
    if (displaybw) {
      cat("\nBandwith\n")
      bw <-structure(x$bandwidth, names = names(x$bandwidth))
      print(bw, digits = digits, ...)
    }
  }
  if (x$smoother=="tps") {
    cat("Base smoother: Thin plate spline of order",x$m,"(with",format(round(x$Initial.Df,2)),"df)\n")
    if (displaybw) {
      cat("\nBandwith\n")
      bw <-structure(x$bandwidth, names = "lambda")
      print(bw, digits = digits, ...)
    }
  }
  if (x$smoother=="ds") {
    cat("Base smoother: Duchon spline with derivative order m=",x$m,"\n  and weight exponent s=",x$s,"(with",format(round(x$Initial.Df,2)),"df)\n")
    if (displaybw) {
      cat("\nBandwith\n")
      bw <-structure(x$bandwidth, names = "lambda")
      print(bw, digits = digits, ...)
    }
  }
  invisible(x)
}

npregress <- function(x,y,criterion="rmse",bandwidth=NULL,kernel="g",control.par=list(),cv.options=list()) {
  kern <- c("g", "e", "q", "u")
  kernel <- match.arg(kernel,kern)
  kernelint <- which(kernel==kern)
  if (any(is.na(x))) stop("NA's in x\n")
  if (any(is.na(y))) stop("NA's in y\n")
  if (!is.numeric(x)&(is.data.frame(x))) {
    x <- x[,1]
    if (!is.numeric(x)) stop("first column of data-frame is not numeric\n")
  }
  if (is.matrix(x)) {
    x <- as.vector(x)
  }
  if (!is.numeric(x)) stop("x must be a numeric vector (or a data-frame with first column of numeric type)\n")
  if (!is.numeric(y)) stop("y must be numeric\n")
  if (! is.vector(y)) {
    y <- as.vector(y)
  }
  if (missing(bandwidth)) bandwidth <- NULL
  if ((!is.null(bandwidth))&(!is.numeric(bandwidth) || (bandwidth<0))) stop("invalid bandwidth\n")
  contr.sp <- list(bandwidth=NULL,degree=0)
  contr.sp[(names(control.par))] <- control.par
  if (is.null(bandwidth)) {
    if ((!is.null(contr.sp$bandwidth))&(!is.numeric(contr.sp$bandwidth) || (contr.sp$bandwidth<0))) stop("invalid bandwidth\n")
    bandwidth <- contr.sp$bandwidth
  }
  if (contr.sp$degree>1) stop("Not implemented. Please consider using KernSmooth or another library for degree greater or equal to 2\n")
  if (contr.sp$degree==0) nom <- "reg" else nom <- "regpol"
  crit <-c("rmse","map")
  criterion <- match.arg(criterion,crit)
  n <- length(x)
  if ((criterion%in%c("rmse","map"))&(is.null(bandwidth))) {
    mini <- 1/n
    kmax=floor(log(n*diff(range(x))/3)/log(1+1/n))
    gridbw <- 1/n*(1+1/n)^(0:kmax)
    cv <- list(gridbw=gridbw,ntest=1,ntrain=NULL,Kfold=TRUE,type="consecutive",seed=NULL,npermut=NULL)
    cv[(names(cv.options))] <- cv.options
    if ((!is.numeric(cv$gridbw))||(!is.vector(cv$gridbw))) stop("invalid gridbw component of cv.options: must be a numeric vector\n")
    if (!all(sapply(cv[c(2,3,6,8)], FUN=function(x) is.numeric(x)||is.null(x)))) stop("invalid cv parameters: must be numeric or NULL\n")
    if (any(names(cv.options)=="ntrain")) cv$ntest <- NULL
    sel <- cvobs(n,cv$ntest,cv$ntrain,cv$Kfold,cv$type,cv$npermut,cv$seed)
    ordre <- unlist(sel)
    xord <- x[ordre]
    yord <- y[ordre]
    nj <- unlist(lapply(sel,length))
    effold <- c(0,cumsum(nj))
    neffold <- length(sel)
    if (nom=="regpol")  {
       prov <- .Call(ibr_npregpolcv, as.double(xord), as.double(yord),as.double(gridbw),as.integer(effold),as.integer(kernelint))
    } else if (nom=="reg")  {
       prov <- .Call(ibr_npregcv, as.double(xord), as.double(yord),as.double(gridbw),as.integer(effold),as.integer(kernelint))
    }
    ################
    rmse <- sqrt(prov[[1]]/sum(n-nj))
    map <- prov[[2]]/sum(n-nj)
    choixbw <- list(gridbw=gridbw,rmse=rmse,map=map)
    bandwidth <- gridbw[switch(criterion,rmse=which.min(rmse),map=which.min(map))]
  } else {
    choixbw <- NULL
    criterion <- "user"
  }
  if (contr.sp$degree==0) {
      prov <- .Call(ibr_npreg, as.double(x), as.double(y), as.double(x), as.double(bandwidth),as.integer(kernelint))
    } else {
      prov <- .Call(ibr_npregpol, as.double(x), as.double(y), as.double(x), as.double(bandwidth),as.integer(kernelint))
    }
  fit <- prov[[1]]
  df <- prov[[2]]
  residuals <- y- fit
  
  res <- list(bandwidth=bandwidth,residuals=residuals,fitted=fit,df=df,call=list(x=x,y=y,criterion=criterion,kernel=kernel,degree=contr.sp$degree),criteria=choixbw)
  class(res) <- c("npregress", "list")
  return(res)
}

     
 

iterchoiceS1 <- function(n,mini,maxi,tUy,eigenvaluesS1,ddlmini,ddlmaxi,y,criterion,fraction){
  res <- list(minimum=0,objective=.Machine$double.xmax)
  tUy2 <- tUy^2
  if (criterion=="gmdl") {
    objectif <- .Machine$double.xmax
    fraction <- sort(unique(c(fraction[(fraction<maxi)&(fraction>mini)],mini,maxi)))
    dep <- length(fraction)
    repeat {
      mini <- fraction[dep-1]
      objectif <- critS1gmdl(mini,valpr=eigenvaluesS1,tUy2=tUy2,n=n,ddlmini=ddlmini,ddlmaxi=ddlmaxi,Y=y)
      if (objectif<.Machine$double.xmax) {
        bb <- fraction[dep]
        break 
      }
      dep <- dep-1
      if (dep==1) stop(paste("decrease Kmax below",fraction[2],"or increase dfmaxi component of control.par list"))
    }
    phi <- (sqrt(5) - 1)/2
    repeat {
      x1 <- mini + (1-phi)*(bb-mini)
      objectif <- critS1gmdl(x1,valpr=eigenvaluesS1,tUy2=tUy2,n=n,ddlmini=ddlmini,ddlmaxi=ddlmaxi,Y=y)
      if (objectif<.Machine$double.xmax) break else {
        bb <- x1
      }
    }
    fraction <- c(fraction[1:(dep-1)],bb)
    for (i in 1:(length(fraction)-1)) {
      res1 <- stats::optimize(critS1gmdl,lower=fraction[i],upper=fraction[i+1],tol=0.5,valpr=eigenvaluesS1,tUy2=tUy2,n=n,ddlmini=ddlmini,ddlmaxi=ddlmaxi,Y=y)
      if (res1$objective<res$objective) res <- res1
    }
  } else {
    fcriterion <- switch(criterion,aic=critS1aic,aicc=critS1aicc,gcv=critS1gcv,bic=critS1bic)
    objectif <- .Machine$double.xmax
    fraction <- sort(unique(c(fraction[(fraction<maxi)&(fraction>mini)],mini,maxi)))
    dep <- length(fraction)
    repeat {
      mini <- fraction[dep-1]
      objectif <- fcriterion(mini,valpr=eigenvaluesS1,tUy2=tUy2,n=n,ddlmini=ddlmini,ddlmaxi=ddlmaxi)
      if (objectif<.Machine$double.xmax) {
        bb <- fraction[dep]
        break 
      }
      dep <- dep-1
      if (dep==1) stop(paste("decrease Kmax below",fraction[2],"or increase dfmaxi component of control.par list"))
    }
    phi <- (sqrt(5) - 1)/2
    repeat {
      x1 <- mini + (1-phi)*(bb-mini)
      objectif <- fcriterion(x1,valpr=eigenvaluesS1,tUy2=tUy2,n=n,ddlmini=ddlmini,ddlmaxi=ddlmaxi)
      if (objectif<.Machine$double.xmax) break else {
        bb <- x1
      }
    }
    fraction <- c(fraction[1:(dep-1)],bb)
    for (i in 1:(length(fraction)-1)) {
      res1 <- stats::optimize(fcriterion,lower=fraction[i],upper=fraction[i+1],tol=0.5,valpr=eigenvaluesS1,tUy2=tUy2,n=n,ddlmini=ddlmini,ddlmaxi=ddlmaxi)
      if (res1$objective<res$objective) res <- res1
    }     
  }
  return(list(iter=floor(res$minimum),objective=res$objective))
}    

iterchoiceS1e <- function(y,K,tUy,eigenvaluesS1,ddlmini,ddlmaxi){
  n <- length(y)
  iter <- 1
  tUy2 <- tUy^2
  GCV <- rep(.Machine$double.xmax,length(K))
  AICc <- rep(.Machine$double.xmax,length(K))
  AIC <- rep(.Machine$double.xmax,length(K))
  BIC <- rep(.Machine$double.xmax,length(K))
  gMDL <- rep(.Machine$double.xmax,length(K))
  traceS <- rep(.Machine$double.xmax,length(K))
  sigma2list <- rep(.Machine$double.xmax,length(K))
  valpr0 <- eigenvaluesS1[-(1:ddlmini)]
  ImLambda <- rep(0,n)
  ImLambda[-(1:ddlmini)] <- (1-valpr0)
  ImLambdak <- rep(1,n)
  kcourant <- K[1]
  if (kcourant>1) {
    for (j in 1:(kcourant-1)){
      ImLambdak <- ImLambdak*ImLambda
    }
  }
  for (k in K) {
    while (kcourant!=k) {
      ImLambdak <- ImLambdak*ImLambda
      kcourant <- kcourant+1
    }
    ImLambdakp1 <- ImLambdak*ImLambda
    traceSk <- sum(1-ImLambdakp1)
    traceS[iter] <- traceSk
    sigma2 <- (sum((ImLambdakp1^2)*(tUy^2)))/n
    sigma2list[iter] <- sigma2
    logsigma2 <- log(sum((ImLambdakp1^2)*(tUy^2)))-log(n)
    if ((sigma2<1e-10)|(traceSk>((1-1e-10)*n))) break
    GCV[iter] <-  logsigma2 - 2*log(1-traceSk/n) 
    AICc[iter] <-  logsigma2 + 1+ 2*(traceSk+1)/(n-traceSk-2)
    AIC[iter] <-  logsigma2 + 2*(traceSk)/(n)
    BIC[iter] <-  logsigma2 + log(n)*(traceSk)/(n)
    Sbul <- n*sigma2/(n-traceSk)
    gMDL[iter] <- log(Sbul)+traceSk/n*log((sum(y^2)-n*sigma2)/(traceSk*Sbul))
    if (traceSk>ddlmaxi) break
    iter <- iter+1
  }
  
  return(list(aic=AIC,aicc=AICc,gcv=GCV,bic=BIC,gmdl=gMDL,df=traceS,sigma2=sigma2list,kmax=iter))
}

lambdachoice <- function(X,ddlobjectif,m=2,s=0,itermax,smoother="tps") {
  n <- nrow(X)
  d <- ncol(X)
  p <- 2*m-d
  ddlmin <- choose(m+d-1,m-1)
  if (ddlobjectif<=ddlmin) stop(paste("the objective df is too small, choose it greater than",ddlmin))
  Sgu <- DuchonS(X,m)
  qrSgu <- qr(Sgu)
  F2 <- qr.Q(qrSgu,complete=TRUE)[,-(1:ncol(Sgu))]
  Kgu <- DuchonQ(X,0,m,s,symmetric=TRUE)
  ainv <- t(F2)%*%Kgu%*%F2
  vp <- eigen(ainv,only.values=TRUE,symmetric=TRUE)$values
  trace <- function(loglambda,vp1=vp) {
    n-sum(1/(vp1/exp(loglambda)+1)) - ddlobjectif
  }
  l1 <- 1
 for (k in 1:25) {
    tr <- n-sum(1/(1 + vp/l1))
       if (tr <= ddlobjectif)  break
        l1 <- l1 * 4
    }
     l2 <- 1
    for (k in 1:25) {
        tr <- n-sum(1/(1 + vp/l2))
         if (tr > ddlobjectif) break
        l2 <- l2/4
    }
    resultat <- stats::uniroot(trace,c(log(l2),log(l1)),vp1=vp,maxiter =itermax)
    return(exp(resultat$root))
}

kernelKx <- function(X,Xetoile=NULL,bx,kernelx="g"){
  X <- as.matrix(X)
  n <- nrow(X)
  p <- ncol(X)
  namekernel <- c("g","e","q","u")
  kernelint <- which(namekernel==kernelx)
  if (length(kernelint)!=1) stop(paste("kernelx must be in",paste(namekernel, collapse=", ")))
  if (is.null(Xetoile)) {
    K <- .Call(ibr_Kmatrix, X, 0, bx, as.integer(c(n, p, 0, kernelint, 1)))
  } else {
    netoile <- nrow(Xetoile)
    K <- .Call(ibr_Kmatrix,  X, as.matrix(Xetoile), bx, as.integer(c(n, p, netoile, kernelint, 0)))
  }
 return(K)
}

lrsmoother <- function(x,bs,listvarx,lambda,m,s,rank) {
    if (bs=="tp") { 
        objet <- list(term=listvarx,bs.dim=rank,fixed=FALSE,dim=ncol(x),p.order=m,by="NA",label=paste("s(",paste(listvarx,collapse=","),")",sep=""),xt=NULL,id=NULL,sp=NULL)
        attr(objet,"class") <- "tp.smooth.spec"
    } else {
        objet <- list(term=colnames(x),bs.dim=rank,fixed=FALSE,dim=ncol(x),p.order=c(m,s),by="NA",label=paste("s(",paste(listvarx,collapse=","),")",sep=""),xt=NULL,id=NULL,sp=NULL)
        attr(objet,"class") <- "ds.smooth.spec"       
    }
    sm <- mgcv::smoothCon(objet,data=data.frame(x),knots=NULL)[[1]]
    Xlr <- sm$X
    Slr <- sm$S[[1]]
    qrx <- qr(Xlr)
    Rlr <- qr.R(qrx)
    es <- eigen(forwardsolve(t(Rlr),t(forwardsolve(t(Rlr),Slr))),symmetric=TRUE)
    ## dans l'ordre decroissant 
    eigenvaluesS1 <- rev(1/(1+lambda*es$values))
    U <- qr.Q(qrx)%*%es$vectors[,rank:1]
    ## calcul de (R^-1)U
    Rm1U <- backsolve(qr.R(qrx),es$vectors[,rank:1])
    return(list(vectors=U,values=eigenvaluesS1,Rm1U=Rm1U,smoothobject=sm))
}

cvobs <- function(n,ntest,ntrain,Kfold,type= c("random","timeseries","consecutive", "interleaved"),npermut,seed) {
  tabletype <- c("random","timeseries","consecutive", "interleaved")
  type <- match.arg(type,tabletype)
  if (missing(ntest)) ntest <- NULL
  if (missing(ntrain)) ntrain <- NULL
  if (missing(npermut)) npermut <- NULL
  if (missing(Kfold)) Kfold <- FALSE
  if (is.logical(Kfold)&&(!Kfold)) {
    if (type%in%c("interleaved","consecutive")) stop("When NOT using Kfold cross-validation, type must be 'random' or 'timeseries'\n")
  }
  if (missing(seed)) seed <- NULL
  if (type=="timeseries") {
    if (is.null(ntest)&is.null(ntrain)) 
      stop("argument ntest or ntrain is missing\n")
    if (!is.null(ntest)) {
      if (is.null(npermut)) stop("number of permutation is missing\n")
      res <- as.list(rep(0,npermut))
      names(res) <- paste("V",1:npermut,sep="")
      for (j in 1:npermut) {
        res[[j]] <- seq(n-ntest+1-npermut+j,n-npermut+j,by=1)
      }
      attr(res, "type") <- "timeseries"
      return(res)
    }
    if (!is.null(ntrain)) {
      ntest <- n-ntrain-npermut+1
      if (is.null(npermut)) stop("number of permutation is missing\n")
      res <- as.list(rep(0,npermut))
      names(res) <- paste("V",1:npermut,sep="")
      for (j in 1:npermut) {
        res[[j]] <- seq(n-ntest+1-npermut+j,n-npermut+j,by=1)
      }
      attr(res, "timeseries")
      attr(res,"incomplete") <- 0
      return(res)
    }
  } else {
  if (is.logical(Kfold)) {
    if ((Kfold)&(is.null(ntest)&is.null(ntrain)))
      stop("argument(s) Kfold and/or ntest and/or ntrain are missing\n")
    if (Kfold&(!is.null(ntest))) {
      if (!is.numeric(ntest)) stop("ntest is not numeric\n")
      Kfold <- n%/%ntest
    }
    if (Kfold&(!is.null(ntrain))) {
      if (!is.numeric(ntrain)) stop("ntrain is not numeric\n")
      Kfold <- n%/%(n-ntrain)
    }
  }
  if (is.numeric(Kfold))   {
    if (n < Kfold) 
      stop("More segments (Kfold) than observations")
    length.seg <- ceiling(n/Kfold)
    incomplete <- Kfold * length.seg - n
    complete <- Kfold - incomplete  
    type <- match.arg(type)
    if (!is.null(seed)) set.seed(seed)
    switch(type, random = {
      inds <- matrix(c(sample(1:n), rep(NA, incomplete)), nrow = length.seg, 
                     byrow = TRUE)
    }, consecutive = {
      if (complete < Kfold) {
        inds <- cbind(matrix(1:(length.seg * complete), nrow = length.seg), 
                      rbind(matrix((length.seg * complete + 1):n, nrow = length.seg - 
                                   1), NA))
      }
      else {
        inds <- matrix(1:n, nrow = length.seg)
      }
    }, interleaved = {
      inds <- matrix(c(1:n, rep(NA, incomplete)), nrow = length.seg, 
                     byrow = TRUE)
    })
    res <- lapply(as.data.frame(inds), function(x) c(na.omit(x)))
    attr(res, "incomplete") <- incomplete
    attr(res, "type") <- if (length.seg == 1) 
      "leave-one-out"
    else type
  }
  if ((!is.numeric(Kfold))&(type=="random")) {
    if (is.null(npermut)) stop("number of random draw of test set (npermut) is unknown\n")
    if (!is.null(seed)) set.seed(seed)
    if (is.null(ntest)&is.null(ntrain)) stop("ntrain or ntest is missing\n")
    if (is.null(ntest)) ntest <- n-ntrain
    if (is.null(ntrain)) ntrain <- n-ntest
    if (n!= (ntrain+ntest)) stop("number of observations not equal the sum of those  in test set plus those in test set\n")
    res <- as.list(rep(0,npermut))
    names(res) <- paste("V",1:npermut,sep="")
    for (j in 1:npermut) {
      res[[j]] <- sample(1:n,ntest)
    }
    attr(res, "type") <- "randperm"
    attr(res,"incomplete") <- 0
  }
}
  return(res)
}

iterchoiceAcv <- function(X,y,bx,df,kernelx,ddlmini,ntest,ntrain,Kfold,type,npermut,seed,Kmin,Kmax,criterion,fraction) {
  choixssecv2 <- function(k,sel,SSx,y,valpr,tPADmdemiY,DdemiPA,ddlmin,index0) {
    sse <- 0
    for (j in 1:length(sel)) {
      if (attr(sel,"type")=="timeseries") {
        nj <- length(y[-(sel[[j]][1]:n)])
      } else {
        nj <- length(y[-sel[[j]]])
      }
      prov <- rev(sumvalpr(k,nj,rev(valpr[[j]]),nj-index0[j]+1,nj-ddlmin[j]+1))
      prov1 <- matrix(prov*as.vector(tPADmdemiY[[j]]),nj,1)
      Yrescv <- SSx[[j]]%*%DdemiPA[[j]]%*%prov1
      sse <- sse+sum((y[sel[[j]]]-Yrescv)^2)
    }
    return(sse)
  }
  choixsapcv2 <- function(k,sel,SSx,y,valpr,tPADmdemiY,DdemiPA,ddlmin,index0) {
    sap <- 0
    for (j in 1:length(sel)) {
      if (attr(sel,"type")=="timeseries") {
        nj <- length(y[-(sel[[j]][1]:n)])
      } else {
        nj <- length(y[-sel[[j]]])
      }
      prov <- rev(sumvalpr(k,nj,rev(valpr[[j]]),nj-index0[j]+1,nj-ddlmin[j]+1))
      prov1 <- matrix(prov*as.vector(tPADmdemiY[[j]]),length(y[-sel[[j]]]),1)
      Yrescv <- SSx[[j]]%*%DdemiPA[[j]]%*%prov1
      sap <- sap+sum(abs((y[sel[[j]]]-Yrescv)/y[sel[[j]]]))
    }
    return(sap)
  }
  
  n <- nrow(X)
  sel <- cvobs(n,ntest,ntrain,Kfold,type,npermut,seed)
  res <- list(minimum=0,objective=.Machine$double.xmax)
  objectif <- .Machine$double.xmax
  fraction <- sort(unique(c(fraction[(fraction<Kmax)&(fraction>Kmin)],Kmin,Kmax)))
  dep <- length(fraction)
    tPADmdemiY <- as.list(rep(0,length(sel)))
    valpr <- DdemiPA <- SSx <- tPADmdemiY
    index0 <- ddlmin <- nj <- rep(0,length(sel))
    for (j in 1:length(sel)) {
      if (attr(sel,"type")=="timeseries") {
        XA <- X[-(sel[[j]][1]:n),,drop=FALSE]
        YA <- y[-(sel[[j]][1]:n)]
      } else  {
        XA <- X[-sel[[j]],,drop=FALSE]
        YA <- y[-sel[[j]]]
      }  
      if (is.null(bx)&(!is.null(df))) {
        bx <- bwchoice(X=XA,objectif=df,kernelx,itermax=100)
      }
      nj[j] <- length(YA)
      listeA <- calcA(X=XA,bx=bx,kernelx=kernelx)
      listeA.eig <- eigen(listeA$A,symmetric=TRUE)
      if (any(zapsmall(listeA.eig$values-1,digits=9)==0)) {
        ddlmin[j] <-  sum(zapsmall(listeA.eig$values-1,digits=9)==0)
      } else ddlmin[j] <- NA
      if (any(zapsmall(listeA.eig$values,digits=9)==0)) {
        index0[j] <-  which(zapsmall(listeA.eig$values,digits=9)==0)[1]
      } else index0[j] <- NA
      tPADmdemiY[[j]] <- t(listeA.eig$vectors*(1/listeA$Ddemi))%*%YA
      valpr[[j]]  <- 1-listeA.eig$values
      DdemiPA[[j]] <- (listeA$Ddemi*listeA.eig$vectors)
      SSx[[j]] <- kernelSx(kernelx=kernelx,X=XA,bx=bx,X[sel[[j]],,drop=FALSE])
    }
    if (criterion=="rmse") {
      repeat {
        mini <- fraction[dep-1]
        objectif <- choixssecv2(mini,sel=sel,SSx=SSx,y=y,valpr=valpr,tPADmdemiY=tPADmdemiY,DdemiPA=DdemiPA,ddlmin=ddlmin,index0=index0)
        if (objectif<.Machine$double.xmax) {
          bb <- fraction[dep]
          break 
        }
        dep <- dep-1
        if (dep==1) stop(paste("decrease Kmax below",fraction[2]))
      }
      phi <- (sqrt(5) - 1)/2
      repeat {
        x1 <- mini + (1-phi)*(bb-mini)
        objectif <- choixssecv2(x1,sel=sel,SSx=SSx,y=y,valpr=valpr,tPADmdemiY=tPADmdemiY,DdemiPA=DdemiPA,ddlmin=ddlmin,index0=index0)
        if (objectif<.Machine$double.xmax) break else {
          bb <- x1
        }
      }
      fraction <- c(fraction[1:(dep-1)],bb)
      for (i in 1:(length(fraction)-1)) {
        res1 <- stats::optimize(choixssecv2,lower=fraction[i],upper=fraction[i+1],tol=0.5,sel=sel,SSx=SSx,y=y,valpr=valpr,tPADmdemiY=tPADmdemiY,DdemiPA=DdemiPA,ddlmin=ddlmin,index0=index0)
        if (res1$objective<res$objective) res <- res1
      }
    } else {
      repeat {
        mini <- fraction[dep-1]
        objectif <- choixsapcv2(mini,sel=sel,SSx=SSx,y=y,valpr=valpr,tPADmdemiY=tPADmdemiY,DdemiPA=DdemiPA,ddlmin=ddlmin,index0=index0)
        if (objectif<.Machine$double.xmax) {
          bb <- fraction[dep]
          break 
        }
        dep <- dep-1
        if (dep==1) stop(paste("decrease Kmax below",fraction[2]))
      }
      phi <- (sqrt(5) - 1)/2
      repeat {
        x1 <- mini + (1-phi)*(bb-mini)
        objectif <- choixsapcv2(x1,sel=sel,SSx=SSx,y=y,valpr=valpr,tPADmdemiY=tPADmdemiY,DdemiPA=DdemiPA,ddlmin=ddlmin,index0=index0)
        if (objectif<.Machine$double.xmax) break else {
          bb <- x1
        }
      }
      fraction <- c(fraction[1:(dep-1)],bb)
      for (i in 1:(length(fraction)-1)) {
        res1 <- stats::optimize(choixsapcv2,lower=fraction[i],upper=fraction[i+1],tol=0.5,sel=sel,SSx=SSx,y=y,valpr=valpr,tPADmdemiY=tPADmdemiY,DdemiPA=DdemiPA,ddlmin=ddlmin,index0=index0)
        if (res1$objective<res$objective) res <- res1
      }
    }
  return(list(iter=round(res$minimum),objective=res$objective/sum(unlist(lapply(sel,length)))))
}

print.ibr <- function(x, digits = max(2, getOption("digits") - 4), ...) {
  cat("\nInitial df:", format(round(x$initialdf,digits)), "; Final df:", format(round(x$finaldf,digits)), "\n")
 if (x$parcall$critmethod=="aggregation") crit4iter <- paste("aggregation of:",paste(x$parcall$criterion,collapse=", ")) else crit4iter <- x$parcall$criterion[1]
  cat("Number of iterations:", x$iter, "chosen by", crit4iter, "\n")
}

ibr <- function(formula,data,subset,criterion="gcv",df=1.5,Kmin=1,Kmax=1e+06,smoother="k",kernel="g",method="eigen",rank=NULL,control.par=list(),cv.options=list()) {
    cl <- match.call() 
    mf <- match.call()
    m <- match(c("formula", "data", "subset"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf$na.action <- "na.fail"
    mf[[1L]] <- quote(stats::model.frame)
    mf <- eval(mf, parent.frame())
    mt <- attr(mf, "terms")
    attr(mt,"intercept") <- 0
    offset <- as.vector(model.offset(mf))
    if (!is.null(offset)) stop("offset are not allowed")
    if (is.empty.model(mt)) stop("no model ?")
    if (any(apply(mf,2,is.factor)))  stop("no factor are allowed")
    y <- model.response(mf, "numeric")
    x <- model.matrix(mt, mf)
    attributes(x) <- attributes(x)[c("dim","dimnames")]
    res <- ibr.fit(x,y,criterion,df,Kmin,Kmax,smoother,kernel,method,rank,control.par,cv.options)
    res$call <- cl
    res$terms <- mt
    class(res) <- c("ibr", "list")
   return(res)
}

iterchoiceA <- function(n,mini,maxi,eigenvaluesA,tPADmdemiY,DdemiPA,ddlmini,ddlmaxi,y,criterion,fraction){
  res <- list(minimum=0,objective=.Machine$double.xmax)
  if (criterion=="gmdl") {
    objectif <- .Machine$double.xmax
    fraction <- sort(unique(c(fraction[(fraction<maxi)&(fraction>mini)],mini,maxi)))
    dep <- length(fraction)
    repeat {
      mini <- fraction[dep-1]
      objectif <- critAgmdl(mini,valpr=eigenvaluesA,tPADmdemiY=tPADmdemiY,DdemiPA=DdemiPA,n=n,ddlmini=ddlmini,ddlmaxi=ddlmaxi,Y=y)
      if (objectif<.Machine$double.xmax) {
        bb <- fraction[dep]
        break 
      }
      dep <- dep-1
      if (dep==1) stop(paste("decrease Kmax below",fraction[2],"or increase dfmaxi component of control.par list"))
    }
    phi <- (sqrt(5) - 1)/2
    repeat {
      x1 <- mini + (1-phi)*(bb-mini)
      objectif <- critAgmdl(x1,valpr=eigenvaluesA,tPADmdemiY=tPADmdemiY,DdemiPA=DdemiPA,n=n,ddlmini=ddlmini,ddlmaxi=ddlmaxi,Y=y)
      if (objectif<.Machine$double.xmax) break else {
        bb <- x1
      }
    }
    fraction <- c(fraction[1:(dep-1)],bb)
    for (i in 1:(length(fraction)-1)) {
      res1 <- stats::optimize(critAgmdl,lower=fraction[i],upper=fraction[i+1],tol=0.5,valpr=eigenvaluesA,tPADmdemiY=tPADmdemiY,DdemiPA=DdemiPA,n=n,ddlmini=ddlmini,ddlmaxi=ddlmaxi,Y=y)
      if (res1$objective<res$objective) res <- res1
    }          
  } else {
    fcriterion <- switch(criterion,aic=critAaic,aicc=critAaicc,gcv=critAgcv,bic=critAbic)
    objectif <- .Machine$double.xmax
    fraction <- sort(unique(c(fraction[(fraction<maxi)&(fraction>mini)],mini,maxi)))
    dep <- length(fraction)
    repeat {
      mini <- fraction[dep-1]
      objectif <- fcriterion(mini,valpr=eigenvaluesA,tPADmdemiY=tPADmdemiY,DdemiPA=DdemiPA,n=n,ddlmini=ddlmini,ddlmaxi=ddlmaxi)
      if (objectif<.Machine$double.xmax) {
        bb <- fraction[dep]
        break 
      }
      dep <- dep-1
      if (dep==1) stop(paste("decrease Kmax below",fraction[2],"or increase dfmaxi component of control.par list"))
    }
    phi <- (sqrt(5) - 1)/2
    repeat {
      x1 <- mini + (1-phi)*(bb-mini)
      objectif <- fcriterion(x1,valpr=eigenvaluesA,tPADmdemiY=tPADmdemiY,DdemiPA=DdemiPA,n=n,ddlmini=ddlmini,ddlmaxi=ddlmaxi)
      if (objectif<.Machine$double.xmax) break else {
        bb <- x1
      }
    }
    fraction <- c(fraction[1:(dep-1)],bb)
    for (i in 1:(length(fraction)-1)) {
      res1 <- stats::optimize(fcriterion,lower=fraction[i],upper=fraction[i+1],tol=0.5,valpr=eigenvaluesA,tPADmdemiY=tPADmdemiY,DdemiPA=DdemiPA,n=n,ddlmini=ddlmini,ddlmaxi=ddlmaxi)
      if (res1$objective<res$objective) res <- res1
    }
  }
  return(list(iter=round(res$minimum),objective=res$objective))
}

iterchoiceAe <- function(Y,K,eigenvaluesA,tPADmdemiY,DdemiPA,ddlmini,ddlmaxi){
  n <- length(Y)
  ImLambda <- rep(0,n)
  ImLambdak <- rep(1,n)
  iter <- 1
  GCV <- rep(.Machine$double.xmax,length(K))
  AICc <- rep(.Machine$double.xmax,length(K))
  AIC <- rep(.Machine$double.xmax,length(K))
  BIC <- rep(.Machine$double.xmax,length(K))
  gMDL <- rep(.Machine$double.xmax,length(K))
  traceS <- rep(.Machine$double.xmax,length(K))
  sigma2list <- rep(.Machine$double.xmax,length(K))
  if (ddlmini>=1) {
    valpr0 <- eigenvaluesA[-(1:ddlmini)]
    ImLambda[-(1:ddlmini)] <- (1-valpr0)
  } else {
    ImLambda <- (1-eigenvaluesA)
  }   
  kcourant <- K[1]
  if (kcourant>1) {
    for (j in 1:(kcourant-1)){
      ImLambdak <- ImLambdak*ImLambda
    }
  }
  for (k in K) {
    while (kcourant!=k) {
      ImLambdak <- ImLambdak*ImLambda
      kcourant <- kcourant+1
    }
    ImLambdakp1 <- ImLambdak*ImLambda
    traceSk <- sum(1-ImLambdakp1)
    traceS[iter] <- traceSk
    sigma2 <- sum((DdemiPA%*%(ImLambdakp1*tPADmdemiY))^2)/n
    sigma2list[iter] <- sigma2
    logsigma2 <- log(sigma2)
    if ((sigma2<1e-10)|(traceSk>((1-1e-10)*n))) break
    GCV[iter] <-  logsigma2 - 2*log(1-traceSk/n) 
    AICc[iter] <-  logsigma2 + 1+ 2*(traceSk+1)/(n-traceSk-2)
    AIC[iter] <-  logsigma2 + 2*(traceSk)/(n)
    BIC[iter] <-  logsigma2 + log(n)*(traceSk)/(n)
    Sbul <- n*sigma2/(n-traceSk)
    gMDL[iter] <- log(Sbul)+traceSk/n*log((sum(Y^2)-n*sigma2)/(traceSk*Sbul))
    if (traceSk>ddlmaxi) break
    iter <- iter+1
  }
  return(list(aic=AIC,aicc=AICc,gcv=GCV,bic=BIC,gmdl=gMDL,df=traceS,sigma2=sigma2list,kmax=iter))
}

betaA <- function(n,eigenvaluesA,tPADmdemiY,DdemiPA,ddlmini,k,index0){
  prov <- rev(sumvalpr(k,n,rev(1-eigenvaluesA),n-index0+1,n-ddlmini+1))
  prov1 <- matrix(prov*as.vector(tPADmdemiY),n,1)
  return(DdemiPA%*%prov1)
}

sumvalpr <- function(k,n,valpr,index1,index0) {
  res <- rep(1,n)
  if (k>1) {
    n <- length(valpr)
    if (is.na(index1)&is.na(index0)) {
      res <- (1-valpr^k)/(1-valpr)
    } else {
      selecti <- NULL
      if (!is.na(index1)) selecti <- 1:index1
      if (!is.na(index0)) selecti <- c(selecti,index0:n)
      res[-selecti] <- (1-valpr[-selecti]^k)/(1-valpr[-selecti])
      if (!is.na(index1)) res[index1] <- k
      if (any(!is.finite(res[-selecti]))) {
        prov <- rep(1,length(valpr[-selecti]))
        kk <- 1
        while (kk <k) {
          prov <- prov+valpr[-selecti]^kk
          kk <- kk+1
        }
        res[-selecti] <- prov
      }
    }
  }
  return(res)
}

forward <- function(formula,data,subset,criterion="gcv",df=1.5,Kmin=1,Kmax=1e+06,smoother="k",kernel="g",rank=NULL,control.par=list(),cv.options=list(),varcrit=criterion) {
  cl <- match.call() 
  mf <- match.call()
  m <- match(c("formula", "data", "subset"), names(mf), 0L)
  mf <- mf[c(1L, m)]
  mf$drop.unused.levels <- TRUE
  mf$na.action <- "na.fail"
  mf[[1L]] <- quote(stats::model.frame)
  mf <- eval(mf, parent.frame())
  mt <- attr(mf, "terms")
  attr(mt,"intercept") <- 0
  offset <- as.vector(model.offset(mf))
  if (!is.null(offset)) stop("offset are not allowed")
  if (is.empty.model(mt)) stop("no model ?")
  if (any(apply(mf,2,is.factor)))  stop("no factor are allowed")
  y <- model.response(mf, "numeric")
  x <- model.matrix(mt, mf)
  attributes(x) <- attributes(x)[c("dim","dimnames")]
  crit <-c("aic","aicc","gcv","bic","gmdl","rmse","map")
  varcrit <- match.arg(varcrit,crit)
  garde <- NULL
  ptot <- ncol(x)
  possible <- 1:ptot
  resultat <- matrix(Inf,ptot,ptot)
  colnames(resultat) <- colnames(x)
  letop <- Inf
  bandwidth <- NULL
  if (!((any(names(control.par)=="bandwidth")&&is.numeric(control.par$bandwidth))|((any(names(control.par)=="dftotal"))&&control.par$dftotal))&(smoother=="k"))
    bandwidth <- rep(0,ptot)
  for (iter in 1:ptot) {
    for (i in possible) {
      garden <- sort(c(garde,i))
      if ((iter>1)&&(!((any(names(control.par)=="bandwidth")&&is.numeric(control.par$bandwidth))|((any(names(control.par)=="dftotal"))&&control.par$dftotal))&((smoother=="k")&(kernel!="g")))) res3 <- ibr.fit(x[,garden,drop=FALSE],y,criterion,df,Kmin,Kmax,smoother,kernel,rank,control.par=c(control.par,list(bandwidth=bandwidth[garden])),cv.options)   else
      res3 <- ibr.fit(x[,garden,drop=FALSE],y,criterion,df,Kmin,Kmax,smoother,kernel,rank,control.par,cv.options)
      if ((iter==1)&&(!((any(names(control.par)=="bandwidth")&&is.numeric(control.par$bandwidth))|((any(names(control.par)=="dftotal"))&&control.par$dftotal))&((smoother=="k")&(kernel!="g"))))
        bandwidth[i] <- res3$bandwidth
      if (any(names(control.par)=="exhaustive")&&control.par$exhaustive) {
        allcrit <- switch(varcrit,rmse=res3$criteria$rmse,map=res3$criteria$map,aic=res3$criteria$aic,aicc=res3$criteria$aicc,gcv=res3$criteria$gcv,bic=res3$criteria$bic,gmdl=res3$criteria$gmdl)
        allcrit <- allcrit[res3$iter]
      } else {
        if (criterion==varcrit) allcrit <- res3$criteria else
        allcrit <- summary(res3,varcrit)$criteria
      }
      resultat[iter,i] <- allcrit
    }
    mini <- which.min(resultat[iter,])
    letopnew <-min(resultat[iter,])
    if (letopnew>letop) {
      resultat <- resultat[1:(iter-1),]
      if (iter>1) {
        class(resultat) <- c("forwardibr", "matrix")
      } else {
        class(resultat) <- c("forwardibr", "vector")
      }
      return(resultat) } else letop <- letopnew
    coordonnee <- which(possible==mini)
    garde <- sort(c(garde,possible[coordonnee]))
    possible <- possible[-coordonnee]
  }
  if (iter>1) {
    class(resultat) <- c("forwardibr", "matrix")
  } else {
    class(resultat) <- c("forwardibr", "vector")
  }
  return(resultat)
}

fittedS1lr <- function(n,U,tUy,eigenvaluesS1,ddlmini,k,rank) {
    if (rank<n) valpr0 <- eigenvaluesS1[-c(1:ddlmini,(rank+1):n)] else
        valpr0 <- eigenvaluesS1[-(1:ddlmini)]
    valpr <- rep(1,rank)
    valpr[-(1:ddlmini)] <- 1-(1-valpr0)^k
    fk <- U%*%(valpr*tUy[1:rank])
    return(list(fit=as.vector(fk),trace=sum(valpr)))
}

summary.ibr <- function(object, criteria="call", ...) {
  r <- object$residuals
  tt <- terms(object)
  colnamesx <- delete.response(tt)
  n <- length(r)
  sigma2 <- sum(r^2)/(n)
  stderr <- sqrt(n*sum(r^2)/(n-object$finaldf))
  if (any(criteria=="call")) {
      criteria <- object$parcall$criterion
      if (any(criteria=="user")) {
          anscrit <- NULL
      } else  {
          if (object$parcall$exhaustive) {
              anscrit <- object$criteria
              } else {
                  anscrit <- object$allcriteria
                  names(anscrit) <- criteria
              }
      }
      if (object$parcall$exhaustive) {
          itercrit <- object$iter
      } else {
          itercrit <- object$alliter
      }
                
  } else {
    crit <-c("aic","aicc","gcv","bic","gmdl")
    if (all(!(criteria%in%crit))) stop(paste("criteria are:",crit,"\n"))
    criteria <- criteria[criteria%in%crit]
    anscrit <- NULL
    if (any(criteria=="gcv"))  anscrit <- c(anscrit,log(sigma2)-2*log(1-object$finaldf/n))
    if (any(criteria=="aic"))  anscrit <- c(anscrit,log(sigma2)+2*object$finaldf/n)
    if (any(criteria=="aicc"))  anscrit <- c(anscrit,log(sigma2)+1+(2*(object$finaldf+1))/(n-object$finaldf-2))
    if (any(criteria=="bic"))  anscrit <- c(anscrit,log(sigma2) + log(n)*(object$finaldf)/n)
    if (any(criteria=="gmdl")) {
        Sbul <-   n*sigma2/(n-object$finaldf)
        mf <- object$call
        m <- match(c("formula", "data", "subset"), names(mf), 0L)
        mf <- mf[c(1L, m)]
        mf$drop.unused.levels <- TRUE
        mf[[1L]] <- quote(stats::model.frame)
        mf <- eval(mf, environment(tt))
        y <- model.response(mf, "numeric")
        anscrit <- c(anscrit,log(Sbul)+object$finaldf/n*log((sum(y^2)-n*sigma2)/(object$finaldf*Sbul)))
    }
    criteria <- criteria[criteria!="user"]
    names(anscrit) <- criteria
    itercrit <- rep(NA,length(anscrit))
    names(itercrit) <- criteria
    if ((all(criteria!="user"))&&(any(criteria %in%object$parcall$criterion))) {
      itercrit[criteria%in%object$parcall$criterion] <- object$alliter[object$parcall$criterion%in%criteria]
      names(itercrit)[criteria%in%object$parcall$criterion] <- object$parcall$criterion[object$parcall$criterion%in%criteria]
    }
  }
  if (object$parcall$critmethod=="aggregation") crit4iter <- paste("aggregation of:",paste(object$parcall$criterion,collapse=", ")) else crit4iter <- object$parcall$criterion[1]
  ans <- list(residuals=r,Std.Error=stderr,Initial.Df=object$initialdf,
              Final.Df=object$finaldf,Resid.Df=n-object$finaldf,criteria=anscrit,
              iterations=itercrit,kernel=object$parcall$kernel, iter=object$iter,
              crit4iter=crit4iter,bandwidth=object$bandwidth,
              smoother=object$parcall$smoother,m=object$parcall$m,s=object$parcall$s)
  if (object$parcall$smoother=="k") {
    if (is.null(colnamesx)) {
      names(ans$bandwidth) <- paste("X",1:object$parcall$p,sep="")
    } else {
      names(ans$bandwidth) <- paste(colnamesx,1:object$parcall$p,sep="")
    }
  }
  class(ans) <- "summary.ibr"
  ans
}

ibr.fit <- function(x,y,criterion="gcv",df=1.5,Kmin=1,Kmax=1e+06,smoother="k",kernel="g",method="eigen",rank=NULL,control.par=list(),cv.options=list()) {
  cl <- match.call() 
  crit <- c("aic","aicc","gcv","bic","gmdl","rmse","map")
  crite <- pmatch(criterion,crit)
  if (any(is.na(crite))) stop(paste("parameter criterion must be in",paste(crit,collapse=", "))) else criterion <- crit[crite]
  methode <- c("eigen", "product")
  nummethod <- pmatch(method, methode)
  if (any(is.na(nummethod))) stop(paste("parameter method must be in",paste(methode,collapse=", "))) else method <- methode[nummethod]
  if ((any(crite>5))&&(length(crite)>2)) stop("RMSE or MAP must be used without any other criterion")
  iterautre <- NULL
  choixkautre <- NULL
  ##  criterion <- match.arg(criterion,crit)
  smoothertable <- c("k","tps","ds","lrtps","lrds")
  smoother <- match.arg(smoother,smoothertable)
  lowrank <- ifelse(substr(smoother,0,2)=="lr",TRUE,FALSE)
  if ((lowrank)&(method!="eigen"))  {
    warning("For lowrank splines only \"eigen\" method (one need to diagonalize to get smoother)")
    method <- "eigen"
  }
  if (!is.matrix(x)) {
    x <- data.matrix(x)
    warning("x is coerced to matrix by data.matrix function\n")
  }
  n <- nrow(x)
  p <- ncol(x)
  if (length(y)!=n) stop("number of observations in x and y do not match\n")
  if (smoother=="k") contr.sp <- list(bandwidth=NULL,iter=NULL,really.big=FALSE,
                                      dftobwitmax=1000,exhaustive=FALSE,m=NULL,s=NULL,dftotal=FALSE,
                                      accuracy=0.01,dfmaxi=2*n/3,fraction=c(100, 200, 500, 1000, 5000,10^4,5e+04,1e+05,5e+05,1e+06),
                                      scale=FALSE,aggregcrit="no",aggregfun=function(x) {floor(stats::median(x[criterion]))}, verbose=FALSE)
  else  contr.sp <- list(bandwidth=NULL,iter=NULL,really.big=FALSE,
                         dftobwitmax=1000,exhaustive=FALSE,m=NULL,s=NULL,dftotal=FALSE,
                         accuracy=0.01,dfmaxi=2*n/3,fraction=c(100, 200, 500, 1000, 5000,10^4,5e+04,1e+05,5e+05,1e+06),
                         scale=TRUE,aggregcrit="no",aggregfun=function(x) {floor(stats::median(x[criterion]))}, verbose=FALSE)
  contr.sp[(names(control.par))] <- control.par
  if (!(is.logical(contr.sp$verbose))) stop("contr.sp$verbose must be logical\n")
  if (!(is.logical(contr.sp$dftotal))) stop("contr.sp$dftotal must be logical\n")
  if ((!is.null(contr.sp$bandwidth))&(!is.numeric(contr.sp$bandwidth) || any(contr.sp$bandwidth<0))) stop("invalid bandwidth\n")
  if ((!is.null(contr.sp$iter))&(!is.numeric(contr.sp$iter) || (contr.sp$iter<0) || (floor(contr.sp$iter)!=contr.sp$iter))) stop("invalid number of iterations\n")
  iter <- contr.sp$iter
  if ((contr.sp$dfmaxi<=0)|(contr.sp$dfmaxi>n)) stop("invalid dfmaxi\n")
  crit <-c("recalc","no","aggregation")
  crite <- pmatch(contr.sp$aggregcrit,crit)
  if (is.na(crite)) stop(paste("control.par$criterion must be in",crit))
  if (length(criterion)==2&&(all(criterion%in%c("map","rmse")))&&(!contr.sp$exhaustive)) stop("when RMSE and MAP are both selected, exhaustive search (in control.par list) must be chosen")
  if ((smoother=="tps")|(smoother=="lrtps")) {
    contr.sp$s <- 0
    if (is.null(contr.sp$m)) contr.sp$m <- floor(p/2)+1 else {
                                                          if (contr.sp$m<=(p/2)) stop("order of thin plate splines is invalid (need to be greater than p/2)\n")
                                                        }
    if (!is.numeric(contr.sp$m) || (contr.sp$m<0) || (floor(contr.sp$m)!=contr.sp$m)) stop("invalid spline order\n")
    ddlmin <- choose(contr.sp$m+p-1,contr.sp$m-1)
    if (ddlmin>=n) stop(paste("ddl min is equal to",ddlmin,"and the number observations is",n))
  }
  if (smoother=="k") {
    contr.sp$m <- NULL
  }
  smoothobject <- NULL
  if ((smoother=="ds")|(smoother=="lrds")) {
    if (is.null(contr.sp$m)) contr.sp$m <- 2 ## default penalty order 2
    if (is.null(contr.sp$s)) contr.sp$s <- (p-1)/2 ## default pseudo cubic
    if ((!is.numeric(contr.sp$m))|(!is.numeric(contr.sp$s))) stop("contr.par$m or contr.par$s is not numeric...\n")
    contr.sp$m <- round(contr.sp$m)     ## m is integer
    contr.sp$s <- round(contr.sp$s*2)/2 ## s is in halfs
    if (contr.sp$m< 1) contr.sp$m <- 1  ## m > 0
    ## check that -p/2 < s < p/2...
    if (contr.sp$s >= p/2) { 
      contr.sp$s <- (p-1)/2
      warning("contr.par$s value reduced")
    } 
    if (contr.sp$s <= -p/2) { 
      contr.sp$s <- -(p-1)/2
      warning("contr.par$s value increased")
    }
    
    ## m + s > p/2 for continuity...
    if ((contr.sp$m+contr.sp$s)<=p/2) {
      contr.sp$s <- 1/2 + p/2 - contr.sp$m
      if (contr.sp$s>=p/2) stop("No suitable contr.par$s try increasing contr.par$m")
      warning("contr.par$s value modified to give continuous function")
    }
    ddlmin <- choose(contr.sp$m+p-1,contr.sp$m-1)
    if (ddlmin>=n) stop(paste("ddl min is equal to",ddlmin,"and the number observations is",n))
  }
  
  if (lowrank) {
    if (is.null(rank)) stop("rank argument for lowrank splines must be chosen...")
    if (rank>n) stop(paste("rank argument must be less than",n))
    bs <- substr(smoother,3,4)
    listvarx <- colnames(x)
  } 
  moy <- NULL
  ec <- NULL
  if (contr.sp$scale) {
    if (smoother=="k") warning("when using kernel smoother, you do not need to scale\n")
    if (lowrank) ec <- apply(x,2, sd)*sqrt((n-1)/n) else  ec <- apply(x,2, sd)
    x <- scale(x,scale=ec)
    moy <- attr(x,"scaled:center")
  }
  if (all(criterion%in%c("rmse","map"))) {
    cv <- list(bwchange=FALSE,ntest=floor(nrow(x)/10),ntrain=NULL,Kfold=FALSE,type="random",seed=NULL,npermut=20)
    cv[(names(cv.options))] <- cv.options
    if (!all(sapply(cv[1],is.logical))) stop("invalid cv$bwchange or cv$Kfold: must be logical\n")
    if (!all(sapply(cv[c(2,3,6,8)], FUN=function(x) is.numeric(x)||is.null(x)))) stop("invalid cv parameters: must be numeric or NULL\n")
    if (any(names(cv.options)=="ntrain")) cv$ntest <- NULL
  } else cv <- NULL
  if (!(lowrank)&&((method=="eigen")&(n>5000)&(! contr.sp$really.big))) stop("number of observations is greater than 1000, set control.par$really.big to TRUE if you really want to do the requested calculations (but computational time -eigen decomposition- could be prohibitive)\n")
  if (smoother=="k") {
    kern <- c("g", "e", "q", "u")
    kernel <- match.arg(kernel,kern)
    kernelint <- which(kernel==kern)
    m <- NULL
    if (!is.null(contr.sp$bandwidth)) {
      if (length(contr.sp$bandwidth)==1) {
        bandwidth <- rep(contr.sp$bandwidth,p)
      } else {
        if (length(contr.sp$bandwidth)!=p)  stop(paste("the length of bandwidth vector have to be",p,"or 1\n"))
        bandwidth <- contr.sp$bandwidth
      }
    } else {
      if (df<=1) stop("degree of freedom should be greater than 1\n")   
      departbw <- apply(x,2,FUN=function(z) 3*abs(diff(range(z))))
      if (contr.sp$verbose) print("Choosing bandwidth")
      bandwidth <- .Call(ibr_choosebw,
                         as.double(departbw),
                         if (contr.sp$dftotal) as.double(rep(1e-10,p)) else
                                                                         as.double(1e-10),
                         as.double(x),
                         c(as.double(.Machine$double.eps^0.25),
                           as.double(df)),
                         as.integer(c(n, p, contr.sp$dftobwitmax,
                                      contr.sp$dftotal, kernelint))
                         )
    }
    if (contr.sp$verbose) print("Smoother Matrices")
    K <- .Call(ibr_Kmatrix, x, 0, bandwidth, as.integer(c(n, p, n, kernelint, 1)))
    listeS <- .Call(ibr_Amatrix, K)
    listeS[[3]] <- sum(diag(K)/listeS[[2]])
    listeS[[2]] <- 1/sqrt(listeS[[2]])
    names(listeS) <- c("S", "Ddemi", "df")
    rm(K)
    dfstart <- listeS$df
  } # end kernel
  if ((smoother=="tps")|(smoother=="ds")|lowrank) {
    bandwidth <- contr.sp$bandwidth
    if (length(df)>1) stop("only one df is possible with Splines\n")
    ddlmini <- choose(contr.sp$m+p-1,contr.sp$m-1)
    if (lowrank) {
     if (contr.sp$verbose) print("LowRank setup and eigen calculations")
     if (is.null(bandwidth)) {
        lambda <- lambdachoicelr(x,ddlmini*df,m=contr.sp$m,contr.sp$s,rank,itermax=contr.sp$dftobwitmax,bs,listvarx)
        bandwidth <- lambda
      } else {
        lambda <- bandwidth
      }
      S2 <- lrsmoother(x,bs,listvarx,lambda=lambda,m=contr.sp$m,s=contr.sp$s,rank)
    } else {
      if (is.null(bandwidth)) {
        lambda <- lambdachoice(x,ddlmini*df,m=contr.sp$m,contr.sp$s,itermax=contr.sp$dftobwitmax,smoother)
        bandwidth <- lambda
      } else {
        lambda <- bandwidth
      }
      listeS <- dssmoother(x, y,lambda=lambda,m=contr.sp$m,s=contr.sp$s)
      dfstart <- sum(diag(listeS$S))
    }
  }
  if (method=="eigen") {
    if (lowrank) {
      ## diagonalization already done
      eigenvaluesS <- S2$values
      dfstart <- sum(eigenvaluesS)
      ## almost 0 eigenvalues
      if (any(zapsmall(eigenvaluesS,digits=9)==0)) {
        index0 <-  which(zapsmall(eigenvaluesS,digits=9)==0)[1]
      } else index0 <- rank+1
      eigenvaluesS <- c(eigenvaluesS,rep(0,n-rank))
      eigenvaluesS[eigenvaluesS<0] <- 0
      U <- S2$vectors
      Rm1U <- S2$Rm1U
      tUy <- c(as.vector(crossprod(U, y)),rep(0,n-rank))
      smoothobject <- S2$smoothobject
      rm(S2)
    } else {
      ## diagonalization
     if (contr.sp$verbose) print("Eigen calculations")
      listeS.eig <- eigen(listeS$S,symmetric=TRUE)
      eigenvaluesS <- listeS.eig$values
      if (any(eigenvaluesS<(-1e-10))) stop("Some eigenvalues of the Kernel smoother matrix are negative, it will explode")
      if (smoother=="k") {
        tPADmdemiY <- t(listeS.eig$vectors*(1/listeS$Ddemi))%*%y
        DdemiPA <- (listeS$Ddemi*listeS.eig$vectors)
      } else {
        U <- listeS.eig$vectors
        tUy <- as.vector(crossprod(U, y))
      }
      ## ddlmini for kernel
      if (smoother=="k"){
        if (any(zapsmall(eigenvaluesS-1,digits=9)==0)) {
          ddlmini <-  sum(zapsmall(eigenvaluesS-1,digits=9)==0)
        } else ddlmini <- max(eigenvaluesS)
      }
      ## almost 0 eigenvalues
      if (any(zapsmall(eigenvaluesS,digits=9)==0)) {
        index0 <-  which(zapsmall(eigenvaluesS,digits=9)==0)[1]
      } else index0 <- NA   
      rm(listeS.eig)  
    }
    if (is.null(iter)) {
     if (contr.sp$verbose) print("Calculating the number of iterations")
      if (Kmax<=Kmin) stop("Kmax hould be greater than Kmin\n")
      if (any(c(Kmin,Kmax)<=0)) stop("Kmin and Kmax should be greater than 0")
      if (all(criterion%in%c("rmse","map"))) {
        ## iter k chosen by CV
        if (cv$bwchange) {
          if (is.null(df)) stop("df needs to be set\n")
          if (smoother=="k") bx <- NULL else lambda <- NULL
        } else {
          if (smoother=="k") bx <- bandwidth
        }
        if (contr.sp$exhaustive) {
          ## exhaustive
          if (smoother=="k") {
            choixkautre <- iterchoiceAcve(x,y,bx,df,kernel,ddlmini,cv$ntest,
                                          cv$ntrain,cv$Kfold,cv$type,cv$npermut,
                                          cv$seed,Kmin,Kmax)
          } else {
            if (lowrank)     {
              choixkautre <- iterchoiceS1lrcve(x,y,lambda,rank,bs,listvarx,df,ddlmini,cv$ntest,cv$ntrain,cv$Kfold,cv$type,cv$npermut,cv$seed,Kmin,Kmax,contr.sp$m,contr.sp$s)
            } else {
              choixkautre <- iterchoiceS1cve(x,y,lambda,df,ddlmini,cv$ntest,cv$ntrain,cv$Kfold,cv$type,cv$npermut,cv$seed,Kmin,Kmax,contr.sp$m,contr.sp$s)
            }
          }
          iterautre <-  (Kmin:Kmax)[unlist(lapply(choixkautre[1:5],FUN=which.min))]
          names(iterautre) <-  names(choixkautre[1:5])
          iter <- switch(contr.sp$aggregcrit,
                         no=iterautre[criterion[1]],
                         aggregation=contr.sp$aggregfun(iterautre),
                         recalc= { Kmax2 <- stats::median(10^(floor(log10(iterautre))+2)) ;
                           iterautre[1] <- (Kmin:Kmax2)[which.min(choixkautre[[criterion[1]]])] ;
                           iterautre[1] })
          if (contr.sp$aggregcrit=="aggregation") {
            choixk <- NA
            names(choixk) <- "aggregation"
          } else {
            choixk <- switch(criterion[1],aic=choixkautre$aic[iter],aicc=choixkautre$aicc[iter],gcv=choixkautre$gcv[iter],bic=choixkautre$bic[iter],gmdl=choixkautre$gmdl[iter])
            names(choixk) <- criterion[1]
          }
        } else {
          ## optimize 
          if (smoother=="k") {
            prov <- iterchoiceAcv(x,y,bx,df,kernel,ddlmini,cv$ntest,cv$ntrain,cv$Kfold,cv$type,cv$npermut,cv$seed,Kmin,Kmax,criterion,contr.sp$fraction)
          } else {
            if (lowrank)     {
              prov <- iterchoiceS1lrcv(x,y,lambda,rank,bs,listvarx,df,ddlmini,cv$ntest,cv$ntrain,cv$Kfold,cv$type,cv$npermut,cv$seed,Kmin,Kmax,criterion,contr.sp$m,contr.sp$s,contr.sp$fraction)
            } else {
              prov <- iterchoiceS1cv(x,y,lambda,df,ddlmini,cv$ntest,cv$ntrain,cv$Kfold,cv$type,cv$npermut,cv$seed,Kmin,Kmax,criterion,contr.sp$m,contr.sp$s,contr.sp$fraction)
            }
          }
          iter <- prov$iter
          choixkautre <- prov$objective
          names(choixk) <- criterion[1]
          if (criterion=="rmse") choixk <- sqrt(choixk)
        }
      } else {
        ## iter k chosen by AIC/BIC etc
        if (contr.sp$exhaustive) {
          ## exhaustive
          if (smoother=="k") {
            choixkautre <- iterchoiceAe(y,Kmin:Kmax,eigenvaluesS,tPADmdemiY,DdemiPA,ddlmini,contr.sp$dfmaxi)
            
          } else {
            choixkautre <- iterchoiceS1e(y,Kmin:Kmax,tUy,eigenvaluesS,ddlmini,contr.sp$dfmaxi)
          }
          iterautre <- (Kmin:Kmax)[unlist(lapply(choixkautre[c("aic","aicc","gcv","bic","gmdl")],FUN=which.min))]
          names(iterautre) <-  names(choixkautre[1:5])
          iter <- switch(contr.sp$aggregcrit,
                         no=iterautre[ criterion[1] ],
                         aggregation=contr.sp$aggregfun(iterautre),
                         recalc= { Kmax2 <- stats::median(10^(floor(log10(iterautre))+2)) ;
                           iterautre[criterion[1]] <- (Kmin:Kmax2)[ which.min(choixkautre[[ criterion[1] ]])] ;
                           iterautre[criterion[1]] })
          if (contr.sp$aggregcrit=="aggregation") {
            choixk <- NA
            names(choixk) <- "aggregation"
          } else {
            choixk <- switch(criterion[1],aic=choixkautre$aic[iter],aicc=choixkautre$aicc[iter],gcv=choixkautre$gcv[iter],bic=choixkautre$bic[iter],gmdl=choixkautre$gmdl[iter])
          }
          ## 
        } else {
          ## optimize
          if (smoother=="k") {
            prov <- iterchoiceA(n,Kmin,Kmax,eigenvaluesS,tPADmdemiY,DdemiPA,ddlmini,contr.sp$dfmaxi,y,criterion[1],contr.sp$fraction)
          }  else {
            prov <- iterchoiceS1(n,Kmin,Kmax,tUy,eigenvaluesS,ddlmini,contr.sp$dfmaxi,y,criterion[1],contr.sp$fraction)
          }
          iter <- prov$iter
          choixk <- prov$objective
          names(choixk) <- criterion[1]
          iterautre <- iter
          choixkautre <- prov$objective
          if (length(criterion)>1) {
            ## several criteria
            for (i in 2:length(criterion)) {
              if (smoother=="k") {
                prov <- iterchoiceA(n,Kmin,Kmax,eigenvaluesS,tPADmdemiY,DdemiPA,ddlmini,contr.sp$dfmaxi,y,criterion[i],contr.sp$fraction)
              } else {
                prov <- iterchoiceS1(n,Kmin,Kmax,tUy,eigenvaluesS,ddlmini,contr.sp$dfmaxi,y,criterion[i],contr.sp$fraction)
              }
              iterautre <- c(iterautre,prov$iter)
              choixkautre <- c(choixkautre,prov$objective)
            }
            names(iterautre) <- criterion
            iter <- switch(contr.sp$aggregcrit,no=iterautre[1],
                           aggregation={
                             choixk <-  "aggregation"
                             contr.sp$aggregfun(iterautre) }, recalc= {
                               Kmax2 <- stats::median(10^(floor(log10(iterautre))+2))
                               if (smoother=="k") {
                                 prov <- iterchoiceA(n,Kmin,Kmax2,eigenvaluesS,tPADmdemiY,DdemiPA,ddlmini,contr.sp$dfmaxi,y,criterion[1],contr.sp$fraction)
                               } else {
                                 prov <- iterchoiceS1(n,Kmin,Kmax2,tUy,eigenvaluesS,ddlmini,contr.sp$dfmaxi,y,criterion[1],contr.sp$fraction)
                               }
                               choixk <- prov$objective
                               iterautre[1] <- prov$iter
                               prov$iter }) 
          } ## several criteria
        } ## exhaustive or optimize
      } ## iter K chosen by CV or Not
      if ((((Kmax-iter)/(Kmax-Kmin)<1e-5)|(Kmax-iter)<3)&(!contr.sp$exhaustive)) warning(paste("Number of iterations is chosen close to the boundary of grid search: ",Kmax,".\n  Increase the maximum number of iterations or use contr.sp$exhaustive search\n",sep=""))
      if ((iter==max(Kmax))&(contr.sp$exhaustive)) warning(paste("Number of iterations is chosen at the boundary of grid search: ",Kmax,".\n  Increase the maximum number of iterations\n",sep=""))
    } else {
      criterion <- "user"
      choixk <- NULL
    } ## iter K chosen by crit or user
  } else {
    ## product
    if (Kmin!=1) {
      Kmin <- 1
      warning("Kmin must be one for product: Kmin is set to 1 and proceed")
    }
    if (smoother=="k") {
      Ddemiy <- y/listeS[[2]]
    } else {
      Ddemiy <- y
    }
    if (!is.null(iter)) {
      if (contr.sp$verbose) print("Product/reccurence until convergence")
      ## prod until iter
      res <- .Call(ibr_product, listeS$S, as.double(Ddemiy), as.integer(iter))
      criterion <- "user"
      choixk <- NULL
      choixkautre <- NULL
      beta <- res[[1]]
      finaldf <- res[[2]]
    } else {
      if(all(criterion%in%c("aic","aicc","bic","gcv"))) {
        prov <- c("aic","aicc","bic","gcv") %in% criterion
        if (any(prov)) { 
          critsint <- which(prov)
          ## if (length(which(prov))>1) warning("only one criterion for method=\"product\"")
        } else {
          stop("Criterion must be in \"aic\",\"aicc\",\"gcv\",\"bic\" for \"product\" method")
        }
      } else {
        stop("criterion must be in aic, aicc, bic, gcv")
      }
      if (smoother=="k") {
        res <- .Call(ibr_productandcrit, listeS$S, as.double(Ddemiy),
                     as.integer(critsint),
                     as.double(listeS$Ddemi),
                     as.integer(c(Kmax, ifelse(smoother=="k", 0, 1))))
      } else {
        res <- .Call(ibr_productandcrit, listeS$S, as.double(Ddemiy),
                     as.integer(critsint),
                     as.double(0.0),
                     as.integer(c(Kmax, ifelse(smoother=="k", 0, 1))))
      }
      if (length(criterion)>1) { 
        myorder <- order(criterion)
        betaautre <- matrix(res[[1]], n, length(criterion))[, myorder, drop=FALSE]
        beta <-  betaautre[,1]
        iterautre <- res[[4]][myorder]
        names(iterautre) <- criterion
        choixkautre <- res[[3]][myorder]
        finaldf <-  res[[2]][myorder]
        if (contr.sp$aggregcrit=="no") {
          iter <- iterautre[1]
          myname <- criterion[1]
          choixk <- c(myname = choixkautre[1])
          finaldf <- finaldf[1]
        }
        if (contr.sp$aggregcrit=="aggregation") {
          iter <- contr.sp$aggregfun(iterautre)
          res <- .Call(ibr_product, listeS$S, as.double(Ddemiy), as.integer(iter))
          choixk <- c(aggregation = NA)
          finaldf <- res[[2]]
        }
        if (contr.sp$aggregcrit=="recalc") {
          Kmax2 <- stats::median(10^(floor(log10(iterautre))+2))
          prov <- (c("aic","aicc","bic","gcv")==criterion[1])
          critsint <- which(prov)
          res <- .Call(ibr_productandcrit, listeS$S, as.double(Ddemiy),
                       as.integer(critsint),
                       as.double(0.0),
                       as.integer(c(Kmax2, ifelse(smoother=="k", 0, 1))))
          beta <-  res[[1]]
          iterautre <- res[[4]]
          choixkautre <- res[[3]]
          names(choixkautre) <- criterion
          choixk <- choixkautre
          iter <- iterautre
          finaldf <- res[[2]]
        }
      } else {
        # one crit only
        beta <-  res[[1]]
        iterautre <- res[[4]]
        choixkautre <- res[[3]]
        names(choixkautre) <- criterion
        choixk <- choixkautre
        iter <- iterautre
        finaldf <- res[[2]]
      }
    }
  } # end of product
  ## beta
  if (contr.sp$verbose) print("Finalizing...")
  if (method=="eigen") {
    if (smoother=="k") {
      beta <- betaA(n,eigenvaluesS,tPADmdemiY,DdemiPA,ddlmini,k=iter,index0)
      listefit <- fittedA(n,eigenvaluesS,tPADmdemiY,DdemiPA,ddlmini,k=iter)
    } else {
      if (lowrank) {
        beta <- betaS1lr(n,U,tUy,eigenvaluesS,ddlmini,iter,lambda,rank,Rm1U,index0)
        listefit <- fittedS1lr(n,U,tUy,eigenvaluesS,ddlmini,iter,rank)
        if (listefit$trace>=0.99*rank) warning(paste("Final rank is chosen equal to the lowrank of spline: ",rank,".\n  Increase the rank argument\n",sep=""))
      } else {
        listebeta <- betaS1(n,U,tUy,eigenvaluesS,ddlmini,iter,lambda,listeS$Sgu,listeS$Qgu,index0)
        beta <- list(d=listebeta$dgub,c=listebeta$cgub)
        listefit <- fittedS1(n,U,tUy,eigenvaluesS,ddlmini,iter)
      }
    }
    finaldf <- listefit$trace
    fitted <- listefit$fit
  } else {
    if (smoother=="k") {
      fitted <- matrix(rep(listeS$Ddemi, n)*listeS$S,n,n)%*%beta
      beta <- listeS$Ddemi*beta
      } else {
       fitted <- listeS$S%*%beta
      }
  }
  residuals <- y - fitted
  res <- list(beta=beta,residuals=residuals,fitted=fitted,iter=iter,initialdf=dfstart,
              finaldf=finaldf,bandwidth=bandwidth,call=cl,
              parcall=list(p=p,m=contr.sp$m,s=contr.sp$s,scaled=contr.sp$scale,mean=moy,sd=ec,critmethod=contr.sp$aggregcrit,method=method,
                           rank=rank,criterion=criterion,smoother = smoother, kernel = kernel,smoothobject=smoothobject,exhaustive=contr.sp$exhaustive),
              criteria=choixk,alliter=iterautre,allcriteria=choixkautre)    
  return(res)
}

dssmoother <- function(X,Y=NULL,lambda,m,s) {
  n <- nrow(X)
  d <- ncol(X)
  Sgu <-  DuchonS(X,m)
  qrSgu <- qr(Sgu)
  F2 <- qr.Q(qrSgu,complete=TRUE)[,-(1:ncol(Sgu))]
  Qgu <- DuchonQ(X,0,m,s,symmetric=TRUE)
  ainv <- t(F2)%*%Qgu%*%F2
  diag(ainv) <- diag(ainv)+lambda
  Sp <- -lambda*F2%*%(solve(ainv))%*%t(F2)
  if (! is.null(Y)) {
    cgu <- as.vector(Sp%*%Y)/(-lambda)
    dgu <- solve(qr.R(qrSgu))%*%(t(qr.Q(qrSgu))%*%(as.matrix(Y)-Qgu%*%cgu))
    diag(Sp) <- rep(1,n)+diag(Sp)
    return(list(S=Sp,Sgu=Sgu,Qgu=Qgu,dgu=dgu,cgu=cgu))
  }  else {
    diag(Sp) <- rep(1,n)+diag(Sp)
    return(list(S=Sp,Sgu=Sgu,Qgu=Qgu,dgu=NULL,cgu=NULL))
  }
}

betaS1 <- function(n,U,tUy,eigenvaluesS1,ddlmini,k,lambda,Sgu,Qgu,index0) {
  prov <- rev(sumvalpr(k,n,rev(1-eigenvaluesS1),n-index0+1,n-ddlmini+1))
  beta <- U%*%(prov*tUy)
  qrSgu <- qr(Sgu)
  F2 <- qr.Q(qrSgu,complete=TRUE)[,-(1:ncol(Sgu))]
  ainv <- t(F2)%*%Qgu%*%F2
  diag(ainv) <- diag(ainv)+lambda
  Sp <- -lambda*F2%*%(solve(ainv))%*%t(F2)
  cgubeta <- as.vector(Sp%*%beta)/(-lambda)
  dgubeta <- solve(qr.R(qrSgu))%*%(t(qr.Q(qrSgu))%*%(as.matrix(beta)-Qgu%*%cgubeta))
  return(list(dgub=dgubeta,cgub=cgubeta))
}

iterchoiceS1cv <- function(X,y,lambda,df,ddlmini,ntest,ntrain,Kfold,type,npermut,seed,Kmin,Kmax,criterion,m,s,fraction) {
  choixssecv2 <- function(k,sel,y,lambdalist,U,S1,valpr,SSx,tUy,Sp,ddlmin,index0) {
    sse <- 0
    for (j in 1:length(sel)) {
      if (attr(sel,"type")=="timeseries") {
        nj <- length(y[-(sel[[j]][1]:n)])
      } else {
        nj <- length(y[-sel[[j]]])
      }
      sommerk <- rev(sumvalpr(k,nj,rev(valpr[[j]]),nj-index0[j]+1,nj-ddlmin[j]+1))
      beta <- U[[j]]%*%(sommerk*tUy[[j]])
      cgubeta <- as.vector(Sp[[j]]%*%beta)/(-lambdalist[[j]])
      dgubeta <- preprod[[j]]%*%(as.matrix(beta)-(S1[[j]])$Qgu%*%cgubeta)
      Yrescv <- as.vector((SSx[[j]])$Sgu%*%dgubeta+(SSx[[j]])$Qgu%*%cgubeta)
      sse <- sse+sum((y[sel[[j]]]-Yrescv)^2)
    }
    return(sse)
  }
  choixsapcv2 <- function(k,sel,y,lambdalist,U,S1,valpr,SSx,tUy,Sp,ddlmin,index0) {
    sap <- 0
    for (j in 1:length(sel)) {
      if (attr(sel,"type")=="timeseries") {
        nj <- length(y[-(sel[[j]][1]:n)])
      } else {
        nj <- length(y[-sel[[j]]])
      }
      sommerk <- rev(sumvalpr(k,nj,rev(valpr[[j]]),nj-index0[j]+1,nj-ddlmin[j]+1))
      beta <- U[[j]]%*%(sommerk*tUy[[j]])
      cgubeta <- as.vector(Sp[[j]]%*%beta)/(-lambdalist[[j]])
      dgubeta <- preprod[[j]]%*%(as.matrix(beta)-(S1[[j]])$Qgu%*%cgubeta)
      Yrescv <- as.vector((SSx[[j]])$Sgu%*%dgubeta+(SSx[[j]])$Qgu%*%cgubeta)
      sap <- sap+sum(abs((y[sel[[j]]]-Yrescv)/y[sel[[j]]]))
    }
    return(sap)
  }
  
  n <- nrow(X)
  sel <- cvobs(n,ntest,ntrain,Kfold,type,npermut,seed)
  res <- list(minimum=0,objective=.Machine$double.xmax)
  objectif <- .Machine$double.xmax
  fraction <- sort(unique(c(fraction[(fraction<Kmax)&(fraction>Kmin)],Kmin,Kmax)))
  dep <- length(fraction)
  U <- as.list(rep(0,length(sel)))
  S1 <- valpr <- tUy <- Sp <- lambdalist <- preprod <- SSx <- U
  ddlmin <- index0 <- rep(0,length(sel))
  for (j in 1:length(sel)) {
    if (attr(sel,"type")=="timeseries") {
      XA <- X[-(sel[[j]][1]:n),,drop=FALSE]
      YA <- y[-(sel[[j]][1]:n)]
    } else  {
      XA <- X[-sel[[j]],,drop=FALSE]
      YA <- y[-sel[[j]]]
    }  
    if (is.null(lambda)&(!is.null(df))) {
      lambdalist[[j]] <- lambdachoice(XA,ddlmini*df,m=m,s=s,itermax=100)
    } else lambdalist[[j]] <- lambda
    S1[[j]] <- dssmoother(XA, YA,lambda=lambdalist[[j]],m=m,s=s) 
    vp1.S1 <- eigen(S1[[j]]$H,symmetric=TRUE)
    U[[j]] <- vp1.S1$vect
    tUy[[j]] <- as.vector(crossprod(U[[j]], YA))
    if (any(zapsmall(vp1.S1$values-1,digits=9)==0)) {
      ddlmin[j] <-  sum(zapsmall(vp1.S1$values-1,digits=9)==0)
    } else ddlmin[j] <- NA
    if (any(zapsmall(vp1.S1$values,digits=9)==0)) {
      index0[j] <-  which(zapsmall(vp1.S1$values,digits=9)==0)[1]
    } else index0[j] <- NA
    valpr[[j]] <- 1-vp1.S1$values
    qrSgu <- qr(S1[[j]]$Sgu)
    F2 <- qr.Q(qrSgu,complete=TRUE)[,-(1:ncol(S1[[j]]$Sgu))]
    ainv <- t(F2)%*%S1[[j]]$Qgu%*%F2
    diag(ainv) <- diag(ainv)+lambdalist[[j]]
    Sp[[j]] <- -lambda*F2%*%(solve(ainv))%*%t(F2)
    SSx[[j]] <- dsSx(X=XA,X[sel[[j]],,drop=FALSE],m,s)
    preprod[[j]] <- solve(qr.R(qrSgu))%*%(t(qr.Q(qrSgu)))
  }
  if (criterion=="rmse") {
    repeat {
      mini <- fraction[dep-1]
      objectif <- choixssecv2(mini,sel=sel,y=y,lambdalist=lambdalist,U=U,S1=S1,valpr=valpr,SSx=SSx,tUy=tUy,Sp=Sp,ddlmin=ddlmin,index0=index0)
      if (objectif<.Machine$double.xmax) {
        bb <- fraction[dep]
        break 
      }
      dep <- dep-1
      if (dep==1) stop(paste("decrease Kmax below",fraction[2]))
    }
    phi <- (sqrt(5) - 1)/2
    repeat {
      x1 <- mini + (1-phi)*(bb-mini)
      objectif <- choixssecv2(x1,sel=sel,y=y,lambdalist=lambdalist,U=U,S1=S1,valpr=valpr,SSx=SSx,tUy=tUy,Sp=Sp,ddlmin=ddlmin,index0=index0)
      if (objectif<.Machine$double.xmax) break else {
        bb <- x1
      }
    }
    fraction <- c(fraction[1:(dep-1)],bb)
    for (i in 1:(length(fraction)-1)) {
      res1 <- stats::optimize(choixssecv2,lower=fraction[i],upper=fraction[i+1],tol=0.5,sel=sel,y=y,lambdalist=lambdalist,U=U,S1=S1,valpr=valpr,SSx=SSx,tUy=tUy,Sp=Sp,ddlmin=ddlmin,index0=index0)
      if (res1$objective<res$objective) res <- res1
    }
  } else {
    repeat {
      mini <- fraction[dep-1]
      objectif <- choixsapcv2(mini,sel=sel,y=y,lambdalist=lambdalist,U=U,S1=S1,valpr=valpr,SSx=SSx,tUy=tUy,Sp=Sp,ddlmin=ddlmin,index0=index0)
      if (objectif<.Machine$double.xmax) {
        bb <- fraction[dep]
        break 
      }
      dep <- dep-1
      if (dep==1) stop(paste("decrease Kmax below",fraction[2]))
    }
    phi <- (sqrt(5) - 1)/2
    repeat {
      x1 <- mini + (1-phi)*(bb-mini)
      objectif <- choixsapcv2(x1,sel=sel,y=y,lambdalist=lambdalist,U=U,S1=S1,valpr=valpr,SSx=SSx,tUy=tUy,Sp=Sp,ddlmin=ddlmin,index0=index0)
      if (objectif<.Machine$double.xmax) break else {
        bb <- x1
      }
    }
    fraction <- c(fraction[1:(dep-1)],bb)
    for (i in 1:(length(fraction)-1)) {
      res1 <- stats::optimize(choixsapcv2,lower=fraction[i],upper=fraction[i+1],tol=0.5,sel=sel,y=y,lambdalist=lambdalist,U=U,S1=S1,valpr=valpr,SSx=SSx,tUy=tUy,Sp=Sp,ddlmin=ddlmin,index0=index0)
      if (res1$objective<res$objective) res <- res1
    }
  }
  return(list(iter=round(res$minimum),objective=res$objective/sum(unlist(lapply(sel,length)))))
}

bwchoice <- function(X,objectif,kernelx="g",itermax=1000) {
  p <- ncol(as.matrix(X))
  res <- rep(0,p)
  if (length(objectif)==1) objectif <- rep(objectif,p)
  if (any(objectif<=1)) stop("degree of freedom should be greater than 1\n")
  choixddlparvar <- function(fenetre,X,objectif,kernel) {
    H <- kernelSx(X, NULL, fenetre, kernel)
    trace <- sum(diag(H))
    res <- trace-objectif
    return(res)
  }
  for (j in 1:ncol(X)) {
    depart <- 3*abs(diff(range(X[,j])))
    if (choixddlparvar(depart,X[,j],objectif[j],kernelx)>0) {
      while (choixddlparvar(depart,X[,j],objectif[j],kernelx)>0) {
        depart <- depart*2
      }
    }
    res[j] <- stats::uniroot(choixddlparvar,interval=c(depart,1e-10),X=X[,j],objectif=objectif[j],maxiter=itermax,kernel=kernelx)$root
  }
  return(res)
}

iterchoiceAcve <- function(X,y,bx,df=NULL,kernelx,ddlmini,ntest,ntrain,Kfold,type,npermut,seed,Kmin,Kmax) {
  iter <- 1
  MSE <- MAP <- rep(.Machine$double.xmax,length(Kmax-Kmin+1))
  n <- nrow(X)
  sel <- cvobs(n,ntest,ntrain,Kfold,type,npermut,seed)
    tPADmdemiY <- as.list(rep(0,length(sel)))
    sumvalprk <- valpr <- DdemiPA <- SSx <- tPADmdemiY
    nj <- rep(0,length(sel))
     for (j in 1:length(sel)) {
      if (attr(sel,"type")=="timeseries") {
        XA <- X[-(sel[[j]][1]:n),,drop=FALSE]
        YA <- y[-(sel[[j]][1]:n)]
      } else  {
        XA <- X[-sel[[j]],,drop=FALSE]
        YA <- y[-sel[[j]]]
      }  
       nj[j] <- length(YA)
      if (is.null(bx)&(!is.null(df))) {
        bx <- bwchoice(X=XA,objectif=df,kernelx,itermax=100)
      }
      listeA <- calcA(X=XA,bx=bx,kernelx=kernelx)
      listeA.eig <- eigen(listeA$A,symmetric=TRUE)
      tPADmdemiY[[j]] <- t(listeA.eig$vectors*(1/listeA$Ddemi))%*%YA
      ddlmini <- sum(zapsmall(listeA.eig$values-1)==0)
      valpr[[j]] <- rep(0,nj[j])
      sumvalprk[[j]] <- rep(1,nj[j])
      if (ddlmini>=1) {
        valpr[[j]][-(1:ddlmini)] <- 1-listeA.eig$values[-(1:ddlmini)]
      } else {
        valpr[[j]] <- 1-listeA.eig$values
      }
      DdemiPA[[j]] <- (listeA$Ddemi*listeA.eig$vectors)
      SSx[[j]] <- kernelSx(kernelx=kernelx,X=XA,bx=bx,X[sel[[j]],,drop=FALSE])
    }
    if (Kmin>1) {
      for (k in 1:(Kmin-1)){
        for (j in 1:length(sel)) {
          sumvalprk[[j]] <- sumvalprk[[j]]+valpr[[j]]^k
        }
      }
    }
    for (k in Kmin:Kmax) {
      sse <- 0
      sap <- 0
      for (j in 1:length(sel)) {
        prov1 <- matrix(sumvalprk[[j]]*as.vector(tPADmdemiY[[j]]),nj[j],1)
        Yrescv <- SSx[[j]]%*%DdemiPA[[j]]%*%prov1
        sse <- sse+sum((y[sel[[j]]]-Yrescv)^2)
        sap <- sap+sum(abs((y[sel[[j]]]-Yrescv)/y[sel[[j]]]))
        sumvalprk[[j]] <- sumvalprk[[j]]+valpr[[j]]^k
      }
      MSE[iter] <- sse/sum(n-nj)
      MAP[iter] <- sap/sum(n-nj)
      iter <- iter+1
    }
  return(list(rmse=sqrt(MSE),map=MAP))
}     

AICc <- function(object, ...) UseMethod("AICc")
GCV <- function(object, ...) UseMethod("GCV")

GCV.ibr <- function(object, ...) {
  r <- object$residuals
  n <- length(r)
  stderr <- sqrt(sum(r^2)/(n))
  return(log(stderr^2)-2*log(1-object$finaldf/n))
}
AICc.ibr <- function(object, ...) {
  r <- object$residuals
  n <- length(r)
  stderr <- sqrt(sum(r^2)/(n))
  return(log(stderr^2)+1+(2*(object$finaldf+1))/(n-object$finaldf-2))
}

plot.forwardibr <- function(x,global=FALSE,...) {
  hasylab <- function(...) !all(is.na(pmatch(names(list(...)),"ylab")))
  hasxlab <- function(...) !all(is.na(pmatch(names(list(...)),"xlab")))
  if (is.null(dim(x))) return("Only one variable is retained") else {
    if (!global) {
      maxx <- apply(x,1,function(x) max(x[is.finite(x)]) )
      minn <- apply(x,1,function(x) min(x[is.finite(x)]) )
      x <- (x-matrix(minn,ncol=ncol(x),nrow=nrow(x)))/matrix(maxx,ncol=ncol(x),nrow=nrow(x))
    }
    x[!is.finite(x)] <- max(x[is.finite(x)])+0.1*diff(range(x[is.finite(x)]))
    if (hasylab(...)&hasxlab(...))
      
      image(1:ncol(x),1:nrow(x),t(x),...) else {
        if (hasxlab(...)) {
          image(1:ncol(x),1:nrow(x),t(x),ylab="Number of variables",...) }
        else {
          image(1:ncol(x),1:nrow(x),t(x),xlab="Variables",ylab="Number of variables",...) }
      }
    coordmin <- apply(x,1,which.min)
     text(coordmin,1:ncol(x),paste("V",1:ncol(x),sep=""))
    abline(h=seq(1.5,nrow(x)-0.5,by=1),col="grey10")
    abline(v=seq(1.5,ncol(x)-0.5,by=1),col="grey10")
  invisible(NULL)
  }
}

fittedA <-  function(n,eigenvaluesA,tPADmdemiY,DdemiPA,ddlmini,k){
  ## premier test
  prov <- rep(1,n)
  if (ddlmini>=1) {
    valpr0 <- 1-eigenvaluesA[-(1:ddlmini)]
    prov[-(1:ddlmini)] <- 1-valpr0^k
  } else {
    valpr0 <- 1-eigenvaluesA
    prov <- 1-valpr0^k
  }
  trace <- sum(prov)
  prov1 <- matrix(prov*as.vector(tPADmdemiY),n,1)
  return(list(fit=as.vector(DdemiPA%*%prov1),trace=trace))
}

\name{BIC}
\alias{BIC}
\alias{GCV}
\alias{AICc}
\alias{BIC.ibr}
\alias{GCV.ibr}
\alias{AICc.ibr}

\title{Information Criterion for ibr}

\description{
  Functions calculating the  Bayesian Informative Criterion , the
  Generalized Cross Validation criterion and the Corrected Akaike information criterion.
}

\usage{
\method{BIC}{ibr}(object, \dots)

\method{GCV}{ibr}(object, \dots)

\method{AICc}{ibr}(object, \dots)}

\arguments{
  \item{object}{A fitted model object of class ibr.}
  \item{\dots}{Only for compatibility purpose with \code{BIC} of
  \code{nlme} package.}
}
\details{
  The ibr method for \code{BIC}, \code{BIC.ibr()} calculates
  \eqn{\log(sigma^2)+log(n)*df/n}{log(sigma^2)+log(n)*df/n}, where \emph{df} is the trace
  of the smoother.

The ibr method for \code{GCV}, \code{GCV.ibr()} calculates
  \eqn{\log(sigma^2)-2*\log(1-df/n)}{log(sigma^2)-log(1-*df/n)}
  
The ibr method for \code{AICc}, \code{AICc.ibr()} calculates
  \eqn{\log(sigma^2)+1+(2*(df+1))/(n-df-2)}{log(sigma^2)+1+(2*(df+1))/(n-df-2)}.
  
}
\value{
  Returns a numeric value
  with the corresponding BIC, GCV or AICc.
}
\author{Pierre-Andre Cornillon, Nicolas Hengartner and Eric Matzner-Lober.}

\references{
Hurvich, C. M., Simonoff J. S. and Tsai, C. L. (1998) Smoothing
Parameter Selection in Nonparametric Regression Using an Improved Akaike
Information Criterion. \emph{Journal of the Royal Statistical Society, Series B}, 60, 271-293 . 
}
\seealso{\code{\link{ibr}}, \code{\link{summary.ibr}}}

\examples{
\dontrun{data(ozone, package = "ibr")
res.ibr <- ibr(ozone[,-1],ozone[,1])
BIC(res.ibr)
GCV(res.ibr)
AICc(res.ibr)
}
}

\keyword{smooth}
\keyword{multivariate}

\name{iterchoiceS1lrcve}

\alias{iterchoiceS1lrcve}

\title{Selection of the number of iterations for iterative bias
  reduction smoothers with base lowrank thin-plate splines smoother or duchon splines smoother}

\description{
Evaluates at each iteration proposed in the grid the cross-validated
root mean squared error (RMSE) and mean of the relative absolute error (MAP). The minimum of these
criteria gives an estimate of the optimal number of iterations.
This function is not intended to be used directly.
}

\usage{
iterchoiceS1lrcve(X, y, lambda, rank, bs, listvarx, df, ddlmini, ntest, ntrain,
Kfold, type, npermut, seed, Kmin, Kmax, m, s)}

\arguments{
  \item{X}{A numeric matrix of explanatory variables, with \emph{n} rows
  and \emph{p} columns.}
  \item{y}{A numeric vector of variable to be explained of length \emph{n}.}
  \item{lambda}{A numeric positive coefficient that governs the
  amount of penalty (coefficient lambda).}
 \item{rank}{The rank of lowrank splines.}
 \item{bs}{The type rank of lowrank splines: \code{tps} or \code{ds}.}
 \item{listvarx}{The vector of the names of explanatory variables}
  \item{df}{A numeric vector of length 1 which is multiplied by the minimum df of thin
  plate splines ; This argument is useless if
  \code{lambda} is supplied (non null).}
  \item{ddlmini}{The number of eigenvalues equals to 1.}
  \item{ntest}{The number of observations in test set.}
  \item{ntrain}{The number of observations in training set.}
  \item{Kfold}{Either the number of folds or a boolean or \code{NULL}.}
  \item{type}{A character string in
  \code{random},\code{timeseries},\code{consecutive}, \code{interleaved}
  and give the type of segments.}
  \item{npermut}{The number of random draw (with replacement), used for
    \code{type="random"}.}
  \item{seed}{Controls the seed of random generator
    (via \code{\link{set.seed}}).}
  \item{Kmin}{The minimum number of bias correction  iterations of the
  search grid considered by
  the model selection procedure for selecting the optimal number of iterations.}
  \item{Kmax}{The maximum number of bias correction  iterations of the
  search grid considered by
  the model selection procedure for selecting the optimal number of iterations.}
  \item{m}{The order of derivatives for the penalty (for thin plate
  splines it is the order). This integer \emph{m} must verify
  2\emph{m}+2\emph{s}/\emph{d}>1, where \emph{d} is the number of explanatory
  variables.} 
  \item{s}{The power of weighting function. For thin plate splines
  \emph{s} is equal to 0. This real must be strictly smaller than \emph{d}/2
  (where \emph{d} is the number of explanatory  variables) and must
  verify 2\emph{m}+2\emph{s}/\emph{d}. To get pseudo-cubic splines,
  choose \emph{m}=2 and \emph{s}=(\emph{d}-1)/2 (See Duchon).} 
}

\value{
Returns the values of RMSE and MAP for each
value of the grid \code{K}. \code{Inf} are returned if the iteration leads
to a smoother with a df bigger than \code{ddlmaxi}.}

\references{   Cornillon, P.-A.; Hengartner, N.; Jegou, N. and Matzner-Lober, E. (2012)
  Iterative bias reduction: a comparative study.
\emph{Statistics and Computing}, \emph{23}, 777-791.



  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2013)
  Recursive bias estimation for multivariate regression smoothers Recursive 
bias estimation for multivariate regression smoothers.
\emph{ESAIM: Probability and Statistics}, \emph{18}, 483-502.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2017)
  Iterative Bias Reduction Multivariate Smoothing in R: The ibr Package.
\emph{Journal of Statistical Software}, \emph{77}, 1--26.

Duchon, J. (1977) Splines minimizing rotation-invariant semi-norms in
Solobev spaces. in W. Shemp and K. Zeller (eds) \emph{Construction theory of
  functions of several variables}, 85-100, Springer, Berlin.

Wood, S.N. (2003) Thin plate regression
  splines. \emph{J. R. Statist. Soc. B}, \emph{65}, 95-114.
}

\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober.}

\seealso{\code{\link{ibr}}}


\keyword{smooth}
\keyword{multivariate}

\name{iterchoiceS1}

\alias{iterchoiceS1}
\alias{critS1gcv}
\alias{critS1aic}
\alias{critS1bic}
\alias{critS1aicc}
\alias{critS1gmdl}

\title{Number of iterations selection for iterative bias reduction model}

\description{
The function \code{iterchoiceS1} searches the interval from \code{mini} to
     \code{maxi} for a minimum of the function which calculates the chosen
\code{criterion} (\code{critS1gcv}, \code{critS1aic}, \code{critS1bic},
\code{critS1aicc} or \code{critS1gmdl})  with respect to its first
argument (a given iteration \code{k}) using \code{\link[stats:optimize]{stats::optimize}}. This function is not intended to be used directly.
}

\usage{
iterchoiceS1(n, mini, maxi, tUy, eigenvaluesS1, ddlmini, ddlmaxi,
y, criterion, fraction)}

\arguments{
  \item{n}{The number of observations.}
  \item{mini}{The lower end point of the interval to be searched.}
  \item{maxi}{The upper end point of the interval to be searched.}
  \item{eigenvaluesS1}{Vector of the eigenvalues of the
  symmetric smoothing matrix \emph{S}.}
  \item{tUy}{The transpose of the matrix of eigen vectors of the
  symmetric smoothing matrix \emph{S} times the vector of observation \emph{y}.}
  \item{ddlmini}{The number of eigen values of \emph{S} equal to 1.}
  \item{ddlmaxi}{The  maximum df. No criterion is calculated and
  \code{Inf} is returned.}
  \item{y}{The vector of observations of dependant variable.}
  \item{criterion}{The criteria available are GCV (default, \code{"gcv"}),
  AIC (\code{"aic"}), corrected AIC  (\code{"aicc"}),   BIC
  (\code{"bic"}) or gMDL  (\code{"gmdl"}).}
  \item{fraction}{The subdivision of the interval [\code{mini},\code{maxi}].}
}

\details{The interval [\code{mini},\code{maxi}] is splitted into
  subintervals using \code{fraction}. In each subinterval the function
  \code{fcriterion} is minimzed  using \code{\link[stats:optimize]{stats::optimize}} (with respect
     to its first argument) and the minimum (and its argument) of the
  result of these optimizations is returned.}

\value{
A list with components \code{iter} and \code{objective} which give the
(rounded) optimum number of iterations (between
\code{Kmin} and \code{Kmax}) and the value
     of the function at that real point (not rounded).}

\references{
  Cornillon, P.-A.; Hengartner, N.; Jegou, N. and Matzner-Lober, E. (2012)
  Iterative bias reduction: a comparative study.
\emph{Statistics and Computing}, \emph{23}, 777-791.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2013)
  Recursive bias estimation for multivariate regression smoothers Recursive 
bias estimation for multivariate regression smoothers.
\emph{ESAIM: Probability and Statistics}, \emph{18}, 483-502.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2017)
  Iterative Bias Reduction Multivariate Smoothing in R: The ibr Package.
\emph{Journal of Statistical Software}, \emph{77}, 1--26.
}

\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober}

\seealso{\code{\link{ibr}}, \code{\link{iterchoiceS1}}}


\keyword{smooth}
\keyword{multivariate}

\name{betaS1lr}

\alias{betaS1lr}

\title{Coefficients for iterative bias reduction method. }

\description{The function evaluates the smoothing matrix \code{H}, the
  matrices \emph{Q} and \emph{S} and their associated
  coefficients \code{c} and \code{s}. This function is not intended to be used directly.}

\usage{
betaS1lr(n,U,tUy,eigenvaluesS1,ddlmini,k,lambda,rank,Rm1U,index0)
}

\arguments{
  \item{n}{The number of observations.}
  \item{U}{The the matrix of eigen vectors of the
  symmetric smoothing matrix \emph{S}.}
  \item{tUy}{The transpose of the matrix of eigen vectors of the
  symmetric smoothing matrix \emph{S} times the vector of observation \emph{y}.}
  \item{eigenvaluesS1}{Vector of the eigenvalues of the
  symmetric smoothing matrix \emph{S}.}
  \item{ddlmini}{The number of eigen values of \emph{S} equal to 1.}
  \item{k}{A numeric vector which give the number of iterations.}
 \item{lambda}{The smoothness coefficient lambda for thin plate splines of
  order \code{m}.}
 \item{rank}{The rank of lowrank splines.}
 \item{Rm1U}{matrix \eqn{R^{-1}U}{R^{-1}U} (see reference).}
  \item{index0}{The index of the first eigen values of \emph{S}
  numerically equal to 0.}
 }
\details{See the reference for detailed explanation of \emph{Q} (the
  semi kernel or radial basis) and
  \emph{S} (the polynomial null space).}
 
\references{
Wood, S.N. (2003) Thin plate regression
  splines. \emph{J. R. Statist. Soc. B}, \emph{65}, 95-114.
} 
\value{
Returns \code{beta}}

\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober}

\seealso{\code{\link{ibr}}}


\keyword{smooth}

\name{iterchoiceAcve}

\alias{iterchoiceAcve}

\title{Selection of the number of iterations for iterative bias reduction smoothers}

\description{
Evaluates at each iteration proposed in the grid the cross-validated
root mean squared error (RMSE) and mean of the relative absolute error (MAP). The minimum of these
criteria gives an estimate of the optimal number of iterations.
This function is not intended to be used directly.
}

\usage{
iterchoiceAcve(X, y, bx, df, kernelx, ddlmini, ntest, ntrain,
Kfold, type, npermut, seed, Kmin, Kmax)}

\arguments{
  \item{X}{A numeric matrix of explanatory variables, with \emph{n} rows
  and \emph{p} columns.}
  \item{y}{A numeric vector of variable to be explained of length \emph{n}.}
  \item{bx}{The vector of different bandwidths, length \eqn{p}.}
  \item{df}{A numeric vector of either length 1 or length equal to the
    number of columns of \code{x}. If \code{smoother="k"}, it indicates
    the  desired effective degree of
  freedom (trace) of the smoothing   matrix for
    each variable ; \code{df} is repeated when the length of vector
  \code{df} is 1. This argument is useless if
  \code{bandwidth} is supplied (non null).}
  \item{kernelx}{Character string which allows to choose between gaussian kernel
  (\code{"g"}), Epanechnikov (\code{"e"}), uniform (\code{"u"}),
  quartic (\code{"q"}). The default (gaussian kernel) is strongly advised.}
  \item{ddlmini}{The number of eigenvalues (numerically) equals to 1.}
  \item{ntest}{The number of observations in test set.}
  \item{ntrain}{The number of observations in training set.}
  \item{Kfold}{Either the number of folds or a boolean or \code{NULL}.}
  \item{type}{A character string in
  \code{random},\code{timeseries},\code{consecutive}, \code{interleaved}
  and give the type of segments.}
  \item{npermut}{The number of random draw (with replacement), used for
    \code{type="random"}.}
  \item{seed}{Controls the seed of random generator
    (via \code{\link{set.seed}}).}
  \item{Kmin}{The minimum number of bias correction  iterations of the
  search grid considered by
  the model selection procedure for selecting the optimal number of iterations.}
  \item{Kmax}{The maximum number of bias correction  iterations of the
  search grid considered by
  the model selection procedure for selecting the optimal number of iterations.}
}

\value{
Returns the values of RMSE and MAP for each
value of the grid \code{K}. \code{Inf} are returned if the iteration leads
to a smoother with a df bigger than \code{ddlmaxi}.}

\references{
  Cornillon, P.-A.; Hengartner, N.; Jegou, N. and Matzner-Lober, E. (2012)
  Iterative bias reduction: a comparative study.
\emph{Statistics and Computing}, \emph{23}, 777-791.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2013)
  Recursive bias estimation for multivariate regression smoothers Recursive 
bias estimation for multivariate regression smoothers.
\emph{ESAIM: Probability and Statistics}, \emph{18}, 483-502.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2017)
  Iterative Bias Reduction Multivariate Smoothing in R: The ibr Package.
\emph{Journal of Statistical Software}, \emph{77}, 1--26.
}
\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober.}

\seealso{\code{\link{ibr}}}


\keyword{smooth}
\keyword{multivariate}

\name{DuchonQ}

\alias{DuchonQ}

\title{Computes the semi-kernel of Duchon splines}

\description{
The function \code{DuchonQ} computes the semi-kernel of Duchon splines. This function is not intended to be used directly.
}

\usage{
DuchonQ(x,xk,m=2,s=0,symmetric=TRUE)}

\arguments{
  \item{x}{A numeric matrix of explanatory variables, with \emph{n} rows
  and \emph{p} columns.}
  \item{xk}{A numeric matrix of explanatory variables, with \emph{nk} rows
  and \emph{p} columns.}
  \item{m}{Order of derivatives.}
  \item{s}{Exponent for the weight function.}
  \item{symmetric}{Boolean: if \code{TRUE} only \code{x} is used and it
  computes the semi-kernel at observations of \code{x} (it should give the
  same result as \code{DuchonQ(x,xk,m,s,FALSE)}).}
}

\value{
The semi-kernel evaluated.}

\references{
Duchon, J. (1977) Splines minimizing rotation-invariant semi-norms in
Solobev spaces. in W. Shemp and K. Zeller (eds) \emph{Construction theory of
functions of several variables}, 85-100, Springer, Berlin. }

\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober.}

\seealso{\code{\link{ibr}}}


\keyword{smooth}
\keyword{multivariate}

\name{betaA}

\alias{betaA}

\title{Calculates coefficients for iterative bias reduction smoothers}

\description{
Calculates the coefficients for the iterative bias reduction  smoothers. This function is not intended to be used directly.}

\usage{
betaA(n, eigenvaluesA, tPADmdemiY, DdemiPA, ddlmini, k, index0)}

\arguments{
  \item{n}{The number of observations.}
  \item{eigenvaluesA}{Vector of the eigenvalues of the
  symmetric matrix \emph{A}.}
  \item{tPADmdemiY}{The transpose of the matrix of eigen vectors of the
  symmetric matrix \emph{A} times the inverse of the square root of the diagonal matrix \emph{D}.}
  \item{DdemiPA}{The square root of the diagonal matrix \emph{D} times
  the eigen vectors of the symmetric matrix \emph{A}.}
  \item{ddlmini}{The number of eigenvalues (numerically) equals to 1.}
  \item{k}{A scalar which gives the number of iterations.}
  \item{index0}{The index of the first eigen values of \emph{S}
  numerically equal to 0.}
}
\details{See the reference for detailed explanation of \emph{A} and
  \emph{D} and the meaning of coefficients.}

\value{
Returns the  vector of coefficients (of length \emph{n}, the number of observations.)}

\references{
  Cornillon, P.-A.; Hengartner, N.; Jegou, N. and Matzner-Lober, E. (2012)
  Iterative bias reduction: a comparative study.
\emph{Statistics and Computing}, \emph{23}, 777-791.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2013)
  Recursive bias estimation for multivariate regression smoothers Recursive 
bias estimation for multivariate regression smoothers.
\emph{ESAIM: Probability and Statistics}, \emph{18}, 483-502.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2017)
  Iterative Bias Reduction Multivariate Smoothing in R: The ibr Package.
\emph{Journal of Statistical Software}, \emph{77}, 1--26.
}
\author{Pierre-Andre Cornillon, Nicolas Hengartner and Eric Matzner-Lober.}

\seealso{\code{\link{ibr}}}


\keyword{smooth}
\keyword{multivariate}

\name{fittedS1lr}

\alias{fittedS1lr}

\title{Evaluate the fit for iterative bias reduction model}

\description{The function evaluates the fit for iterative bias reduction
  model for iteration \code{k}. This function is not intended to be used directly.}

\usage{
fittedS1lr(n,U,tUy,eigenvaluesS1,ddlmini,k,rank)
}

\arguments{
  \item{n}{The number of observations.}
  \item{U}{The the matrix of eigen vectors of the
  symmetric smoothing matrix \emph{S}.}
  \item{tUy}{The transpose of the matrix of eigen vectors of the
  symmetric smoothing matrix \emph{S} times the vector of observation \emph{y}.}
  \item{eigenvaluesS1}{Vector of the eigenvalues of the
  symmetric smoothing matrix \emph{S}.}
  \item{ddlmini}{The number of eigen values of \emph{S} equal to 1.}
  \item{k}{A numeric vector which gives the number of iterations}
 \item{rank}{The rank of lowrank splines.}
}
 
\references{
  Cornillon, P.-A.; Hengartner, N.; Jegou, N. and Matzner-Lober, E. (2012)
  Iterative bias reduction: a comparative study.
\emph{Statistics and Computing}, \emph{23}, 777-791.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2013)
  Recursive bias estimation for multivariate regression smoothers Recursive 
bias estimation for multivariate regression smoothers.
\emph{ESAIM: Probability and Statistics}, \emph{18}, 483-502.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2017)
  Iterative Bias Reduction Multivariate Smoothing in R: The ibr Package.
\emph{Journal of Statistical Software}, \emph{77}, 1--26.

Wood, S.N. (2003) Thin plate regression
  splines. \emph{J. R. Statist. Soc. B}, \emph{65}, 95-114.
}
\details{see the reference for detailed explanation of computation of iterative bias reduction smoother}

\value{
Returns a vector containing the fit}

\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober}

\seealso{\code{\link{ibr}}}


\keyword{smooth}

\name{fittedA}

\alias{fittedA}

\title{Evaluates the fits for iterative bias reduction method}

\description{
Evaluates the fits for the iterative bias reduction smoother, using a kernel
smoother and its decomposition into a symmetric matrix and a diagonal
matrix. This function is not intended to be used directly.
}

\usage{
fittedA(n, eigenvaluesA, tPADmdemiY, DdemiPA, ddlmini, k)}

\arguments{
  \item{n}{The number of observations.}
  \item{eigenvaluesA}{Vector of the eigenvalues of the
  symmetric matrix \emph{A}.}
  \item{tPADmdemiY}{The transpose of the matrix of eigen vectors of the
  symmetric matrix \emph{A} times the inverse of the square root of the diagonal matrix \emph{D}.}
  \item{DdemiPA}{The square root of the diagonal matrix \emph{D} times
  the eigen vectors of the symmetric matrix \emph{A}.}
  \item{ddlmini}{The number of eigenvalues (numerically) equals to 1.}
  \item{k}{A scalar which gives the number of iterations.}
}
\details{See the reference for detailed explanation of \emph{A} and \emph{D}.}

\value{
Returns a list of two components: \code{fitted} contains fitted values
and \code{trace} contains the trace (effective degree of freedom) of the iterated
bias reduction smoother.}

\references{
  Cornillon, P.-A.; Hengartner, N.; Jegou, N. and Matzner-Lober, E. (2012)
  Iterative bias reduction: a comparative study.
\emph{Statistics and Computing}, \emph{23}, 777-791.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2013)
  Recursive bias estimation for multivariate regression smoothers Recursive 
bias estimation for multivariate regression smoothers.
\emph{ESAIM: Probability and Statistics}, \emph{18}, 483-502.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2017)
  Iterative Bias Reduction Multivariate Smoothing in R: The ibr Package.
\emph{Journal of Statistical Software}, \emph{77}, 1--26.
}

\author{Pierre-Andre Cornillon, Nicolas Hengartner and Eric Matzner-Lober.}

\seealso{\code{\link{ibr}}}


\keyword{smooth}
\keyword{multivariate}

\name{predict.npregress}

\alias{predict.npregress}

\title{Predicted values using using local polynomials

}

\description{
  Predicted values from a local polynomials of degree less than 2. See
   \code{\link[KernSmooth]{locpoly}} for fast binned implementation
  over an equally-spaced grid of local polynomial (gaussian kernel only) 
  \cr
 Missing values are not allowed.
}

\usage{
\method{predict}{npregress}(object, newdata, interval=
 c("none", "confidence", "prediction"), deriv=FALSE, \dots)}

\arguments{
  \item{object}{Object of class \code{\link{npregress}}.}
  \item{newdata}{An optional vector of values to be predicted. If omitted, the fitted values are used.}
  \item{interval}{Type of interval calculation. Only \code{none} is
  currently avalaible.}
  \item{deriv}{Bolean. If \code{TRUE} it returns the first derivative of
  the local  polynomial (of degree1).}
  \item{\dots}{Further arguments passed to or from other methods.}
}

\value{ Produces a vector of predictions. If \code{deriv}
  is \code{TRUE} the value is a named list with components: \code{yhat}
  which contains predictions and (if relevant) \code{deriv} the
  first derivative of the local polynomial of degree 1.}

\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober.}

\references{
Wand, M. P. and Jones, M. C. (1995). \emph{Kernel Smoothing}. Chapman and Hall, London.}

\seealso{\code{\link{npregress}}, \code{\link{summary.npregress}},
  \code{\link[KernSmooth]{locpoly}}
}

\examples{
f <- function(x){sin(5*pi*x)}
n <- 100
x <- runif(n)
z <- f(x)
sigma2 <- 0.05*var(z)
erreur<-rnorm(n,0,sqrt(sigma2))
y<-z+erreur
grid <- seq(min(x),max(x),length=500)
res <- npregress(x,y,bandwidth=0.02,control.par=list(degree=1))
plot(x,y)
lines(grid,predict(res,grid))
}

\keyword{smooth}
\keyword{multivariate}

\name{sumvalpr}

\alias{sumvalpr}

\title{Sum of a geometric series}

\description{
Calculates the sum of the first (k+1) terms of a geometric series with initial term 1
  and common ratio equal to \code{valpr} (lower or equal to 1).}

\usage{
sumvalpr(k,n,valpr,index1,index0)}

\arguments{
  \item{k}{The number of terms minus 1.}
  \item{n}{The length of \code{valpr}.}
  \item{valpr}{Vector of common ratio in decreasing order.}
  \item{index1}{The index of the last common ratio equal to 1.}
  \item{index0}{The index of the first common ratio equal to 0.}
}

\value{
Returns the vector of the sums of the first (k+1) terms of the geometric series.}

\references{
  Cornillon, P.-A.; Hengartner, N.; Jegou, N. and Matzner-Lober, E. (2012)
  Iterative bias reduction: a comparative study.
\emph{Statistics and Computing}, \emph{23}, 777-791.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2013)
  Recursive bias estimation for multivariate regression smoothers Recursive 
bias estimation for multivariate regression smoothers.
\emph{ESAIM: Probability and Statistics}, \emph{18}, 483-502.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2017)
  Iterative Bias Reduction Multivariate Smoothing in R: The ibr Package.
\emph{Journal of Statistical Software}, \emph{77}, 1--26.
}

\author{Pierre-Andre Cornillon, Nicolas Hengartner and Eric Matzner-Lober.}

\seealso{\code{\link{ibr}}}


\keyword{smooth}
\keyword{multivariate}

\name{predict.ibr}

\alias{predict.ibr}

\title{Predicted values using iterative bias reduction smoothers}

\description{
  Predicted values from iterative bias reduction object.
  \cr
 Missing values are not allowed.
}

\usage{
\method{predict}{ibr}(object, newdata, interval=
 c("none", "confidence", "prediction"), \dots)}

\arguments{
  \item{object}{Object of class \code{\link{ibr}}.}
  \item{newdata}{An optional matrix in which to look for variables with which to predict. If omitted, the fitted values are used.}
  \item{interval}{Type of interval calculation. Only \code{none} is
  currently avalaible.}
  \item{\dots}{Further arguments passed to or from other methods.}
}

\value{
  Produces a vector of predictions.}

\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober.}

\references{
  Cornillon, P.-A.; Hengartner, N.; Jegou, N. and Matzner-Lober, E. (2012)
  Iterative bias reduction: a comparative study.
\emph{Statistics and Computing}, \emph{23}, 777-791.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2013)
  Recursive bias estimation for multivariate regression smoothers Recursive 
bias estimation for multivariate regression smoothers.
\emph{ESAIM: Probability and Statistics}, \emph{18}, 483-502.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2017)
  Iterative Bias Reduction Multivariate Smoothing in R: The ibr Package.
\emph{Journal of Statistical Software}, \emph{77}, 1--26.
}

\seealso{\code{\link{ibr}}, \code{\link{summary.ibr}}}

\examples{
\dontrun{data(ozone, package = "ibr")
res.ibr <- ibr(ozone[,-1],ozone[,1],df=1.2,K=1:500)
summary(res.ibr)
predict(res.ibr)}
}

\keyword{smooth}
\keyword{multivariate}

\name{cvobs}

\alias{cvobs}

\title{Selection of the number of iterations for iterative bias reduction smoothers}

\description{
The function \code{cvobs} gives the index of observations in each test set. This function is not intended to be used directly.
}

\usage{
cvobs(n,ntest,ntrain,Kfold,type=
c("random", "timeseries", "consecutive", "interleaved"), npermut, seed)}

\arguments{
  \item{n}{The total number of observations.}
  \item{ntest}{The number of observations in test set.}
  \item{ntrain}{The number of observations in training set.}
  \item{Kfold}{Either the number of folds or a boolean or \code{NULL}.}
  \item{type}{A character string in
  \code{random},\code{timeseries},\code{consecutive}, \code{interleaved}
  and give the type of segments.}
  \item{npermut}{The number of random draw (with replacement), used for
    \code{type="random"}.}
  \item{seed}{Controls the seed of random generator
    (via \code{\link{set.seed}}).}
}

\value{
Returns a list with in each component the index of observations to be
used as a test set.}

\references{
  Cornillon, P.-A.; Hengartner, N.; Jegou, N. and Matzner-Lober, E. (2012)
  Iterative bias reduction: a comparative study.
\emph{Statistics and Computing}, \emph{23}, 777-791.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2013)
  Recursive bias estimation for multivariate regression smoothers Recursive 
bias estimation for multivariate regression smoothers.
\emph{ESAIM: Probability and Statistics}, \emph{18}, 483-502.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2017)
  Iterative Bias Reduction Multivariate Smoothing in R: The ibr Package.
\emph{Journal of Statistical Software}, \emph{77}, 1--26.
}

\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober.}

\seealso{\code{\link{ibr}}}


\keyword{smooth}
\keyword{multivariate}

\name{ozone}
\docType{data}
\alias{ozone}
\title{Los Angeles ozone pollution data, 1976.}

\description{
Los Angeles ozone pollution data, 1976.  
We deleted from the original data, the first 3 columns which were the  
\code{Month}, \code{Day of the month} and \code{Day of the week}. Each
observation is one day, so there is 366 rows.
The \code{ozone} data is a matrix with 9 columns.
}

\format{
This data set is a matrix containing the following columns: 
\tabular{rlll}{
         [,1] \tab Ozone   \tab numeric \tab Daily maximum
	 one-hour-average ozone reading 
	 (parts per million) at Upland, CA.\cr
         [,2] \tab Pressure.Vand \tab numeric \tab 500 millibar pressure height (m) measured at Vandenberg AFB.\cr
         [,3] \tab Wind    \tab numeric \tab Wind speed (mph) at Los Angeles International Airport (LAX).\cr
         [,4] \tab Humidity \tab numeric \tab Humidity in percentage at LAX.\cr
         [,5] \tab Temp.Sand \tab numeric \tab Temperature (degrees F) measured at Sandburg, CA.\cr
         [,6] \tab Inv.Base.height \tab numeric \tab Inversion base height (feet) at LAX.\cr
         [,7] \tab Pressure.Grad   \tab numeric \tab Pressure gradient (mm Hg) from LAX to Daggett, CA.\cr
         [,8] \tab Inv.Base.Temp   \tab numeric \tab Inversion base temperature (degrees F) at LAX.\cr
         [,9] \tab Visilibity     \tab numeric \tab Visibility (miles) measured at LAX.
       }
     }
\source{
Leo Breiman, Department of Statistics, UC Berkeley.  Data used in
     Breiman, L. and  Friedman, J. H. (1985). Estimating optimal
     transformations for multiple regression and correlation,
     \emph{Journal of American Statistical Association}, \bold{80},
     580--598.
}
\seealso{ \code{\link{ibr}}}
\keyword{datasets}

\name{ibr-package}
\alias{ibr-package}
\docType{package}
\title{
Iterative Bias Reduction
}
\description{
an R package for multivariate smoothing using Iterative Bias Reduction smoother.
}
\details{
  \itemize{\item We are interested in smoothing (the values of) a vector of
  \eqn{n} observations \eqn{y} by  \eqn{d} covariates measured at the
  same \eqn{n} observations (gathered in the matrix \eqn{X}).
 The iterated Bias Reduction produces a sequence of smoothers
  \deqn{\hat y=S_k y =(I - (I-S)^k)y,}
  where \eqn{S} is the pilot smoother which can be either a kernel or a
  thin plate spline smoother. In case of a kernel smoother, the kernel
  is built as a product of univariate kernels.
  \item The most important parameter of the iterated bias reduction is
  \eqn{k} the
  number of 
  iterationsr. Usually this parameter is unknown and is
  chosen from the search grid \code{K} to minimize the criterion (GCV,
  AIC, AICc, BIC or gMDL).\cr 
The user must choose the pilot smoother
  (kernel \code{"k"}, thin plate splines \code{"tps"} or Duchon splines \code{"ds"})
  plus the values of bandwidths (kernel)
  or \eqn{\lambda} thin plate splines). As the choice of these raw
  values depend on each particular dataset, one
  can rely on effective degrees of freedom or default values given as degree of
  freedom, see argument \code{df} of the main function \code{\link{ibr}}.}

\bold{Index of functions to be used by end user:}
\preformatted{
ibr:               Iterative bias reduction smoothing
plot.ibr:          Plot diagnostic for an ibr object
predict.ibr:       Predicted values using iterative bias reduction
                   smoothers
forward:           Variable selection for ibr (forward method)
print.summary.ibr: Printing iterative bias reduction summaries
summary.ibr:       Summarizing iterative bias reduction fits
}
}

  
\examples{
\dontrun{
data(ozone, package = "ibr")
res.ibr <- ibr(ozone[,-1],ozone[,1],smoother="k",df=1.1)
summary(res.ibr)
predict(res.ibr)
plot(res.ibr)
}
}


\author{
Pierre-Andre Cornillon, Nicolas Hengartner, Eric Matzner-Lober

Maintainer: Pierre-Andre Cornillon <pierre-andre.cornillon@supagro.inra.fr>
}
\keyword{ package }

\name{dsSx}

\alias{dsSx}

\title{Evaluate the smoothing matrix at any point}

\description{The function evaluates the matrix \emph{Q} and \emph{S} related to
  the explanatory variables \eqn{X} at any points. This function is not intended to be used directly.
}

\usage{
dsSx(X,Xetoile,m=2,s=0)
}

\arguments{
  \item{X}{Matrix of explanatory variables, size n,p.}
  \item{Xetoile}{Matrix of new observations with the same number of
    variables as \eqn{X}, size m,p.}
  \item{m}{The order of derivatives for the penalty (for thin plate
  splines it is the order). This integer \emph{m} must verify
  2\emph{m}+2\emph{s}/\emph{d}>1, where \emph{d} is the number of explanatory
  variables.} 
  \item{s}{The power of weighting function. For thin plate splines
  \emph{s} is equal to 0. This real must be strictly smaller than \emph{d}/2
  (where \emph{d} is the number of explanatory  variables) and must
  verify 2\emph{m}+2\emph{s}/\emph{d}. To get pseudo-cubic splines,
  choose \emph{m}=2 and \emph{s}=(\emph{d}-1)/2 (See Duchon, 1977).} 
 }
\details{see the reference for detailed explanation of \emph{Q} (the
  semi kernel) and
  \emph{S} (the polynomial null space).}
 
\references{
Duchon, J. (1977) Splines minimizing rotation-invariant semi-norms in
Solobev spaces. in W. Shemp and K. Zeller (eds) \emph{Construction theory of
  functions of several variables}, 85-100, Springer, Berlin.

C. Gu (2002) \emph{Smoothing spline anova models}. New York:
Springer-Verlag.
}
 
\value{
Returns a list containing two matrices denoted \code{Sgu} (for null
space) and \code{Qgu}}

\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober}

\seealso{\code{\link{ibr}}}


\keyword{smooth}

\name{bwchoice}

\alias{bwchoice}

\title{Choice of bandwidth achieving a prescribed effective degree of freedom}

\description{
Perform a search for the bandwidths in the given grid. For each explanatory
variable, the bandwidth is chosen such that the trace of the smoothing
matrix according to that variable (effective degree of freedom) is equal to a
prescribed value.
This function is not intended to be used directly.}

\usage{
bwchoice(X,objectif,kernelx="g",itermax=1000)}

\arguments{
  \item{X}{A matrix with \eqn{n} rows (individuals) and \eqn{p} columns (numeric variables).}
  \item{objectif}{A numeric vector of either length 1 or length equal to the
  number of columns of \code{X}. It indicates the  desired effective degree of
  freedom (trace) of the smoothing   matrix for
    each variable. \code{objectif} is repeated when the length of vector
  \code{objectif} is 1.}
  \item{kernelx}{String which allows to choose between gaussian kernel
  (\code{"g"}), Epanechnikov (\code{"e"}), uniform (\code{"u"}),
  quartic (\code{"q"}).}
  \item{itermax}{A scalar which controls the number of iterations for
    that search.}
}

\value{
Returns a vector of length d, the number of explanatory variable, where
each coordinate is the value of the selected bandwidth for each
explanatory variable}

\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober.}

\seealso{ \code{\link{ibr}}}


\keyword{smooth}

\name{summary.npregress}

\alias{summary.npregress}

\title{Summarizing local polynomial fits}

\description{
  \code{summary} method for class  \dQuote{\code{npregress}}.
}

\usage{
\method{summary}{npregress}(object,  criteria="call", \dots)}

\arguments{
  \item{object}{Object of class \code{\link{npregress}}.}
  \item{criteria}{Character string which gives the criteria evaluated for the model. The criteria available are GCV (default, \code{"gcv"}),
  AIC (\code{"aic"}), corrected AIC  (\code{"aicc"}),   BIC
  (\code{"bic"}) or gMDL  (\code{"gmdl"}). The string \code{"call"}
  return the criterion used in the call of \code{npregress}.}
  \item{\dots}{Further arguments passed to or from other methods.}
}

\value{
The function \code{summary.npregress} computes and returns a list of summary
     statistics of the local polynomial smoother given in \code{object}}

\author{Pierre-Andre Cornillon, Nicolas Hengartner and Eric Matzner-Lober.}

\references{
Wand, M. P. and Jones, M. C. (1995). \emph{Kernel Smoothing}. Chapman and Hall, London.}

\seealso{\code{\link{npregress}}, \code{\link{summary.npregress}}}

\examples{
f <- function(x){sin(5*pi*x)}
n <- 100
x <- runif(n)
z <- f(x)
sigma2 <- 0.05*var(z)
erreur <- rnorm(n,0,sqrt(sigma2))
y <- z+erreur
res <- npregress(x,y,bandwidth=0.02)
summary(res)
}

\keyword{smooth}
\keyword{multivariate}

\name{npregress}

\alias{npregress}
\alias{print.npregress}
\alias{residuals.npregress}

\title{Local polynomials smoothing}

\description{
  Predicted values from a local polynomials of degree less than 2.  \cr
Missing values are not allowed.
}

\usage{
npregress(x, y, criterion="rmse", bandwidth=NULL,kernel="g",
             control.par=list(), cv.options=list())}

\arguments{
  \item{x}{A numeric vector of explanatory variable of length \emph{n}.}
  \item{y}{A numeric vector of variable to be explained of length \emph{n}.}
  \item{criterion}{Character string. If the bandwidth 
    (\code{bandwidth}) is missing or 
    \code{NULL} the number of iterations is chosen using
    \code{criterion}. The criterion available is (cross-validated) rmse
  (\code{"rmse"}) and mean (relative) absolute error.}
\item{bandwidth}{The kernel bandwidth smoothing parameter (a numeric vector of either length 1).}
\item{kernel}{Character string which allows to choose between gaussian kernel
  (\code{"g"}), Epanechnikov (\code{"e"}), uniform (\code{"u"}),
  quartic (\code{"q"}).}
\item{control.par}{A named list that control optional parameters. The
  two components are \code{bandwidth} for compatibility with \code{\link{ibr}}
  arguments and \code{degree} which controls the degree of the local
  polynomial regression. If argument \code{bandwidth} is not null or missing, its
  value is used instead \code{control.par$bandwidth}. \code{degree} must
  be smaller than 2. For (gaussian binned) local polynomial see
  \code{\link[KernSmooth]{locpoly}}}
  \item{cv.options}{A named list which controls the way to do cross
  validation with component \code{gridbw},
  \code{ntest}, \code{ntrain}, \code{Kfold}, \code{type},
  \code{seed}, \code{method} and \code{npermut}. \code{gridbw} is
  numeric vector which contains the search grid for optimal bandwidth  (default
  to \code{1/n*(1+1/n)^(0:kmax)}, with \code{kmax=floor(log(n*diff(range(x))/3)/log(1+1/n))}). \code{ntest} is the number of observations in test set and
  \code{ntrain} is the number of observations in training set. Actually,
  only one of these is needed the other can be \code{NULL} or missing. \code{Kfold} a boolean or an integer. If
  \code{Kfold} is \code{TRUE} then the number of fold is deduced from
  \code{ntest} (or \code{ntrain}).  \code{type} is a character string in
  \code{random},\code{timeseries},\code{consecutive}, \code{interleaved}
  and give the type of segments.  \code{seed} controls the seed of
  random generator. \code{npermut} is the number of random draws.   If
  \code{cv.options} is \code{list()}, then component \code{ntest} is set to
  \code{1}, \code{type} is consecutive, \code{Kfold} is \code{TRUE}, and
  the other components are \code{NULL}, which leads to leave-one-out
  cross-validation.}
}

\value{
Returns an object of class \code{npregress} which is a list including:
  \item{bandwidth}{The kernel bandwidth smoothing parameter.}
  \item{residuals}{Vector of residuals.}
  \item{fitted}{Vector of fitted values.}
  \item{df}{The effective degree of freedom of the smoother.}
   \item{call}{A list containing four components: \code{x} contains the
    initial explanatory variables, \code{y} contains the
    initial dependant variables, 
    \code{criterion} contains the chosen criterion, \code{kernel} the
    kernel and \code{degree} the chosen degree}
  \item{criteria}{either a named list containing the bandwidth search
    grid and all the criteria (\code{rmse} and \code{mae}) evaluated on the
    grid \code{gridbw}. If the bandwidth
    \code{bandwidth} is given by the user \code{NULL} is returned}
}

\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober.}

\note{See \code{\link[KernSmooth]{locpoly}} for fast binned implementation
  over an equally-spaced grid of local polynomial. See \code{\link{ibr}}
  for univariate and \strong{multivariate} smoothing. 
}
\references{
Wand, M. P. and Jones, M. C. (1995). \emph{Kernel Smoothing}. Chapman and Hall, London.}

\seealso{\code{\link{predict.npregress}},
  \code{\link{summary.npregress}},
  \code{\link[KernSmooth]{locpoly}}, \code{\link{ibr}}}

\examples{
f <- function(x){sin(5*pi*x)}
n <- 100
x <- runif(n)
z <- f(x)
sigma2 <- 0.05*var(z)
erreur <- rnorm(n,0,sqrt(sigma2))
y <- z+erreur
# given bandwidth (default Gaussian kernel)
res <- npregress(x,y,bandwidth=0.02)
summary(res)
ord <- order(x)
plot(x,y)
lines(x[ord],predict(res)[ord])
# local polynomial of degree 1 (default Gaussian kernel)
# bandwidth chosen with RMSE on 5-fold cross-validation
res <- npregress(x,y,bandwidth=0.02, control.par=list(degree=1),
                 cv.options=list(Kfold=5))
summary(res)
}

\keyword{smooth}
\keyword{multivariate}

\name{iterchoiceS1e}

\alias{iterchoiceS1e}

\title{Number of iterations selection for iterative bias reduction model}

\description{
Evaluate at each iteration proposed in the grid the value of different
criteria: GCV, AIC, corrected AIC, BIC and gMDL (along with the ddl and sigma
squared). The minimum of these
criteria gives an estimate of the optimal number of iterations.
This function is not intended to be used directly.
}

\usage{
iterchoiceS1e(y, K, tUy, eigenvaluesS1, ddlmini, ddlmaxi)}

\arguments{
  \item{y}{The response variable}
  \item{K}{A numeric vector which give the search grid for iterations}
  \item{eigenvaluesS1}{Vector of the eigenvalues of the
  symmetric smoothing matrix \emph{S}.}
  \item{tUy}{The transpose of the matrix of eigen vectors of the
  symmetric smoothing matrix \emph{S} times the vector of observation \emph{y}.}
  \item{ddlmini}{The number of eigen values of \emph{S} equal to 1.}
  \item{ddlmaxi}{The maximum df. No criteria are calculated beyond the
  number of iterations that leads to df bigger than this bound.}
}

\value{ Returns the values of GCV, AIC, corrected AIC, BIC, gMDL, df
and sigma squared for each value of the grid \code{K}. \code{Inf} are
returned if the iteration leads to a smoother with a df bigger than
\code{ddlmaxi}.}

\references{
  Cornillon, P.-A.; Hengartner, N.; Jegou, N. and Matzner-Lober, E. (2012)
  Iterative bias reduction: a comparative study.
\emph{Statistics and Computing}, \emph{23}, 777-791.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2013)
  Recursive bias estimation for multivariate regression smoothers Recursive 
bias estimation for multivariate regression smoothers.
\emph{ESAIM: Probability and Statistics}, \emph{18}, 483-502.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2017)
  Iterative Bias Reduction Multivariate Smoothing in R: The ibr Package.
\emph{Journal of Statistical Software}, \emph{77}, 1--26.
}

\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober}

\seealso{\code{\link{ibr}}, \code{\link{iterchoiceS1}}}


\keyword{smooth}
\keyword{multivariate}

\name{lrsmoother}

\alias{lrsmoother}

\title{Evaluate the lowrank spline}

\description{The function evaluates all the features needed for a
  lowrank spline smoothing. This function is not intended to be used directly.}

\usage{
lrsmoother(x,bs,listvarx,lambda,m,s,rank)
}

\arguments{
  \item{x}{Matrix of explanatory variables, size n,p.}
 \item{bs}{The type rank of lowrank splines: \code{tps} or \code{ds}.}
 \item{listvarx}{The vector of the names of explanatory variables}
  \item{lambda}{The smoothness coefficient lambda for thin plate splines of
  order \code{m}.}
  \item{m}{The order of derivatives for the penalty (for thin plate
  splines it is the order). This integer \emph{m} must verify
  2\emph{m}+2\emph{s}/\emph{d}>1, where \emph{d} is the number of explanatory
  variables.} 
  \item{s}{The power of weighting function. For thin plate splines
  \emph{s} is equal to 0. This real must be strictly smaller than \emph{d}/2
  (where \emph{d} is the number of explanatory  variables) and must
  verify 2\emph{m}+2\emph{s}/\emph{d}. To get pseudo-cubic splines,
  choose \emph{m}=2 and \emph{s}=(\emph{d}-1)/2 (See Duchon, 1977).} 
 \item{rank}{The rank of lowrank splines.}
 }
\details{see the reference for detailed explanation of the matrix matrix
  \eqn{R^{-1}U}{R^{-1}U} (see reference) and \link[mgcv:smoothCon]{mgcv::smoothCon} for the definition of smoothobject}
 
\references{
Duchon, J. (1977) Splines minimizing rotation-invariant semi-norms in
Solobev spaces. in W. Shemp and K. Zeller (eds) \emph{Construction theory of
functions of several variables}, 85-100, Springer, Berlin. 

Wood, S.N. (2003) Thin plate regression
  splines. \emph{J. R. Statist. Soc. B}, \emph{65}, 95-114.
}
 
\value{ Returns a list containing the smoothing matrix eigenvectors and eigenvalues
  \code{vectors} and \code{values},  and one
matrix denoted \code{Rm1U} and one smoothobject \code{smoothobject}.}

\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober}

\seealso{\code{\link{ibr}}}


\keyword{smooth}

\name{kernelSx}

\alias{kernelSx}

\title{Evaluates the smoothing matrix at x*}

\description{The function evaluates the matrix (kernel) smoothing 
  matrix at arbitrary locations \emph{x*}.}

\usage{
kernelSx(X,Xetoile=NULL,bx,kernelx="g")
}

\arguments{
  \item{X}{Matrix of explanatory variables, size \emph{n}, \emph{p}.}
  \item{Xetoile}{Matrix of new design points \emph{x}* at which to predict the
  response variable, size \emph{n*}, \emph{p}. If \code{NULL} then
  \code{Xetoile} is equal to \code{X}.}
  \item{bx}{The vector of different bandwidths, length \eqn{p}.}
  \item{kernelx}{Character string which allows to choose between gaussian kernel
  (\code{"g"}), Epanechnikov (\code{"e"}), uniform (\code{"u"}),
  quartic (\code{"q"}).}
}

\value{
Returns the matrix \eqn{Sx} which \eqn{i,j} entry is equal to
\deqn{\frac{\prod_{k=1}^p K((x_{ik}^*-x_{jk})/h_k)}{\sum_{j=1}^n \prod_{k=1}^p K((x_{ik}^*-x_{jk})/h_k)}} where \eqn{K(.)} is the
chosen kernel.
}

\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober.}

\seealso{\code{\link{kernelKx}}, \code{\link{ibr}}}


\keyword{smooth}

\name{plot.forwardibr}

\alias{plot.forwardibr}

\title{Plot diagnostic for an ibr object}

\description{
  One plot  is  currently available: a plot
     of residuals against fitted values.
}

\usage{
\method{plot}{forwardibr}(x,global=FALSE,\ldots )} 

\arguments{
  \item{x}{Object of class \code{\link{forwardibr}}.}
  \item{global}{Boolean: if \code{global} is \code{TRUE} the color code
  is between the min and the max of \code{x} (except infinite value); if
  \code{global} is \code{FALSE} the color code is between the min and
  the max of each row.}
  \item{\dots}{further arguments passed to \code{\link{image}}.}
}

\value{ The function \code{plot.forwardibr} give an image plot of the
values of the criterion obtained by the forward selection process. Image
is read from the bottom to the top. At the bottom row, there are all the
univariate models and the selected variable is given by the lowest
criterion. This variable is selected for the second row. At the second
(bottom) row the second variable included is those which give the lowest
criterion for this row etc. All the variables included in the final
model (selected by forward search) are numbered on the image (by order of
inclusion).}

\author{Pierre-Andre Cornillon, Nicolas Hengartner and Eric Matzner-Lober.}

\references{
  Cornillon, P.-A.; Hengartner, N.; Jegou, N. and Matzner-Lober, E. (2012)
  Iterative bias reduction: a comparative study.
\emph{Statistics and Computing}, \emph{23}, 777-791.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2013)
  Recursive bias estimation for multivariate regression smoothers Recursive 
bias estimation for multivariate regression smoothers.
\emph{ESAIM: Probability and Statistics}, \emph{18}, 483-502.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2017)
  Iterative Bias Reduction Multivariate Smoothing in R: The ibr Package.
\emph{Journal of Statistical Software}, \emph{77}, 1--26.
}
\seealso{\code{\link{ibr}}, \code{\link{forward}}}

\examples{
\dontrun{data(ozone, package = "ibr")
ibrsel <- forward(ibr(ozone[,-1],ozone[,1],df=1.2)
plot(ibrsel)
plot(apply(ibrsel,1,min,na.rm=TRUE),type="l")
}
}

\keyword{smooth}
\keyword{multivariate}

\name{forward}

\alias{forward}
\alias{forwardibr}

\title{Iterative bias reduction smoothing}

\description{ Performs a forward variable selection for iterative bias
  reduction using kernel, thin plate splines or low rank splines.
  Missing values are not allowed.}

\usage{
forward(formula,data,subset,criterion="gcv",df=1.5,Kmin=1,Kmax=1e+06,
   smoother="k",kernel="g",rank=NULL,control.par=list(),cv.options=list(),
   varcrit=criterion)}

\arguments{
  \item{formula}{An object of class \code{"\link{formula}"} (or one that
    can be coerced to that class): a symbolic description of the
    model to be fitted. }

  \item{data}{An optional data frame, list or environment (or object
    coercible by \code{\link{as.data.frame}} to a data frame) containing
    the variables in the model.  If not found in \code{data}, the
    variables are taken from \code{environment(formula)},
    typically the environment from which \code{forward} is called.}
  \item{subset}{An optional vector specifying a subset of observations
    to be used in the fitting process.}
  \item{criterion}{Character string. If the number of iterations
  (\code{iter}) is missing or 
  \code{NULL} the number of iterations is chosen using
  \code{criterion}. The criteria available are GCV (default, \code{"gcv"}),
  AIC (\code{"aic"}), corrected AIC  (\code{"aicc"}),   BIC
  (\code{"bic"}), gMDL  (\code{"gmdl"}), map (\code{"map"}) or rmse
  (\code{"rmse"}). The last two are designed for cross-validation.}
  \item{df}{A numeric vector of either length 1 or length equal to the
    number of columns of \code{x}. If \code{smoother="k"}, it indicates
    the  desired degree of
  freedom (trace) of the smoothing   matrix for
    each variable or for the initial smoother (see \code{contr.sp$dftotal}); \code{df} is repeated when the length of vector
  \code{df} is 1. If \code{smoother="tps"}, the minimum df of thin
  plate splines is multiplied by \code{df}. This argument is useless if
  \code{bandwidth} is supplied (non null).}
  \item{Kmin}{The minimum number of bias correction  iterations of the
  search grid considered by
  the model selection procedure for selecting the optimal number of iterations.}
  \item{Kmax}{The maximum number of bias correction  iterations of the
  search grid considered by
  the model selection procedure for selecting the optimal number of iterations.}
\item{smoother}{Character string which allows to choose between thine plate
  splines \code{"tps"} or kernel (\code{"k"}).}

\item{kernel}{Character string which allows to choose between gaussian kernel
  (\code{"g"}), Epanechnikov (\code{"e"}), uniform (\code{"u"}),
  quartic (\code{"q"}). The default (gaussian kernel) is strongly advised.}

\item{rank}{Numeric value that control the rank of low rank splines
  (denoted as \code{k} in mgcv package ; see also \link[mgcv]{choose.k}
  for further details or \link[mgcv:gam]{mgcv::gam} for another smoothing approach with
  reduced rank smoother.}

\item{control.par}{a named list that control optional parameters. The
  components are \code{bandwidth} (default to NULL), \code{iter}
  (default to NULL), \code{really.big} (default to \code{FALSE}),
  \code{dftobwitmax} (default to 1000), \code{exhaustive} (default to
  \code{FALSE}),\code{m} (default to NULL), \code{dftotal} (default to
  \code{FALSE}), \code{accuracy} (default to 0.01), \code{ddlmaxi}
  (default to 2n/3) and \code{fraction} (default to \code{c(100, 200, 500, 1000, 5000,10^4,5e+04,1e+05,5e+05,1e+06)}).

  \code{bandwidth}: a vector of either length 1 or length equal to the
    number of columns of \code{x}. If \code{smoother="k"},
    it indicates the bandwidth used for
  each variable, bandwidth is repeated when the length of vector
  \code{bandwidth} is 1. If \code{smoother="tps"}, it indicates the
  amount of penalty (coefficient lambda).
  The default (missing) indicates, for \code{smoother="k"}, that
  bandwidth for each variable is
  chosen such that each univariate kernel
  smoother (for each explanatory variable) has \code{df} degrees of
  freedom and for \code{smoother="tps"} that lambda is chosen such that
  the df of the smoothing matrix is \code{df} times the minimum df.

  \code{iter}: the number of iterations. If null or missing, an optimal number of
  iterations is chosen from 
  the search grid (integer from \code{Kmin} to \code{Kmax}) to minimize the \code{criterion}.
 
  \code{really.big}: a boolean: if \code{TRUE} it overides the limitation
  at 5000 observations. Expect long computation times if \code{TRUE}.
  
\code{dftobwitmax}: When bandwidth is chosen by specifying the degree
  of freedom (see \code{df}) a search is done by
  \code{\link[stats:uniroot]{stats::uniroot}}. This argument specifies the maximum number of iterations transmitted to \code{\link[stats:uniroot]{stats::uniroot}} function.
  
   \code{exhaustive}: boolean, if \code{TRUE} an exhaustive search of
  optimal number of iteration on the
  grid \code{Kmin:Kmax} is performed. If \code{FALSE} the minimum  of
  criterion is searched using \code{\link[stats:optimize]{stats::optimize}} between \code{Kmin}
  and \code{Kmax}.
  
  \code{m}: the order of thin plate splines. This integer \emph{m} must verifies
  2\emph{m}/\emph{d}>1, where \emph{d} is the number of explanatory
  variables.  The missing default to choose the order \emph{m} as the first integer
  such that 2\emph{m}/\emph{d}>1, where \emph{d} is the number of
  explanatory variables (same for \code{NULL}). 
  
  \code{dftotal}: a boolean wich indicates when \code{FAlSE} that the
  argument \code{df} is the objective df for each univariate kernel (the
  default) calculated for each explanatory variable or for the overall
  (product) kernel, that is the base smoother (when \code{TRUE}).
  
  \code{accuracy}: tolerance when searching bandwidths which lead to a
  chosen overall intial df.
  
  \code{dfmaxi}: the maximum degree of freedom allowed for iterated
  biased reduction smoother. 
  
  \code{fraction}: the subdivistion of interval \code{Kmin},\code{Kmax}
  if non exhaustive search is performed (see also \code{\link{iterchoiceA}} or \code{\link{iterchoiceS1}}). 
}

   \item{cv.options}{A named list which controls the way to do cross
  validation with component \code{bwchange},
  \code{ntest}, \code{ntrain}, \code{Kfold}, \code{type},
  \code{seed}, \code{method} and \code{npermut}. \code{bwchange} is a boolean (default to \code{FALSE})
  which indicates if bandwidth have to be recomputed each
  time. \code{ntest} is the number of observations in test set and
  \code{ntrain} is the number of observations in training set. Actually,
  only one of these is needed the other can be \code{NULL} or missing. \code{Kfold} a boolean or an integer. If
  \code{Kfold} is \code{TRUE} then the number of fold is deduced from
  \code{ntest} (or \code{ntrain}).  \code{type} is a character string in
  \code{random},\code{timeseries},\code{consecutive}, \code{interleaved}
  and give the type of segments.  \code{seed} controls the seed of
  random generator. \code{method} is either \code{"inmemory"} or
  \code{"outmemory"}; \code{"inmemory"} induces some calculations outside
  the loop saving computational time but leading to an increase of the required
  memory. \code{npermut} is the number of random draws.   If
  \code{cv.options} is \code{list()}, then component \code{ntest} is set to
  \code{floor(nrow(x)/10)}, \code{type} is random, \code{npermut} is 20
  and \code{method} is \code{"inmemory"}, and the other components are
  \code{NULL}}
\item{varcrit}{Character string. Criterion used for variable
  selection. The criteria available are GCV,
  AIC (\code{"aic"}), corrected AIC  (\code{"aicc"}),   BIC
  (\code{"bic"}) and gMDL  (\code{"gmdl"}).}
}

\value{ Returns an object of class \code{forwardibr} which is a matrix
  with \code{p} columns. In the first row, each entry \emph{j} contains
  the value of the chosen criterion for the univariate smoother using
  the jth explanatory variable. The variable which realize the minimum
  of the first row is included in the model. All the column of this
  variable will be \code{Inf} except the first row. In the second row,
  each entry \emph{j} contains the bivariate smoother using the jth
  explanatory variable and the variable already included. The variable
  which realize the minimum of the second row is included in the
  model. All the column of this variable will be \code{Inf} except the
  two first row. This forward selection process continue until the
  chosen criterion increases.}

\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober.}

\references{
  Cornillon, P.-A.; Hengartner, N.; Jegou, N. and Matzner-Lober, E. (2012)
  Iterative bias reduction: a comparative study.
\emph{Statistics and Computing}, \emph{23}, 777-791.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2013)
  Recursive bias estimation for multivariate regression smoothers Recursive 
bias estimation for multivariate regression smoothers.
\emph{ESAIM: Probability and Statistics}, \emph{18}, 483-502.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2017)
  Iterative Bias Reduction Multivariate Smoothing in R: The ibr Package.
\emph{Journal of Statistical Software}, \emph{77}, 1--26.
}
\seealso{\code{\link{ibr}}, \code{\link{plot.forwardibr}}}

\examples{
\dontrun{
data(ozone, package = "ibr")
res.ibr <- forward(ozone[,-1],ozone[,1],df=1.2)
apply(res.ibr,1,which.min)
}
}
\keyword{smooth}
\keyword{multivariate}

\name{departnoyau}

\alias{departnoyau}
\title{Trace of the product kernel smoother}


\description{
 Search bandwidth for each univariate kernel smoother such that the
 product of these univariate kernel gives a kernel smoother with a
 chosen effective degree of freedom (trace of the smoother). The bandwidths are
 constrained to give, for each explanatory
variable, a kernel smoother with same trace as the others.
This function is not intended to be used directly.
}

\usage{
departnoyau(df, x, kernel, dftobwitmax, n, p, dfobjectif)
}

\arguments{
  \item{df}{A numeric vector giving the effective degree of freedom (trace) of the
    univariate smoothing  matrix  for each variable of 
    \code{x}.}
  \item{x}{Matrix of explanatory variables, size \emph{n}, \emph{p}.}
  \item{kernel}{Character string which allows to choose between gaussian kernel
    (\code{"g"}), Epanechnikov (\code{"e"}), uniform (\code{"u"}),
    quartic (\code{"q"}).}
  \item{dftobwitmax}{Specifies the maximum number of iterations
    transmitted   to \code{\link[stats:uniroot]{stats::uniroot}} function.}
  \item{n}{Number of rows of data matrix \emph{x}.}
  \item{p}{Number of columns of data matrix \emph{x}.}
  \item{dfobjectif}{A numeric vector of length 1 which indicates
    the  desired effective degree of freedom (trace) of the smoothing
    matrix (product kernel smoother) for \code{x}.}
 }

\value{
Returns the desired bandwidths.}


\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober.}

\seealso{ \code{\link{ibr}}}


\keyword{smooth}

\name{summary.ibr}

\alias{summary.ibr}

\title{Summarizing iterative bias reduction fits}

\description{
  \code{summary} method for class  \dQuote{\code{ibr}}.
}

\usage{
\method{summary}{ibr}(object,  criteria="call", \dots)}

\arguments{
  \item{object}{Object of class \code{\link{ibr}}.}
  \item{criteria}{Character string which gives the criteria evaluated for the model. The criteria available are GCV (default, \code{"gcv"}),
  AIC (\code{"aic"}), corrected AIC  (\code{"aicc"}),   BIC
  (\code{"bic"}) or gMDL  (\code{"gmdl"}). The string \code{"call"}
  return the criterion used in the call of \code{ibr}.}
  \item{\dots}{Further arguments passed to or from other methods.}
}

\value{
The function \code{summary.ibr} computes and returns a list of summary
     statistics of the fitted  iterative bias reduction smoother given in \code{object}}

\author{Pierre-Andre Cornillon, Nicolas Hengartner and Eric Matzner-Lober.}

\references{
  Cornillon, P.-A.; Hengartner, N.; Jegou, N. and Matzner-Lober, E. (2012)
  Iterative bias reduction: a comparative study.
\emph{Statistics and Computing}. Doi: 10.1007/s11222-012-9346-4

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2017)
  Iterative Bias Reduction Multivariate Smoothing in R: The ibr Package.
\emph{Journal of Statistical Software}, \emph{77}, 1--26.
}
\seealso{\code{\link{ibr}}, \code{\link{summary.ibr}}}

\examples{
\dontrun{data(ozone, package = "ibr")
res.ibr <- ibr(ozone[,-1],ozone[,1],df=1.2)
summary(res.ibr)
predict(res.ibr)}
}

\keyword{smooth}
\keyword{multivariate}

\name{calcA}

\alias{calcA}

\title{Decomposition of the kernel smoother}

\description{
Calculates the decomposition of the kernel smoothing matrix in two part: a
diagonal matrix \emph{D} and a symmetric matrix \emph{A}.
This function is not intended to be used directly.
}

\usage{
calcA(X,bx,kernelx="g")}

\arguments{
  \item{X}{The matrix of explanatory variables, size \emph{n}, \emph{p}.}
  \item{bx}{The vector of bandwidth of length \emph{p}.}
  \item{kernelx}{Character string which allows to choose between gaussian kernel
  (\code{"g"}), Epanechnikov (\code{"e"}), uniform (\code{"u"}),
  quartic (\code{"q"}).}
}
\details{see the reference for detailed explanation of \emph{A} and
  \emph{D} and the meaning of coefficients.}

\value{
  Returns a list containing two matrices: the symmetric matrix \emph{A}
  in component \code{A}) and the square root of the diagonal matrix
  \emph{D} in the component \code{Ddemi} and the trace of the smoother
  in the component \code{df}.}

\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober.}

\seealso{\code{\link{ibr}}}


\keyword{smooth}
\keyword{multivariate}

\name{plot.ibr}

\alias{plot.ibr}

\title{Plot diagnostic for an ibr object}

\description{
  One plot  is  currently available: a plot
     of residuals against fitted values.
}

\usage{
\method{plot}{ibr}(x,\ldots )} 

\arguments{
  \item{x}{Object of class \code{\link{ibr}}.}
  \item{\dots}{Further arguments passed to or from other methods.}
}

\value{
The function \code{plot.ibr} computes and returns a list of summary
     statistics of the fitted  iterative bias reduction smoother given in \code{object}}

\author{Pierre-Andre Cornillon, Nicolas Hengartner and Eric Matzner-Lober.}

\references{
  Cornillon, P.-A.; Hengartner, N.; Jegou, N. and Matzner-Lober, E. (2012)
  Iterative bias reduction: a comparative study.
\emph{Statistics and Computing}, \emph{23}, 777-791.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2013)
  Recursive bias estimation for multivariate regression smoothers Recursive 
bias estimation for multivariate regression smoothers.
\emph{ESAIM: Probability and Statistics}, \emph{18}, 483-502.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2017)
  Iterative Bias Reduction Multivariate Smoothing in R: The ibr Package.
\emph{Journal of Statistical Software}, \emph{77}, 1--26.
}
\seealso{\code{\link{ibr}}, \code{\link{summary.ibr}}}

\examples{
\dontrun{data(ozone, package = "ibr")
res.ibr <- ibr(ozone[,-1],ozone[,1],df=1.2)
plot(res.ibr)}
}

\keyword{smooth}
\keyword{multivariate}

\name{lambdachoicelr}

\alias{lambdachoicelr}

\title{Choice of bandwidth according to a given effective degree of freedom}

\description{
Perform a search for the different bandwidths in the given grid. For each explanatory
variable, the bandwidth is chosen such that the trace of the smoothing
matrix according to that variable (effective degree of freedom) is equal to a given value.
This function is not intended to be used directly.}

\usage{
lambdachoicelr(x,ddlobjectif,m=2,s=0,rank,itermax,bs,listvarx)}

\arguments{
  \item{x}{A matrix with \eqn{n} rows (individuals) and \eqn{p} columns (numeric variables)}
  \item{ddlobjectif}{A numeric vector of length 1 which indicates the  desired effective degree of
  freedom (trace) of the smoothing   matrix for
    thin plate splines of order \code{m}.}
  \item{m}{The order of derivatives for the penalty (for thin plate
  splines it is the order). This integer \emph{m} must verify
  2\emph{m}+2\emph{s}/\emph{d}>1, where \emph{d} is the number of explanatory
  variables.} 
  \item{s}{The power of weighting function. For thin plate splines
  \emph{s} is equal to 0. This real must be strictly smaller than \emph{d}/2
  (where \emph{d} is the number of explanatory  variables) and must
  verify 2\emph{m}+2\emph{s}/\emph{d}. To get pseudo-cubic splines,
  choose \emph{m}=2 and \emph{s}=(\emph{d}-1)/2 (See Duchon, 1977).}
  \item{itermax}{A scalar which controls the number of iterations for
    that search}
 \item{rank}{The rank of lowrank splines.}
 \item{bs}{The type rank of lowrank splines: \code{tps} or \code{ds}.}
 \item{listvarx}{The vector of the names of explanatory variables}
}

\references{
Duchon, J. (1977) Splines minimizing rotation-invariant semi-norms in
Solobev spaces. in W. Shemp and K. Zeller (eds) \emph{Construction theory of
functions of several variables}, 85-100, Springer, Berlin. 

Wood, S.N. (2003) Thin plate regression
  splines. \emph{J. R. Statist. Soc. B}, \emph{65}, 95-114.
}
\value{
Returns the coefficient lambda that control smoothness for the desired
effective degree of freedom}

\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober}

\seealso{ \code{\link{ibr}}}


\keyword{smooth}

\name{iterchoiceAcv}

\alias{iterchoiceAcv}

\title{Selection of the number of iterations for iterative bias reduction smoothers}

\description{
The function \code{iterchoiceAcv} searches the interval from \code{mini} to
     \code{maxi} for a minimum of the function \code{criterion} with respect
     to its first argument using \code{\link[stats:optimize]{stats::optimize}}. This function is not intended to be used directly.
}

\usage{
iterchoiceAcv(X, y, bx, df, kernelx, ddlmini, ntest, ntrain, Kfold,
type, npermut, seed, Kmin, Kmax, criterion, fraction)}

\arguments{
  \item{X}{A numeric matrix of explanatory variables, with \emph{n} rows
  and \emph{p} columns.}
  \item{y}{A numeric vector of variable to be explained of length \emph{n}.}
  \item{bx}{The vector of different bandwidths, length \eqn{p}.}
  \item{df}{A numeric vector of either length 1 or length equal to the
    number of columns of \code{x}. If \code{smoother="k"}, it indicates
    the  desired effective degree of
  freedom (trace) of the smoothing   matrix for
    each variable ; \code{df} is repeated when the length of vector
  \code{df} is 1. This argument is useless if
  \code{bandwidth} is supplied (non null).}
  \item{kernelx}{Character string which allows to choose between gaussian kernel
  (\code{"g"}), Epanechnikov (\code{"e"}), uniform (\code{"u"}),
  quartic (\code{"q"}). The default (gaussian kernel) is strongly advised.}
  \item{ddlmini}{The number of eigenvalues (numerically) equals to 1.}
  \item{ntest}{The number of observations in test set.}
  \item{ntrain}{The number of observations in training set.}
  \item{Kfold}{Either the number of folds or a boolean or \code{NULL}.}
  \item{type}{A character string in
  \code{random},\code{timeseries},\code{consecutive}, \code{interleaved}
  and give the type of segments.}
  \item{npermut}{The number of random draw (with replacement), used for
    \code{type="random"}.}
  \item{seed}{Controls the seed of random generator
    (via \code{\link{set.seed}}).}
  \item{Kmin}{The minimum number of bias correction  iterations of the
  search grid considered by
  the model selection procedure for selecting the optimal number of iterations.}
  \item{Kmax}{The maximum number of bias correction  iterations of the
  search grid considered by
  the model selection procedure for selecting the optimal number of iterations.}
  \item{criterion}{The criteria available are map (\code{"map"}) or rmse
    (\code{"rmse"}).}
  \item{fraction}{The subdivision of the interval [\code{Kmin},\code{Kmax}].}
}

\value{
Returns the optimum number of iterations (between \code{Kmin} and \code{Kmax}).}

\references{
  Cornillon, P.-A.; Hengartner, N.; Jegou, N. and Matzner-Lober, E. (2012)
  Iterative bias reduction: a comparative study.
\emph{Statistics and Computing}, \emph{23}, 777-791.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2013)
  Recursive bias estimation for multivariate regression smoothers Recursive 
bias estimation for multivariate regression smoothers.
\emph{ESAIM: Probability and Statistics}, \emph{18}, 483-502.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2017)
  Iterative Bias Reduction Multivariate Smoothing in R: The ibr Package.
\emph{Journal of Statistical Software}, \emph{77}, 1--26.
}
\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober.}

\seealso{\code{\link{ibr}}}


\keyword{smooth}
\keyword{multivariate}

\name{DuchonS}

\alias{DuchonS}

\title{Computes the semi-kernel of Duchon splines}

\description{
The function \code{DuchonS} computes the semi-kernel of Duchon splines. This function is not intended to be used directly.
}

\usage{
DuchonS(x,m=2)}

\arguments{
  \item{x}{A numeric matrix of explanatory variables, with \emph{n} rows
  and \emph{p} columns.}
  \item{m}{Order of derivatives.}
}

\value{
The polynomial part evaluated.}

\references{
Duchon, J. (1977) Splines minimizing rotation-invariant semi-norms in
Solobev spaces. in W. Shemp and K. Zeller (eds) \emph{Construction theory of
functions of several variables}, 85-100, Springer, Berlin. }

\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober.}

\seealso{\code{\link{ibr}}}


\keyword{smooth}
\keyword{multivariate}

\name{iterchoiceA}

\alias{iterchoiceA}
\alias{critAgcv}
\alias{critAaic}
\alias{critAbic}
\alias{critAaicc}
\alias{critAgmdl}

\title{Selection of the number of iterations for iterative bias reduction smoothers}

\description{ The function \code{iterchoiceA} searches the interval from
\code{mini} to \code{maxi} for a minimum of the function
which calculates the chosen
\code{criterion} (\code{critAgcv}, \code{critAaic}, \code{critAbic},
\code{critAaicc} or \code{critAgmdl})  with respect to its first
argument (a given iteration \code{k}) using \code{\link[stats:optimize]{stats::optimize}}. This function is not
intended to be used directly.  }

\usage{
iterchoiceA(n, mini, maxi, eigenvaluesA, tPADmdemiY, DdemiPA, 
ddlmini, ddlmaxi, y, criterion, fraction)}

\arguments{
  \item{n}{The number of observations.}
  \item{mini}{The lower end point of the interval to be searched.}
  \item{maxi}{The upper end point of the interval to be searched.}
  \item{eigenvaluesA}{Vector of the eigenvalues of the
  symmetric matrix \emph{A}.}
  \item{tPADmdemiY}{The transpose of the matrix of eigen vectors of the
  symmetric matrix \emph{A} times the inverse of the square root of the diagonal matrix \emph{D}.}
  \item{DdemiPA}{The square root of the diagonal matrix \emph{D} times
  the eigen vectors of the symmetric matrix \emph{A}.}
  \item{ddlmini}{The number of eigenvalues (numerically) equals to 1.}
  \item{ddlmaxi}{The  maximum df. No criterion is calculated and
  \code{Inf} is returned.}
  \item{y}{The vector of observations of dependant variable.}
  \item{criterion}{The criteria available are GCV (default, \code{"gcv"}),
  AIC (\code{"aic"}), corrected AIC  (\code{"aicc"}),   BIC
  (\code{"bic"}) or gMDL  (\code{"gmdl"}).}
  \item{fraction}{The subdivision of the interval [\code{mini},\code{maxi}].}
}
\details{See the reference for detailed explanation of \emph{A} and
  \emph{D}. The interval [\code{mini},\code{maxi}] is splitted into
  subintervals using \code{fraction}. In each subinterval the function
  \code{fcriterion} is minimzed  using \code{\link[stats:optimize]{stats::optimize}} (with respect
     to its first argument) and the minimum (and its argument) of the
  result of these optimizations is returned.}

\value{
A list with components \code{iter} and \code{objective} which give the
(rounded) optimum number of iterations (between
\code{Kmin} and \code{Kmax}) and the value
     of the function at that real point (not rounded).}

\references{
  Cornillon, P.-A.; Hengartner, N.; Jegou, N. and Matzner-Lober, E. (2012)
  Iterative bias reduction: a comparative study.
\emph{Statistics and Computing}, \emph{23}, 777-791.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2013)
  Recursive bias estimation for multivariate regression smoothers Recursive 
bias estimation for multivariate regression smoothers.
\emph{ESAIM: Probability and Statistics}, \emph{18}, 483-502.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2017)
  Iterative Bias Reduction Multivariate Smoothing in R: The ibr Package.
\emph{Journal of Statistical Software}, \emph{77}, 1--26.
}
\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober.}

\seealso{\code{\link{ibr}}, \code{\link{iterchoiceA}}}


\keyword{smooth}
\keyword{multivariate}

\name{ibr.fit}

\alias{ibr.fit}

\title{Iterative bias reduction smoothing}

\description{
  Performs iterative bias reduction using kernel, thin plate
  splines, Duchon splines or low rank splines. 
Missing values are not allowed. This function is not intended to be used directly.
}

\usage{
ibr.fit(x, y, criterion="gcv", df=1.5, Kmin=1, Kmax=1e+06, smoother="k",
 kernel="g", method="eigen", rank=NULL, control.par=list(), cv.options=list())}

\arguments{
  \item{x}{A numeric matrix of explanatory variables, with \emph{n} rows
    and \emph{p} columns.}
  \item{y}{A numeric vector of variable to be explained of length \emph{n}.}
  \item{criterion}{A vector of string. If the number of iterations
    (\code{iter}) is missing or 
    \code{NULL} the number of iterations is chosen using the either one
  criterion (the first
  coordinate of \code{criterion}) or several (see component
  \code{criterion} of argument list \code{control.par}). The criteria available are GCV (default, \code{"gcv"}),
    AIC (\code{"aic"}), corrected AIC  (\code{"aicc"}),   BIC
    (\code{"bic"}), gMDL  (\code{"gmdl"}), map (\code{"map"}) or rmse
  (\code{"rmse"}). The last two are designed for cross-validation.}
\item{df}{A numeric vector of either length 1 or length equal to the
  number of columns of \code{x}. If \code{smoother="k"}, it indicates
  the  desired effective degree of
  freedom (trace) of the smoothing   matrix for
  each variable or for the initial smoother (see \code{contr.sp$dftotal}); \code{df} is repeated when the length of vector
  \code{df} is 1. If \code{smoother="tps"} or  \code{smoother="ds"}, the
  minimum df of splines is multiplied by \code{df}. This argument is useless if
  \code{bandwidth} is supplied (non null).}
\item{Kmin}{The minimum number of bias correction  iterations of the
  search grid considered by
  the model selection procedure for selecting the optimal number of iterations.}
\item{Kmax}{The maximum number of bias correction  iterations of the
  search grid considered by
  the model selection procedure for selecting the optimal number of iterations.}
\item{smoother}{Character string which allows to choose between thin plate
  splines \code{"tps"}, Duchon
  splines \code{"tps"} (see Duchon, 1977) or kernel (\code{"k"}). }
\item{kernel}{Character string which allows to choose between gaussian kernel
  (\code{"g"}), Epanechnikov (\code{"e"}), uniform (\code{"u"}),
  quartic (\code{"q"}). The default (gaussian kernel) is strongly advised.}
\item{method}{Character string which allows to choose between methods/algorithms
  for ibr: \code{"eigen"} diagonalize the smoother matrix (advised for
  \emph{n}<5000),  \code{"product"} reduce bias iteratively smoothing
  the residuals \emph{r}
  using product  \emph{Sr}.}
\item{rank}{Numeric value that control the rank of low rank splines
  (denoted as \code{k} in mgcv package ; see also \link[mgcv]{choose.k}
  for further details or \link[mgcv:gam]{mgcv::gam} for another smoothing approach with
  reduced rank smoother.}

\item{control.par}{A named list that control optional parameters. The
  components are \code{bandwidth} (default to NULL), \code{iter}
  (default to NULL), \code{really.big} (default to \code{FALSE}),
  \code{dftobwitmax} (default to 1000), \code{exhaustive} (default to
  \code{FALSE}),\code{m} (default to NULL), ,\code{s} (default to NULL),
  \code{dftotal} (default to \code{FALSE}), \code{accuracy} (default to
  0.01), \code{ddlmaxi} (default to 2n/3), \code{fraction} (default
  to \code{c(100, 200, 500, 1000, 5000, 10^4, 5e+04, 1e+05, 5e+05,
    1e+06)}), \code{scale} (default to \code{FALSE}),
  \code{aggregcrit} (default to \code{"no"}) and
  \code{aggregfun} (default to 10^(floor(log10(x[2]))+2)).

  \code{bandwidth}: a vector of either length 1 or length equal to the
    number of columns of \code{x}. If \code{smoother="k"},
    it indicates the bandwidth used for
  each variable, bandwidth is repeated when the length of vector
  \code{bandwidth} is 1. If \code{smoother="tps"}, it indicates the
  amount of penalty (coefficient lambda).
  The default (missing) indicates, for \code{smoother="k"}, that
  bandwidth for each variable is
  chosen such that each univariate kernel
  smoother (for each explanatory variable) has \code{df} effective degrees of
  freedom and for \code{smoother="tps"} or \code{smoother="ds"} that lambda is chosen such that
  the df of the smoothing matrix is \code{df} times the minimum df.

  \code{iter}: the number of iterations. If null or missing, an optimal number of
  iterations is chosen from 
  the search grid (integer from \code{Kmin} to \code{Kmax}) to minimize the \code{criterion}.
 
  \code{really.big}: a boolean: if \code{TRUE} it overides the limitation
  at 5000 observations. Expect long computation times if \code{TRUE}.
  
\code{dftobwitmax}: When bandwidth is chosen by specifying the effective
  degree
  of freedom (see \code{df}) a search is done by
  \code{\link[stats:uniroot]{stats::uniroot}}. This argument specifies the maximum number of iterations transmitted to \code{\link[stats:uniroot]{stats::uniroot}} function.
  
   \code{exhaustive}: boolean, if \code{TRUE} an exhaustive search of
  optimal number of iteration on the grid \code{Kmin:Kmax} is
  performed. All criteria for all iterations in the same class (class
  one: GCV, AIC, corrected AIC, BIC, gMDL ; class two : MAP, RMSE) are
  returned in argument \code{allcrit}. If \code{FALSE} the minimum of
  criterion is searched using \code{\link[stats:optimize]{stats::optimize}} between \code{Kmin}
  and \code{Kmax}.
  
  \code{m}: The order of derivatives for the penalty (for thin plate
  splines it is the order). This integer \emph{m} must verify
  2\emph{m}+2\emph{s}/\emph{d}>1, where \emph{d} is the number of
  explanatory variables. The default (for \code{smoother="tps"}) is to
  choose the order \emph{m} as the first integer such that
  2\emph{m}/\emph{d}>1, where \emph{d} is the number of explanatory
  variables. The default (for \code{smoother="ds"}) is to choose
  \emph{m}=2 (p
  seudo cubic splines).

    \code{s}: the power of weighting function. For thin plate splines
  \emph{s} is equal to 0. This real must be strictly smaller than \emph{d}/2
  (where \emph{d} is the number of explanatory  variables) and must
  verify 2\emph{m}+2\emph{s}/\emph{d}. To get pseudo-cubic splines (the default),
  choose \emph{m}=2 and \emph{s}=(\emph{d}-1)/2 (See Duchon, 1977).the order of thin plate splines. This integer \emph{m} must verifies
  2\emph{m}/\emph{d}>1, where \emph{d} is the number of explanatory
  variables. 

  \code{dftotal}: a boolean wich indicates when \code{FAlSE} that the
  argument \code{df} is the objective df for each univariate kernel (the
  default) calculated for each explanatory variable or for the overall
  (product) kernel, that is the base smoother (when \code{TRUE}).
  
  \code{accuracy}: tolerance when searching bandwidths which lead to a
  chosen overall intial df.
  
  \code{dfmaxi}: the maximum effective degree of freedom allowed for iterated
  biased reduction smoother. 
  
  \code{fraction}: the subdivision of interval \code{Kmin},\code{Kmax}
  if non exhaustive search is performed (see also \code{\link{iterchoiceA}} or \code{\link{iterchoiceS1}}). 

  \code{scale}: boolean. If \code{TRUE} \code{x} is scaled (using
  \code{\link{scale}}); default to \code{FALSE}.

  \code{criterion} Character string. Possible choices are \code{strict},
  \code{aggregation} or \code{recalc}. \code{strict}
  allows to select the number of iterations according to
  the first coordinate of argument \code{criterion}.
  \code{aggregation}
  allows to select the number of iterations by applying the
  function \code{control.par$aggregfun} to the number of iterations
  selected by all the criteria chosen in argument \code{criterion}.
    \code{recalc}
    allows to select the number of iterations by first calculating the
    optimal number of the second coordinate of argument
  \code{criterion}, then applying the function
  \code{control.par$aggregfun} (to add some number to
  it) resulting in a new \code{Kmax} and then doing the optimal selction
  between  \code{Kmin} and this new \code{Kmax} using the first coordinate of argument
  \code{criterion}.
  ; default to \code{strict}.

  \code{aggregfun} function to be applied when
  \code{control.par$criterion} is either \code{recalc} or
  \code{aggregation}.

  \code{verbose} boolean for verbose output (default to \code{FALSE}).
}

   \item{cv.options}{A named list which controls the way to do cross
  validation with component \code{bwchange},
  \code{ntest}, \code{ntrain}, \code{Kfold}, \code{type},
  \code{seed}, \code{method} and \code{npermut}. \code{bwchange} is a boolean (default to \code{FALSE})
  which indicates if bandwidth have to be recomputed each
  time. \code{ntest} is the number of observations in test set and
  \code{ntrain} is the number of observations in training set. Actually,
  only one of these is needed the other can be \code{NULL} or missing. \code{Kfold} a boolean or an integer. If
  \code{Kfold} is \code{TRUE} then the number of fold is deduced from
  \code{ntest} (or \code{ntrain}).  \code{type} is a character string in
  \code{random},\code{timeseries},\code{consecutive}, \code{interleaved}
  and give the type of segments.  \code{seed} controls the seed of
  random generator. \code{method} is either \code{"inmemory"} or
  \code{"outmemory"}; \code{"inmemory"} induces some calculations outside
  the loop saving computational time but leading to an increase of the required
  memory. \code{npermut} is the number of random draws.   If
  \code{cv.options} is \code{list()}, then component \code{ntest} is set to
  \code{floor(nrow(x)/10)}, \code{type} is random, \code{npermut} is 20
  and \code{method} is \code{"inmemory"}, and the other components are \code{NULL}}
}

\value{
Returns a list including:
  \item{beta}{Vector of coefficients.}
  \item{residuals}{Vector of residuals.}
  \item{fitted}{Vector of fitted values.}
  \item{iter}{The number of iterations used.}
  \item{initialdf}{The initial effective degree of freedom of the pilot (or base) smoother.}
  \item{finaldf}{The effective degree of freedom of the iterated bias reduction
    smoother at the \code{iter} iterations.}
  \item{bandwidth}{Vector of bandwith for each explanatory variable}
  \item{call}{The matched call}
  \item{parcall}{A list containing several components: 
    \code{p} contains the number of explanatory variables and \code{m}
    the order of the splines (if relevant), \code{s}
    the power of weights, \code{scaled} boolean which is \code{TRUE}
    when explanatory variables are scaled, \code{mean} mean of explanatory
    variables if \code{scaled=TRUE}, \code{sd} standard deviation of
    explanatory variables if \code{scaled=TRUE},  \code{critmethod} that indicates the method chosen
    for criteria \code{strict},
    \code{rank} the rank of low rank splines if relevant,
    \code{criterion} the chosen criterion,
    \code{smoother} the chosen smoother,
    \code{kernel} the chosen kernel,
    \code{smoothobject} the smoothobject returned by
    \link[mgcv:smoothCon]{mgcv::smoothCon},
    \code{exhaustive} a boolean which indicates if an exhaustive
    search was chosen}
  \item{criteria}{Value
    of the chosen criterion at the given iteration, \code{NA} is
    returned when aggregation of criteria is chosen (see component
    \code{criterion} of list \code{control.par}). If the number of iterations
    \code{iter} is given by the user, \code{NULL} is returned}
  \item{alliter}{Numeric vector giving all the optimal number of iterations
    selected by  the chosen criteria.}
  \item{allcriteria}{either a list containing all the criteria evaluated on the
    grid \code{Kmin:Kmax} (along with the effective degree of freedom of the
    smoother and the sigma squared on this grid) if an exhaustive search is chosen (see the
    value of function 
    \code{\link{iterchoiceAe}} or \code{\link{iterchoiceS1e}}) 
     or all the values
    of criteria at the given optimal iteration if a non exhaustive
    search is chosen (see also \code{exhaustive} component of list
    \code{control.par}).}
}

\author{Pierre-Andre Cornillon, Nicolas Hengartner  and Eric Matzner-Lober.}

\references{
  Cornillon, P.-A.; Hengartner, N.; Jegou, N. and Matzner-Lober, E. (2012)
  Iterative bias reduction: a comparative study.
\emph{Statistics and Computing}, \emph{23}, 777-791.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2013)
  Recursive bias estimation for multivariate regression smoothers Recursive 
bias estimation for multivariate regression smoothers.
\emph{ESAIM: Probability and Statistics}, \emph{18}, 483-502.

  Cornillon, P.-A.; Hengartner, N. and Matzner-Lober, E. (2017)
  Iterative Bias Reduction Multivariate Smoothing in R: The ibr Package.
\emph{Journal of Statistical Software}, \emph{77}, 1--26.

  Wood, S.N. (2003) Thin plate regression
  splines. \emph{J. R. Statist. Soc. B}, \emph{65}, 95-114.
}

\seealso{\code{\link{ibr}}, \code{\link{predict.ibr}}, \code{\link{summary.ibr}}, \link[mgcv:gam]{mgcv::gam}}


\keyword{smooth}
\keyword{multivariate}

\name{AIC.ibr}

\alias{AIC.ibr}

\title{Summarizing iterative bias reduction fits}

\description{
  Generic function calculating the Akaike information criterion for
  one model objects of ibr class for which a log-likelihood value
  can be obtained, according to the formula
  \eqn{-2 \log(sigma^2) + k df/n}{-2 log(sigma^2) + k df/n},
  where \eqn{df}{df} represents the effective degree of freedom (trace) of the
  smoother in the
  fitted model, and \eqn{k = 2} for the usual AIC, or \eqn{k = \log(n)}
  (\eqn{n} the number of observations) for the so-called BIC or SBC
  (Schwarz's Bayesian criterion).
}

\usage{
\method{AIC}{ibr}(object, \dots, k = 2)}

\arguments{
  \item{object}{A fitted model object of class ibr.}
  \item{\dots}{Not used.}
  \item{k}{Numeric, the \emph{penalty} per parameter to be used; the
    default \code{k = 2} is the classical AIC.}
}
\details{
  The ibr method for \code{AIC}, \code{AIC.ibr()} calculates
  \eqn{\log(sigma^2)+2*df/n}{log(sigma^2)+2*df/n}, where \emph{df} is the trace
  of the smoother.
}
\value{
  returns a numeric value
  with the corresponding AIC (or BIC, or \dots, depending on \code{k}).
}
\author{Pierre-Andre Cornillon, Nicolas Hengartner and Eric Matzner-Lober.}

\references{
Hurvich, C. M., Simonoff J. S. and Tsai, C. L. (1998) Smoothing
Parameter Selection in Nonparametric Regression Using an Improved Akaike
Information Criterion. \emph{Journal of the Royal Statistical Society, Series B}, 60, 271-293 . 
}
\seealso{\code{\link{ibr}}, \code{\link{summary.ibr}}}

\examples{
\dontrun{data(ozone, package = "ibr")
res.ibr <- ibr(ozone[,-1],ozone[,1],df=1.2)
summary(res.ibr)
predict(res.ibr)}
}

\keyword{smooth}
\keyword{multivariate}

\name{print.summary.npregress}

\alias{print.summary.npregress}

\title{Printing iterative bias reduction summaries}

\description{
  \code{print} method for class  \dQuote{\code{summary.npregress}}.
}

\usage{
\method{print}{summary.npregress}(x,digits =
max(3, getOption("digits") - 3), \dots)}

\arguments{
  \item{x}{Object of class \code{\link{npregress}}.}
  \item{digits}{Rounds the values in its first argument to the specified
     number of significant digits.}
  \item{\dots}{Further arguments passed to or from other methods.}
}

\value{
The function \code{print.summary.npregress} prints a list of summary
     statistics of the fitted  iterative bias reduction model given in \code{x}.}

\author{Pierre-Andre Cornillon, Nicolas Hengartner and Eric Matzner-Lober.}

\references{
Wand, M. P. and Jones, M. C. (1995). \emph{Kernel Smoothing}. Chapman and Hall, London.}

\examples{
f <- function(x){sin(5*pi*x)}
n <- 100
x <- runif(n)
z <- f(x)
sigma2 <- 0.05*var(z)
erreur <- rnorm(n,0,sqrt(sigma2))
y <- z+erreur
res <- npregress(x,y,bandwidth=0.02)
summary(res)
}

\keyword{smooth}
\keyword{multivariate}
